
#include "ActionsComponent.h"
#include "SpectrumApplication.h"

ActionsComponent::ActionsComponent (Spectrum *aspectrum, SpectrumApplication *anapplication)
    : Component (T("Actions")),
      groupMemory (0),
      groupSnapshots (0),
      textButtonLoad (0),
      textButtonSave (0),
      textEditorAddress (0),
      labelAddress (0),
      labelValue (0),
      textEditorValue (0),
      textButtonModify (0),
      groupRom (0),
      textButtonRomLoad (0),
      textButtonRomSave (0),
      groupCpu (0),
      textButtonReset (0),
      groupScreenshot (0),
      textButtonScreenShot (0),
	  application(anapplication),
	  spectrum(aspectrum)
{
    addAndMakeVisible (groupMemory = new GroupComponent (T("groupMemory"),
                                                         T("Memory")));
    addAndMakeVisible (groupSnapshots = new GroupComponent (T("groupSnapshots"),
                                                            T("Snapshots")));
    addAndMakeVisible (textButtonLoad = new TextButton (T("textButtonLoad")));
    textButtonLoad->setTooltip (T("Load a snapshot"));
    textButtonLoad->setButtonText (T("Load ..."));
    textButtonLoad->addButtonListener (this);

    addAndMakeVisible (textButtonSave = new TextButton (T("textButtonSave")));
    textButtonSave->setTooltip (T("Save a snapshot"));
    textButtonSave->setButtonText (T("Save ..."));
    textButtonSave->addButtonListener (this);

    addAndMakeVisible (textEditorAddress = new TextEditor (T("textEditorAddress")));
    textEditorAddress->setMultiLine (false);
    textEditorAddress->setReturnKeyStartsNewLine (false);
    textEditorAddress->setReadOnly (false);
    textEditorAddress->setScrollbarsShown (false);
    textEditorAddress->setCaretVisible (true);
    textEditorAddress->setPopupMenuEnabled (false);
    textEditorAddress->setText (T("16384"));

    addAndMakeVisible (labelAddress = new Label (T("labelAddress"),
                                                 T("Address")));
    labelAddress->setFont (Font (15.0000f, Font::plain));
    labelAddress->setJustificationType (Justification::centredLeft);
    labelAddress->setEditable (false, false, false);
    labelAddress->setColour (Label::backgroundColourId, Colour (0x0));
    labelAddress->setColour (Label::textColourId, Colours::black);
    labelAddress->setColour (Label::outlineColourId, Colour (0x0));
    labelAddress->setColour (TextEditor::textColourId, Colours::black);
    labelAddress->setColour (TextEditor::backgroundColourId, Colour (0x0));

    addAndMakeVisible (labelValue = new Label (T("labelValue"),
                                               T("Value")));
    labelValue->setFont (Font (15.0000f, Font::plain));
    labelValue->setJustificationType (Justification::centredLeft);
    labelValue->setEditable (false, false, false);
    labelValue->setColour (Label::backgroundColourId, Colour (0x0));
    labelValue->setColour (Label::textColourId, Colours::black);
    labelValue->setColour (Label::outlineColourId, Colour (0x0));
    labelValue->setColour (TextEditor::textColourId, Colours::black);
    labelValue->setColour (TextEditor::backgroundColourId, Colour (0x0));

    addAndMakeVisible (textEditorValue = new TextEditor (T("textEditorValue")));
    textEditorValue->setMultiLine (false);
    textEditorValue->setReturnKeyStartsNewLine (false);
    textEditorValue->setReadOnly (false);
    textEditorValue->setScrollbarsShown (false);
    textEditorValue->setCaretVisible (true);
    textEditorValue->setPopupMenuEnabled (false);
    textEditorValue->setText (T("0"));

    addAndMakeVisible (textButtonModify = new TextButton (T("textButtonModify")));
    textButtonModify->setTooltip (T("Change a value in memory"));
    textButtonModify->setButtonText (T("Modify"));
    textButtonModify->addButtonListener (this);

    addAndMakeVisible (groupRom = new GroupComponent (T("groupRom"),
                                                      T("Rom")));
    addAndMakeVisible (textButtonRomLoad = new TextButton (T("textButtonRomLoad")));
    textButtonRomLoad->setTooltip (T("Load a Rom file"));
    textButtonRomLoad->setButtonText (T("Load ..."));
    textButtonRomLoad->addButtonListener (this);

    addAndMakeVisible (textButtonRomSave = new TextButton (T("textButtonRomSave")));
    textButtonRomSave->setTooltip (T("Save Rom content to file"));
    textButtonRomSave->setButtonText (T("Save ..."));
    textButtonRomSave->addButtonListener (this);

    addAndMakeVisible (groupCpu = new GroupComponent (T("groupCpu"),
                                                      T("Cpu")));
    addAndMakeVisible (textButtonReset = new TextButton (T("textButtonReset")));
    textButtonReset->setTooltip (T("Reset the CPU"));
    textButtonReset->setButtonText (T("Reset"));
    textButtonReset->addButtonListener (this);

    addAndMakeVisible (groupScreenshot = new GroupComponent (T("groupScreenshot"),
                                                             T("ScreenShot")));
    addAndMakeVisible (textButtonScreenShot = new TextButton (T("textButtonScreenShot")));
    textButtonScreenShot->setTooltip (T("Take a JPEG screenshot"));
    textButtonScreenShot->setButtonText (T("Take"));
    textButtonScreenShot->addButtonListener (this);

    setSize (SCREEN_W, SCREEN_H);
}

ActionsComponent::~ActionsComponent() {
    deleteAndZero (groupMemory);
    deleteAndZero (groupSnapshots);
    deleteAndZero (textButtonLoad);
    deleteAndZero (textButtonSave);
    deleteAndZero (textEditorAddress);
    deleteAndZero (labelAddress);
    deleteAndZero (labelValue);
    deleteAndZero (textEditorValue);
    deleteAndZero (textButtonModify);
    deleteAndZero (groupRom);
    deleteAndZero (textButtonRomLoad);
    deleteAndZero (textButtonRomSave);
    deleteAndZero (groupCpu);
    deleteAndZero (textButtonReset);
    deleteAndZero (groupScreenshot);
    deleteAndZero (textButtonScreenShot);
}

void ActionsComponent::paint (Graphics& g) {
    g.fillAll (Colours::white);
}

void ActionsComponent::resized() {
    groupMemory->setBounds (8, 80, 136, 112);
    groupSnapshots->setBounds (8, 0, 112, 80);
    textButtonLoad->setBounds (16, 16, 96, 24);
    textButtonSave->setBounds (16, 48, 96, 24);
    textEditorAddress->setBounds (80, 96, 56, 24);
    labelAddress->setBounds (16, 96, 55, 24);
    labelValue->setBounds (16, 128, 56, 24);
    textEditorValue->setBounds (80, 128, 56, 24);
    textButtonModify->setBounds (16, 160, 120, 24);
    groupRom->setBounds (136, 0, 112, 80);
    textButtonRomLoad->setBounds (144, 16, 96, 24);
    textButtonRomSave->setBounds (144, 48, 96, 24);
    groupCpu->setBounds (144, 80, 104, 48);
    textButtonReset->setBounds (152, 96, 88, 24);
    groupScreenshot->setBounds (144, 144, 104, 47);
    textButtonScreenShot->setBounds (152, 160, 88, 24);
}

void ActionsComponent::buttonClicked (Button* buttonThatWasClicked) {
    if (buttonThatWasClicked == textButtonLoad) { // LOAD SNA
		FileChooser myChooser ("Please select the SNA you want to load...",
								File (SystemStats::getUserHomeDirectory()),
                               "*.sna");

        if (myChooser.browseForFileToOpen()) {
            File mooseFile (myChooser.getResult());
			application->setPause (0); // STOP
			if (spectrum->LoadSNAFile (mooseFile.getFullPathName())) {
							AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("LoadSNAFile"),
				                           T("Unable to load the SNA file"),
					                       T("ok"));
			}
			application->setPause (1); // RUN
        }
    }
    else if (buttonThatWasClicked == textButtonSave) { // SAVE SNA
		FileChooser myChooser ("Please select the SNA you want to save ...",
                               File (SystemStats::getUserHomeDirectory()),
                               "*.sna");

        if (myChooser.browseForFileToSave(1)) {
            File mooseFile (myChooser.getResult());
			application->setPause (0); // STOP
			if (spectrum->SaveSNAFile (mooseFile.getFullPathName())) {
											AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("SaveSNAFile"),
				                           T("Unable to save the SNA file"),
					                       T("ok"));
			}
			application->setPause (1); // RUN
        }
    }
    else if (buttonThatWasClicked == textButtonModify) { // POKE MEMORY
		String a = textEditorAddress->getText();
		String v = textEditorValue->getText();
		int ai = a.getIntValue();
		int vi = v.getIntValue();
		if (spectrum->PokeMemory ((word) ai, (byte) vi)) {
				AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("PokeMemory"),
				                           T("Unable to modify the memory"),
					                       T("ok"));
		}
		else {
			AlertWindow::showMessageBox (AlertWindow::InfoIcon,
			                               T("Memory"),
				                           T("The value has been successfully modified"),
					                       T("ok"));
		}
    }
    else if (buttonThatWasClicked == textButtonRomLoad) { // LOAD ROM
		FileChooser myChooser ("Please select the ROM you want to load...",
                               File (SystemStats::getUserHomeDirectory()),
                               "*.rom");
		if (myChooser.browseForFileToOpen()) {
            File mooseFile (myChooser.getResult());
			application->setPause (0); // STOP
			if (spectrum->LoadRom (mooseFile.getFullPathName())) {
							AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("LoadRom"),
				                           T("Unable to load the ROM file"),
					                       T("ok"));
			}
			application->setPause (1); // RUN
        }
    }
    else if (buttonThatWasClicked == textButtonRomSave) { // SAVE ROM
		FileChooser myChooser ("Please select the ROM you want to save ...",
                               File (SystemStats::getUserHomeDirectory()),
                               "*.rom");

        if (myChooser.browseForFileToSave(1)) {
            File mooseFile (myChooser.getResult());
			application->setPause (0); // STOP
			if (spectrum->SaveRom (mooseFile.getFullPathName())) {
											AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("SaveRom"),
				                           T("Unable to save the ROM file"),
					                       T("ok"));
			}
			application->setPause (1); // STOP
        }
    }
    else if (buttonThatWasClicked == textButtonReset) { // RESET
		application->setPause (0); // STOP
		spectrum->Reset();
		application->setPause (1); // RUN
    }
    else if (buttonThatWasClicked == textButtonScreenShot) { // TAKE SCREENSHOT
 		File aFile (SystemStats::getUserHomeDirectory());
		Time t = Time::getCurrentTime();
		char buf[256];
		sprintf (buf, "/screenshot-%02d%02d%02d.jpg", t.getHours(), t.getMinutes(), t.getSeconds());
		String s;
		s = aFile.getFullPathName() + String (buf);
		FileOutputStream fos (s);
		JPEGImageFormat jpg;
		int res = jpg.writeImageToStream(*(spectrum->GetULA()->GetImage()), fos);
		fos.flush();

		if (res==0) {
				AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               T("Screenshot"),
				                           T("Unable to make a screenshot"),
					                       T("ok"));
		}
		else {
				AlertWindow::showMessageBox (AlertWindow::InfoIcon,
			                               T("Screenshot"),
				                           T("A screenshot has been successfully generated"),
					                       T("ok"));
		}
    }
}
