#ifndef AUDIO_COMPONENT_H
#define AUDIO_COMPONENT_H

#include "juce.h"
#include "Spectrum.h"

#if 0
class SynthAudioSource  : public AudioSource
{
public:
    Synthesiser synth;

    SynthAudioSource() { 
		 synth.addVoice (new SineWaveVoice()); 
		 synth.addVoice (new SineWaveVoice()); 
	}

    void prepareToPlay (int samplesPerBlockExpected, double sampleRate) {
        synth.setCurrentPlaybackSampleRate (sampleRate);
    }

    void releaseResources() { }

    void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) {
        bufferToFill.clearActiveBufferRegion();
        MidiBuffer incomingMidi;
        synth.renderNextBlock (*bufferToFill.buffer, incomingMidi, 0, bufferToFill.numSamples);
    }
};
#endif



class SynthAudioSource  : public AudioSource {
public:
	SynthAudioSource()
	{
		spectrum = 0; // ???
	}
	void setSpectrum (Spectrum *aspectrum) { spectrum = aspectrum; }
	void prepareToPlay (int samplesPerBlockExpected, double sampleRate) { }
    void releaseResources() { }

    void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) {
        bufferToFill.clearActiveBufferRegion();
		spectrum->GetULA()->prepareAudioSampleBuffer(*bufferToFill.buffer, 0, bufferToFill.numSamples);
    }
private:
	Spectrum *spectrum;
};

class AudioObject  :  public AudioIODeviceCallback {
private:
    AudioDeviceManager audioDeviceManager;
    AudioSourcePlayer audioSourcePlayer;
    SynthAudioSource synthSource;
    MixerAudioSource mixerSource;

public:  
	AudioObject(Spectrum *aspectrum) 
	{
		synthSource.setSpectrum(aspectrum);
        const String error (audioDeviceManager.initialise (1, /* number of input channels */
                                                           2, /* number of output channels */
                                                           0  /* no XML settings.. */));
        if (error.isNotEmpty()) {
            AlertWindow::showMessageBox (AlertWindow::WarningIcon,
                                         T("Audio Object"),
                                         T("Couldn't open an output device!\n\n") + error);
        }
		else {
            mixerSource.addInputSource (&synthSource, false);
            audioSourcePlayer.setSource (&mixerSource);
            //audioDeviceManager.setAudioCallback (this); 
        }
    }

    ~AudioObject() {
        audioDeviceManager.setAudioCallback (0);
        audioSourcePlayer.setSource (0);
    }

    void audioDeviceIOCallback (const float** inputChannelData,
                                int totalNumInputChannels,
                                float** outputChannelData,
                                int totalNumOutputChannels,
                                int numSamples)
    { 
        audioSourcePlayer.audioDeviceIOCallback (inputChannelData, totalNumInputChannels, outputChannelData, totalNumOutputChannels, numSamples);
    }

    void audioDeviceAboutToStart (double sampleRate, int numSamplesPerBlock)
    {
        audioSourcePlayer.audioDeviceAboutToStart (sampleRate, numSamplesPerBlock);
    }

    void audioDeviceStopped() { audioSourcePlayer.audioDeviceStopped(); }

	AudioDeviceManager & getAudioDeviceManager() { return audioDeviceManager; }
	void StartSound () { 
		audioDeviceManager.setAudioCallback (this); 
	}
	void StopSound () { 
		audioDeviceManager.setAudioCallback (0); 
	}
};
#endif