#include "ParametersComponent.h"
#include "AudioObjects.h"
#include "Spectrum.h"
#include "SpectrumApplication.h"

ParametersComponent::ParametersComponent (Spectrum *aspectrum, SpectrumApplication *anapplication)
    : Component (T("Parameters")),
      groupArrows (0),
      groupSound (0),
      toggleSound (0),
      toggleMapping (0),
      toggleBasic (0),
      toggleSinclairJ1 (0),
      toggleSinclairJ2 (0),
      toggleKempston (0),
      toggleCursor (0),
	  sliderVolume(0),
	  spectrum(aspectrum),
	  application (anapplication)
{
#ifdef DEBUG
	filelogger = new FileLogger (File ("C:/Temp/parameters.txt"), String ("Parameters")); 
#endif
    addAndMakeVisible (groupArrows = new GroupComponent (T("groupArrows"),
                                                         T("Keys Mapping")));
    addAndMakeVisible (groupSound = new GroupComponent (T("groupSound"),
                                                        T("Sound")));
    addAndMakeVisible (toggleSound = new ToggleButton (T("toggleSound")));
    toggleSound->setButtonText (T("On/Off"));
	toggleSound->setTooltip (T("Enable or disable sound"));
    toggleSound->addButtonListener (this);
    toggleSound->setToggleState (true, false);

    addAndMakeVisible (toggleMapping = new ToggleButton (T("toggleMapping")));
    toggleMapping->setButtonText (T("No mapping"));
	toggleMapping->setTooltip (T("No arrow mapping"));
    toggleMapping->addButtonListener (this);
    toggleMapping->setToggleState (true, false);

    addAndMakeVisible (toggleBasic = new ToggleButton (T("toggleBasic")));
    toggleBasic->setTooltip (T("Arrows + backspace keys mapped to BASIC editor"));
    toggleBasic->setButtonText (T("Basic"));
    toggleBasic->addButtonListener (this);

    addAndMakeVisible (toggleSinclairJ1 = new ToggleButton (T("toggleSinclairJ1")));
    toggleSinclairJ1->setTooltip (T("Arrows + CTRL keys mapped to Sinclair 2 Joystick 1 interface"));
    toggleSinclairJ1->setButtonText (T("Sinclair J1"));
    toggleSinclairJ1->addButtonListener (this);

    addAndMakeVisible (toggleSinclairJ2 = new ToggleButton (T("toggleSinclairJ2")));
    toggleSinclairJ2->setTooltip (T("Arrows + CTRL keys mapped to Sinclair 2 Joystick 2 interface"));
    toggleSinclairJ2->setButtonText (T("Sinclair J2"));
    toggleSinclairJ2->addButtonListener (this);

    addAndMakeVisible (toggleKempston = new ToggleButton (T("toggleKempston")));
    toggleKempston->setTooltip (T("Arrows + CTRL keys mapped to Kempston joystick interface"));
    toggleKempston->setButtonText (T("Kempston"));
    toggleKempston->addButtonListener (this);

    addAndMakeVisible (toggleCursor = new ToggleButton (T("toggleCursor")));
    toggleCursor->setTooltip (T("Arrows + CTRL keys mapped to Cursor joystick interface"));
    toggleCursor->setButtonText (T("Cursor"));
    toggleCursor->addButtonListener (this);

	addAndMakeVisible (sliderVolume = new Slider (T("sliderVolume")));
	sliderVolume->setSliderStyle (Slider::Rotary);
    sliderVolume->setRotaryParameters (float_Pi * 1.2f, float_Pi * 2.8f, true);
	sliderVolume->setRange (0.0f, 100.0f, 10.0f);
    sliderVolume->setTextBoxStyle (Slider::TextBoxRight, true, 80, 120);
    sliderVolume->addListener (this);	
	sliderVolume->setValue(sliderVolume->getMaximum());
	sliderVolume->setTextValueSuffix("% Max Volume"); 
	addAndMakeVisible (labelShft = new Label (T("labelShft"),
                                                 T("CAPS SHIFT is mapped to SHIFT key")));
    labelShft->setFont (Font (15.0000f, Font::plain));
    labelShft->setJustificationType (Justification::centredLeft);
    labelShft->setEditable (false, false, false);
    labelShft->setColour (Label::backgroundColourId, Colour (0x0));
    labelShft->setColour (Label::textColourId, Colours::black);
    labelShft->setColour (Label::outlineColourId, Colour (0x0));
    labelShft->setColour (TextEditor::textColourId, Colours::black);
    labelShft->setColour (TextEditor::backgroundColourId, Colour (0x0));

	addAndMakeVisible (labelAlt = new Label (T("labelAlt"),
                                                 T("SYMBOL SHIFT is mapped to ALT key")));
    labelAlt->setFont (Font (15.0000f, Font::plain));
    labelAlt->setJustificationType (Justification::centredLeft);
    labelAlt->setEditable (false, false, false);
    labelAlt->setColour (Label::backgroundColourId, Colour (0x0));
    labelAlt->setColour (Label::textColourId, Colours::black);
    labelAlt->setColour (Label::outlineColourId, Colour (0x0));
    labelAlt->setColour (TextEditor::textColourId, Colours::black);
    labelAlt->setColour (TextEditor::backgroundColourId, Colour (0x0));


    setSize (SCREEN_W, SCREEN_H);

	toggleMapping->setRadioGroupId(123);
    toggleBasic->setRadioGroupId(123);
    toggleSinclairJ1->setRadioGroupId(123);
    toggleSinclairJ2->setRadioGroupId(123);
    toggleKempston->setRadioGroupId(123);
    toggleCursor->setRadioGroupId(123);
}

ParametersComponent::~ParametersComponent() {
    deleteAndZero (groupArrows);
    deleteAndZero (groupSound);
    deleteAndZero (toggleSound);
    deleteAndZero (toggleMapping);
    deleteAndZero (toggleBasic);
    deleteAndZero (toggleSinclairJ1);
    deleteAndZero (toggleSinclairJ2);
    deleteAndZero (toggleKempston);
    deleteAndZero (toggleCursor);
    deleteAndZero (labelShft);
	deleteAndZero (labelAlt);
	deleteAndZero (sliderVolume);
#ifdef DEBUG
	delete filelogger;
#endif
}

void ParametersComponent::sliderValueChanged(Slider *slider) {
	if (slider == sliderVolume) {
		double g = sliderVolume->getValue();
		spectrum->GetULA()->setGain ((float) g / 10000.0f); // Why do we have to divide ???
	}
}

void ParametersComponent::paint (Graphics& g) {
    g.fillAll (Colours::white);
}

void ParametersComponent::resized() {
    groupArrows->setBounds (8, 0, 240, 128);
    groupSound->setBounds (8, 128, 240, 56);
    toggleMapping->setBounds (24, 16, 104, 32);
    toggleBasic->setBounds (144, 16, 64, 32);
    toggleSinclairJ1->setBounds (24, 48, 104, 32);
    toggleSinclairJ2->setBounds (24, 80, 104, 32);
    toggleKempston->setBounds (144, 48, 96, 32);
    toggleCursor->setBounds (144, 80, 72, 32);
	labelAlt->setBounds (16, 192,  200, 24);
	labelShft->setBounds (16, 208, 200, 24);
	toggleSound->setBounds (24, 144, 80, 32);
	sliderVolume->setBounds (110, 144, 120, 34);
}

void ParametersComponent::buttonClicked (Button* buttonThatWasClicked) {
    if (buttonThatWasClicked == toggleSound) {
		AudioObject *audioobject = application->getAudioObject();
		if (toggleSound->getToggleState()) audioobject->StartSound();
		else {
			audioobject->StopSound();
		}
	}
    else if (buttonThatWasClicked == toggleMapping) {
		spectrum->ChangeJoystick (NO_MAPPING);
    }
    else if (buttonThatWasClicked == toggleBasic) {
		spectrum->ChangeJoystick (BASIC_MAPPING);
    }
    else if (buttonThatWasClicked == toggleSinclairJ1) {
		spectrum->ChangeJoystick (SINCLAIR_J1_MAPPING);
    }
    else if (buttonThatWasClicked == toggleSinclairJ2) {
		spectrum->ChangeJoystick (SINCLAIR_J2_MAPPING);
    }
    else if (buttonThatWasClicked == toggleKempston) {
		spectrum->ChangeJoystick (KEMPSTON_MAPPING);
    }
    else if (buttonThatWasClicked == toggleCursor) {
		spectrum->ChangeJoystick (CURSOR_MAPPING);
    }
}

