// File : Spectrum.cc
#include <stdio.h>
#include "Spectrum.h"
#include "Z80.h"

word Keys[]={
 KEY_SPACE,
 KEY_ENTER,
 KEY_Z,
 KEY_X,
 KEY_C,
 KEY_V,
 KEY_A,
 KEY_S,
 KEY_D,
 KEY_F,
 KEY_G,
 KEY_Q,
 KEY_W,
 KEY_E,
 KEY_R,
 KEY_T,
 KEY_P,
 KEY_O,
 KEY_I,
 KEY_U,
 KEY_Y,
 KEY_L,
 KEY_K,
 KEY_J,
 KEY_H,
 KEY_M,
 KEY_N,
 KEY_B,
 KEY_1,
 KEY_1_PAD,
 KEY_2,
 KEY_2_PAD,
 KEY_3,
 KEY_3_PAD,
 KEY_4,
 KEY_4_PAD,
 KEY_5,
 KEY_5_PAD,
 KEY_0,
 KEY_0_PAD,
 KEY_9,
 KEY_9_PAD,
 KEY_8,
 KEY_8_PAD,
 KEY_7,
 KEY_7_PAD,
 KEY_6,
 KEY_6_PAD,
 0 // The end
};

// Samples from ZX 
void Samples::AddSample (int aSampleIndex, byte aSampleValue) {
  // If the Sound is user disabled, the buffer is filled and never cleared by the ULA
  // 300 samples is large enough to store samples in one execution period (20ms) 
  // between 2 refresh
  // After reading, ZXULA::RefreshSound clears the sample buffer (nb_samples = 0)

  if (nb_samples >= 300) {
    //printf ("Samples::AddSample : Buffer is full. Overwrite sample buffer\n");
    nb_samples = 0;
  }

  SampleIndex [nb_samples] = aSampleIndex;
  SampleValue [nb_samples] = aSampleValue;
  nb_samples++;
}

void Samples::GetSample (int index, int &aSampleIndex, byte& aSampleValue) const {
  if (index >=nb_samples) {
    printf ("Samples::GetSample : Invalid index %d\n", index);
    aSampleIndex = 0;
    aSampleValue = 0;
    return;
  }
  
  aSampleIndex = SampleIndex [index];
  aSampleValue = SampleValue [index];
}

//
// MEMORY
//
ZXMemory::ZXMemory (void) {
  memory = new byte [65536];
  for (int i = 0; i < 65536; i++) memory [i] = 0; /// rand ();
}

ZXMemory::~ZXMemory (void) {
  if (memory) {
    delete [] memory;
    memory = 0;
  }
}

int ZXMemory::LoadFromFile (const char* aRomFile) {
  FILE *fp = fopen (aRomFile, "rb");
  if (!fp) {
    printf ( "ZXMemory::LoadFromFile : Unable to open %s\n", aRomFile);
    return 1;
  }
  
  byte data;
  int address = 0;
  while (fread (&data, sizeof (byte), 1, fp) != 0) {
    if (WriteToZXMemory ((word) address, data) == 1) {
      fclose (fp);
      return 1;
    }
    address++;
  }
  
  fclose (fp);
  return 0;
}

int ZXMemory::SaveToFile (const char* aRomFile) {
  FILE *fp = fopen (aRomFile, "wb");
  if (!fp) {
    printf ( "ZXMemory::SaveToFile : Unable to open %s\n", aRomFile);
    return 1;
  }
  
  fwrite (memory, 1, 65536, fp);
  
  fclose (fp);
  return 0;
}

int ZXMemory::WriteToZXMemory (word adr, byte data) {  
  memory [adr] = data;
  return 0;
}

int ZXMemory::ReadFromZXMemory (word adr, byte& data) const {  
  data = memory [adr];
  return 0;
}

//
// ZXULA
//
ZXULA::ZXULA (ZXMemory *aZXMemory, ZXIO *aZXIO) {
  theZXMemory = aZXMemory;
  theZXIO = aZXIO;

  // Not bright colors
  ColorTable [0] = Colour (0,0,0); // Black
  ColorTable [1] = Colour (0,0,225); // Blue
  ColorTable [2] = Colour (225,0,0); // Red
  ColorTable [3] = Colour (225,0,225); // Magenta
  ColorTable [4] = Colour (0,225, 0); // Green
  ColorTable [5] = Colour (0,225, 225);// Cyan
  ColorTable [6] = Colour (225,225, 0); // Yellow
  ColorTable [7] = Colour (225,225,225); // White

  // Bright colors
  ColorTable [8] = Colour (0,0,0); // Black
  ColorTable [9] = Colour (0,0,255); // Blue
  ColorTable [10] = Colour (255,0,0); // Red
  ColorTable [11] = Colour (255,0,255); // Magenta
  ColorTable [12] = Colour (0,255, 0); // Green
  ColorTable [13] = Colour (0,255, 255);// Cyan
  ColorTable [14] = Colour (255,255, 0); // Yellow
  ColorTable [15] = Colour (255,255,255); // White

  off_x = (FULL_SCREEN_W - SCREEN_W) / 2;
  off_y = (FULL_SCREEN_H - SCREEN_H) / 2;
  flash = 0;

  joystick_mode = 0; // No remapping for arrows

  //last_sample_value = 0.0f; // Last sound sample value
  ClearAudioBuffer ();
  setGain (100.0f);

  image = new Image (Image::RGB, FULL_SCREEN_W, FULL_SCREEN_H, 1); // Back buffer
}

void ZXULA::ClearAudioBuffer () {	
	for (int i=0; i < 256; i++) AudioBuffer[i] = 0.0f;
}

ZXULA::~ZXULA (void) {
	if (image) {
		delete image;
		image = 0;
	}
}

void ZXULA::ChangeJoystick (int aMode) {
	switch (aMode) {
		case NO_MAPPING: 
		case BASIC_MAPPING:
		case SINCLAIR_J1_MAPPING:
		case SINCLAIR_J2_MAPPING:
		case CURSOR_MAPPING:
		case KEMPSTON_MAPPING:	
			joystick_mode = aMode;
		break;
		default:
		break;
	}
}

void ZXULA::RefreshScreen () {
  int adr;
  byte data;

  if (!image) return;

  flash++; // used by the Draw routine to flash some caracters every 500ms
  if (flash > 50) flash = 0; 

  // Change border color
  int border_color = theZXIO->GetBorderColor ();
  image->clear (0,0,FULL_SCREEN_W, FULL_SCREEN_H, ColorTable [border_color]);

  // Fill the sheet
  for (int block = 0; block < 3; ++block) {
    for (int subblock = 0; subblock < 8; subblock++) {
      for (int line = 0; line < 8; line++) {
	// y = (line * 8) + subblock + (block * 64);
	int y = (line << 3) + subblock + (block << 6);

	for (int x = 0; x < 32; x++) {
	  //adr = 16384 + x + (line * 32) + (subblock * 256) + (block * 2048); 
	  adr = 16384 + x + (line << 5) + (subblock << 8) + (block << 11); 
	  theZXMemory->ReadFromZXMemory ((word) adr, data);
	  Draw (x, y, data);
	}
      }
    }
  }
}

void ZXULA::Draw (int x, int y, byte data) {
  int theData = data;
  byte paper;
  byte ink;
  byte attr;
  byte f;
  
  // Get color attributs
  int they = ((y << 2) & 0xFFE0) + x;
  //theZXMemory->ReadFromZXMemory (22528 + (they << 5) + x, attr);
  theZXMemory->ReadFromZXMemory ((word) (22528 + they), attr); // Read color attributes

  // Extracts paper, ink and flash bits from the attributes
  paper = (attr >> 3) & 0x07; // D5-D3
  ink = attr & 0x07; // D2-D0
  if (attr & 0x40) { // D6 : Bright bit
    ink+=8; // Select ink color from 8 to 15 instead of 0 to 7
    paper+=8;// Select paper color from 8 to 15 instead of 0 to 7
  }
  
  // If Flash bit is set (D7) we must invert ink and paper every 500ms
  f = attr & 0x80;
  if (f != 0) { // Flash mode {
    if (flash < 25) { // 25*20ms => 500ms
      byte tmp;
      tmp = ink;
      ink = paper;
      paper = tmp;
    }
  }

  for (int i=0; i < 8; i++) {
    theData = (data << i) & 0x80;
    if (theData) { // ink color (foreground)
      Colour &col = ColorTable [ink];
	  image->setPixelAt((x << 3)+i+off_x, y+off_y, col);
    }
    else { // paper color (background)
	  Colour &col = ColorTable [paper];
	  image->setPixelAt((x << 3)+i+off_x, y+off_y, col);
    }
  }
}

void ZXULA::RefreshSound (void) {
	// Get samples stored into the ZXIO during execution
	int nbsample = theZXIO->GetSamples().GetNbSamples();
	int i;

	// Clear internal buffer to avoid glitch 
    ClearAudioBuffer();
  
    // Fill and expand samples into the sound stream
    for (int j = 0; j < nbsample; j++) {
		byte aSampleValue;
		int aSampleIndex;
		theZXIO->GetSamples().GetSample(j, aSampleIndex, aSampleValue);
      // the Sound stream size is 256. 
      // The sample index is between 0 and 50000 (processor cycles)
      // So, index 0 correspond to 0 and sample stamped 50000 (if exists) correspond to 256
      for (i=aSampleIndex/200; i < 256; i++) AudioBuffer[i]= (float) aSampleValue; // Durty !!!
    }

	theZXIO->ModifySamples().EmptySamples ();
}

void ZXULA::prepareAudioSampleBuffer (AudioSampleBuffer &outputsb, int startSample, int numSamples) {
    outputsb.copyFrom (0, 0, AudioBuffer, 256);
	outputsb.copyFrom (1, 0, AudioBuffer, 256);	
	outputsb.applyGain(0, 256, gain);
}

void ZXULA::KeyChanged (word aKeyCode, byte aState) {
  byte portF7FE;
  byte portFBFE;
  byte portFDFE;
  byte portFEFE;
  byte portEFFE;
  byte portDFFE;
  byte portBFFE;
  byte port7FFE;

  // Read all
  theZXIO->ReadFromZXIO (0xF7FE, portF7FE);
  theZXIO->ReadFromZXIO (0xFBFE, portFBFE);
  theZXIO->ReadFromZXIO (0xFDFE, portFDFE);
  theZXIO->ReadFromZXIO (0xFEFE, portFEFE);
  theZXIO->ReadFromZXIO (0xEFFE, portEFFE);
  theZXIO->ReadFromZXIO (0xDFFE, portDFFE);
  theZXIO->ReadFromZXIO (0xBFFE, portBFFE);
  theZXIO->ReadFromZXIO (0x7FFE, port7FFE);
  
  switch (aKeyCode) {
	// Left side
	case KEY_SHFT:
		if (aState) portFEFE &= 0x1E; // Bit 0 // CAPS SHIFT
		else portFEFE |= 0x01; // Bit 0
	break;
	case KEY_Z:
		if (aState)	portFEFE &= 0x1D; // Bit 1
		else portFEFE |= 0x02; // Bit 1
	break;
	case KEY_X:
		if (aState) portFEFE &= 0x1B; // Bit 2
		else portFEFE |= 0x04; // Bit 2
	break;
	case KEY_C:
		if (aState) portFEFE &= 0x17; // Bit 3
		else portFEFE |= 0x08; // Bit 3
	break;
	case KEY_V:
		if (aState) portFEFE &= 0x0F; // Bit 4
		else portFEFE |= 0x10; // Bit 4
	break;
	case KEY_A:
		if (aState) portFDFE &= 0x1E; // Bit 0
		else portFDFE |= 0x01; // Bit 0
	break;
	case KEY_S:
		if (aState) portFDFE &= 0x1D; // Bit 1
		else portFDFE |= 0x02; // Bit 1
	break;
	case KEY_D:
		if (aState) portFDFE &= 0x1B; // Bit 2
		else portFDFE |= 0x04; // Bit 2
	break;
	case KEY_F:
		if (aState) portFDFE &= 0x17; // Bit 3
		else portFDFE |= 0x08; // Bit 3
	break;
	case KEY_G:
		if (aState) portFDFE &= 0x0F; // Bit 4
		else portFDFE |= 0x10; // Bit 4
	break;
	case KEY_Q:
		if (aState) portFBFE &= 0x1E; // Bit 0 
		else portFBFE |= 0x01; // Bit 0
	break;
	case KEY_W:
		if (aState) portFBFE &= 0x1D; // Bit 1
		else portFBFE |= 0x02; // Bit 1
	break;
	case KEY_E:
		if (aState) portFBFE &= 0x1B; // Bit 2
		else portFBFE |= 0x04; // Bit 2
	break;
	case KEY_R:
		if (aState) portFBFE &= 0x17; // Bit 3
		else portFBFE |= 0x08; // Bit 3
	break;
	case KEY_T:
		if (aState) portFBFE &= 0x0F; // Bit 4
		else portFBFE |= 0x10; // Bit 4
	break;
 	case KEY_1:
	case KEY_1_PAD:
		if (aState) portF7FE &= 0x1E; // Bit 0
		else portF7FE |= 0x01; // Bit 0
	break;
 	case KEY_2:
	case KEY_2_PAD:
		if (aState) portF7FE &= 0x1D; // Bit 1
		else portF7FE |= 0x02; // Bit 1
	break;
 	case KEY_3:
	case KEY_3_PAD:
		if (aState) portF7FE &= 0x1B; // Bit 2
		else portF7FE |= 0x04; // Bit 2
	break;
 	case KEY_4:
	case KEY_4_PAD:
		if (aState) portF7FE &= 0x17; // Bit 3
		else portF7FE |= 0x08; // Bit 3
	break;
 	case KEY_5:
	case KEY_5_PAD:
		if (aState) portF7FE &= 0x0F; // Bit 4
		else portF7FE |= 0x10; // Bit 4
	break;

  // Right side
	case KEY_0:
	case KEY_0_PAD:
		if (aState) portEFFE &= 0x1E; // Bit 0
		else portEFFE |= 0x01; // Bit 0
	break;
	case KEY_9:
	case KEY_9_PAD:
		if (aState) portEFFE &= 0x1D; // Bit 1
		else portEFFE |= 0x02; // Bit 1
	break;
	case KEY_8:
	case KEY_8_PAD:
		if (aState) portEFFE &= 0x1B; // Bit 2
		else portEFFE |= 0x04; // Bit 2
	break;
	case KEY_7:
	case KEY_7_PAD:
		if (aState) portEFFE &= 0x17; // Bit 3
		else portEFFE |= 0x08; // Bit 3
	break;
	case KEY_6:
	case KEY_6_PAD:
		if (aState) portEFFE &= 0x0F; // Bit 4
		else portEFFE |= 0x10; // Bit 4
	break;

	case KEY_P:
		if (aState) portDFFE &= 0x1E; // Bit 0
		else portDFFE |= 0x01; // Bit 0
	break;
	case KEY_O:
		if (aState) portDFFE &= 0x1D; // Bit 1
		else portDFFE |= 0x02; // Bit 1
	break;
	case KEY_I:
		if (aState) portDFFE &= 0x1B; // Bit 2
		else portDFFE |= 0x04; // Bit 2
	break;
	case KEY_U:
		if (aState) portDFFE &= 0x17; // Bit 3
		else portDFFE |= 0x08; // Bit 3
	break;
	case KEY_Y:
		if (aState) portDFFE &= 0x0F; // Bit 4
		else portDFFE |= 0x10; // Bit 4
	break;

	case KEY_ENTER:
		if (aState) portBFFE &= 0x1E; // Bit 0
		else portBFFE |= 0x01; // Bit 0
	break;
	case KEY_L:
		if (aState) portBFFE &= 0x1D; // Bit 1
		else portBFFE |= 0x02; // Bit 1
	break;
	case KEY_K:
		if (aState) portBFFE &= 0x1B; // Bit 2
		else portBFFE |= 0x04; // Bit 2
	break;
	case KEY_J:
		if (aState) portBFFE &= 0x17; // Bit 3
		else portBFFE |= 0x08; // Bit 3
	break;
	case KEY_H:
		if (aState) portBFFE &= 0x0F; // Bit 4
		else portBFFE |= 0x10; // Bit 4
	break;

	case KEY_SPACE:
		if (aState) port7FFE &= 0x1E; // Bit 0
		else port7FFE |= 0x01; // Bit 0
	break;
	case KEY_ALT:
		if (aState) port7FFE &= 0x1D; // Bit 1 // SYMBOL SHIFT
		else port7FFE |= 0x02; // Bit 1
	break;
	case KEY_M:
		if (aState) port7FFE &= 0x1B; // Bit 2
		else port7FFE |= 0x04; // Bit 2
	break;
	case KEY_N:
		if (aState) port7FFE &= 0x17; // Bit 3
		else port7FFE |= 0x08; // Bit 3
	break;
	case KEY_B:
		if (aState) port7FFE &= 0x0F; // Bit 4
		else port7FFE |= 0x10; // Bit 4
	break;
	default:
	break;
	}
  // Hack for BackSpace/Up/Down/Left/Right
  // Mapping for the arrows & backspace (basic only)
  // 0 : None
  // 1 : For Basic
  // 2 : Sinclair 2 interface Joystick 1
  // 3 : Sinclair 2 interface Joystick 2
  // 4 : Cursor interface
  // 5 : Kempston interface
  if (joystick_mode == BASIC_MAPPING) {
	  switch (aKeyCode) {
		case KEY_BACKSPACE:
			if(aState) {
			portEFFE &= 0x1E; // KEY_0
			portFEFE &= 0x1E; // KEY_LEFTSHIFT
			}
		break;
		case KEY_LEFT:
			if(aState) {	
			portF7FE &= 0x0F; // KEY_5
			portFEFE &= 0x1E; // KEY_LEFTSHIFT
			}
		break;
		case KEY_RIGHT:
			if(aState) {
			portEFFE &= 0x1B; // KEY_8
			portFEFE &= 0x1E; // KEY_LEFTSHIFT
			}
		break;
		case KEY_UP:
			if(aState) {
			portEFFE &= 0x17; // KEY_7
			portFEFE &= 0x1E; // KEY_LEFTSHIFT
			}
		break;
		case KEY_DOWN:
			if(aState) {
			portEFFE &= 0x0F; // KEY_6
			portFEFE &= 0x1E; // KEY_LEFTSHIFT
			}
		break;
		default:
		break;
	  }
  }
  else if (joystick_mode == SINCLAIR_J1_MAPPING) { // Sinclair 2 interface : Joystick 1
	switch (aKeyCode) {
		case KEY_LEFT: if(aState) portEFFE &= 0xEF; // Bit 4
		break;
		case KEY_RIGHT: if(aState) portEFFE &= 0xF7; // Bit 3
		break;
		case KEY_DOWN: if(aState) portEFFE &= 0xFB; // Bit 2
		break;
		case KEY_UP: if(aState) portEFFE &= 0xFD; // Bit 1
		break;
		case KEY_CTRL: if(aState) portEFFE &= 0xFE; // Bit 0
		break;
		default:
		break;
	}
  }
  else if (joystick_mode == SINCLAIR_J2_MAPPING) { // Sinclair 2 interface : Joystick 2
	switch (aKeyCode) {
		case KEY_LEFT: if(aState) portF7FE &= 0xFE; // Bit 0
		break;
		case KEY_RIGHT: if(aState) portF7FE &= 0xFD;// Bit 1
		break;
		case KEY_DOWN: if(aState) portF7FE &= 0xFB; // Bit 2
		break;
		case KEY_UP: if(aState) portF7FE &=  0xF7; // Bit 3
		break;
		case KEY_CTRL: if(aState) portF7FE &=0xEF; // Bit 4
		break;
		default:
		break;
	}
  }
  else if (joystick_mode == CURSOR_MAPPING) { // Cursor interface
	switch (aKeyCode) {
		case KEY_LEFT: if(aState) portF7FE &= 0x0F;
		break;
		case KEY_RIGHT: if(aState) portEFFE &= 0x1B;
		break;
		case KEY_UP: if(aState) portEFFE &= 0x17;
		break;
		case KEY_DOWN: if(aState) portEFFE &= 0x0F;
		break;
		case KEY_CTRL: if(aState) portEFFE &= 0x1E;
		break;
		default:
		break;
	}
  }
  else if (joystick_mode == KEMPSTON_MAPPING) { // Kempston interface
    byte port001F; // Kempston port
    theZXIO->ReadFromZXIO (0x001F, port001F);
	switch (aKeyCode) {
		case KEY_LEFT: 
			if (aState) port001F |= 0x02; // Bit 1
			else port001F &= 0xFD;
		break;
		case KEY_RIGHT: 
			if (aState) port001F |=0x01; // Bit 0
			else port001F &= 0xFE;
		break;
		case KEY_UP: 
			if (aState) port001F |= 0x08; // Bit 3
			else port001F &= 0xF7;
		break;
		case KEY_DOWN: 
			if (aState) port001F |= 0x04; // Bit 2
			else port001F &= 0xFB;
		break;
		case KEY_CTRL: 
			if (aState) port001F  |= 0x10; // Bit 4
			else port001F &= 0xEF;
		break;
		default:
		break;
	}
     theZXIO->WriteToZXIO (0x001F, port001F);
  }

  // Write all
  theZXIO->WriteToZXIO (0xF7FE, portF7FE);
  theZXIO->WriteToZXIO (0xFBFE, portFBFE);
  theZXIO->WriteToZXIO (0xFDFE, portFDFE);
  theZXIO->WriteToZXIO (0xFEFE, portFEFE);
  theZXIO->WriteToZXIO (0xEFFE, portEFFE);
  theZXIO->WriteToZXIO (0xDFFE, portDFFE);
  theZXIO->WriteToZXIO (0xBFFE, portBFFE);
  theZXIO->WriteToZXIO (0x7FFE, port7FFE);
}

//
// ZXIO
//
ZXIO::ZXIO (void) {
  // A map could be better than an array ...
  io = new byte [65536];
  for (int i = 0; i < 65536; i++) {
    io [i] = 0xFF;
  }

  // Kempston port
  io [0x001F] = 0;

  border_color = 0; // Black
  mic = 0;
  speaker = 0;
}

ZXIO::~ZXIO (void) {
  if (io) {
    delete [] io;
    io = 0;
  }
}

int ZXIO::CPUWriteToZXIO (word Port, byte data, int current_cycle) {
#if 0
  switch (Port) {
  case 65278:
  case 65022:
  case 64510:
  case 63486:
  case 61438:
  case 57342:
  case 49150:
  case 32766:
    break;
  default:
    printf ("ZXIO::WriteToZXIO : Port %04X, Data %02X\n", Port, data);
    break;
  }

  printf ("ZXIO::CPUWriteToZXIO : Port %04X, Data %02X\n", Port, data);
#endif
  // Manage port 254
  // D2-D1-D0 : border color
  // D3 : MIC
  // D4 : Speaker
  if ((Port & 0xE0FE) == 0x00FE) { // A7-A5 must be stucked at 0 if it is not the keyboard
    border_color = data & 0x07; // D2-D1-D0
    mic = data & 0x08; // D3

    if (data & 0x10) speaker = 0xFF; // D4
    else speaker = 0;
    theSamples.AddSample (50000-current_cycle, speaker); // 50000 cycles executed every 20ms
  }

  io [Port] = data; // usefull ?
  return 0;
}

int ZXIO::WriteToZXIO (word Port, byte data) {
  io [Port] = data;
  return 0;
}

int ZXIO::CPUReadFromZXIO (word Port, byte& data) const {
#if 0
  switch (Port) {
  case 65278: // FEFE
  case 65022: // FDFE
  case 64510: // FBFE
  case 63486: // F7FE
  case 61438: // EFFE
  case 57342: // DFFE
  case 49150: // BFFE
  case 32766: // 7FFE
    break;
  default:
    printf ("ZXIO::CPUReadFromZXIO : Port %04X, Data %02X\n", Port, data);
    break;
  }
#endif

  // hack for Kempston. Sometimes port is 001F (SABRE) and sometimes 021F (PSSST)
  if ((Port & 0x001F) == 0x001F) data = io [0x001F];
  else data = io [Port];
  return 0;
}

int ZXIO::ReadFromZXIO (word Port, byte& data) const {
  data = io [Port];
#if 0
  switch (Port) {
  case 65278: // FEFE
  case 65022: // FDFE
  case 64510: // FBFE
  case 63486: // F7FE
  case 61438: // EFFE
  case 57342: // DFFE
  case 49150: // BFFE
  case 32766: // 7FFE
    break;
  default:
    printf ("ZXIO::ReadFromZXIO : Port %04X, Data %02X\n", Port, data);
    break;
  }
#endif

  return 0;
}

//
// SPECTRUM
//
Spectrum::Spectrum (const char* aRomFile) {
  theZXMemory = new ZXMemory ();
  theZXIO = new ZXIO ();
  theZXULA = new ZXULA (theZXMemory, theZXIO);
  theProcessor = new Z80 (theZXMemory, theZXULA, theZXIO);

  if (LoadRom (aRomFile)) // ROM at @0000
	validity = 0; // Bad ROM
  else
	validity = 1; // ROM OK
}

int Spectrum::LoadRom (const char *aRomFile) {
	int res = theZXMemory->LoadFromFile (aRomFile); // ROM at @0000
	return res; 
}
int Spectrum::SaveRom (const char *aRomFile) const {
	int res = theZXMemory->SaveToFile (aRomFile); // ROM at @0000
	return res; 
}
void Spectrum::ChangeJoystick (int aMode) {
	return theZXULA->ChangeJoystick(aMode);
}

Spectrum::~Spectrum (void) {
  if (theZXMemory) {
    delete theZXMemory;
    theZXMemory = 0;
  }

  if (theZXULA) {
    delete theZXULA;
    theZXULA = 0;
  }

  if (theZXIO) {
    delete theZXIO;
    theZXIO = 0;
  }

  if (theProcessor) {
    delete theProcessor;
    theProcessor = 0;
  }
}

void Spectrum::Reset (void) {
  theProcessor->Reset ();
}

int Spectrum::PokeMemory (word adr, byte data) {
	return theZXMemory->WriteToZXMemory (adr, data);
}

int Spectrum::Run (void) {
	theProcessor->Execute (50000); // 50000 steps every 20ms
	theZXULA->RefreshSound ();
	theProcessor->Z80_Cause_Interrupt (0);
	return 0;
}

/*
 *      Load a 48K  .SNA file.
 *
 *      48K Format as follows:
 *      Offset  Size    Description (all registers stored with LSB first)
 *      0       1       I
 *      1       18      HL',DE',BC',AF',HL,DE,BC,IY,IX
 *      19      1       Interrupt (bit 2 contains IFF2 1=EI/0=DI
 *      20      1       R
 *      21      4       AF,SP
 *      25      1       Interrupt Mode (0=IM0/1=IM1/2=IM2)
 *      26      1       Border Colour (0..7)
 *      27      48K     RAM dump 0x4000-0xffff
 *      PC is stored on stack. */

int Spectrum::LoadSNAFile (const char *aSNAFile) {
  FILE *fp = fopen (aSNAFile, "rb");
  if (!fp) {
    printf ("Spectrum::LoadSNAFile : unable to open SNA file %s\n", aSNAFile);
    return 1;
  }

  Z80_Regs regs;
  byte dummy;

  fread (&regs.I, sizeof (byte), 1, fp); // I
  regs.I &= 0xFF;

  fread (&regs.HL2.B.l, sizeof (byte), 1, fp); // HL'
  fread (&regs.HL2.B.h, sizeof (byte), 1, fp);
  regs.HL2.B.l &= 0xFF;
  regs.HL2.B.h &= 0xFF;

  fread (&regs.DE2.B.l, sizeof (byte), 1, fp); // DE'
  fread (&regs.DE2.B.h, sizeof (byte), 1, fp);
  regs.DE2.B.l &= 0xFF;
  regs.DE2.B.h &= 0xFF;

  fread (&regs.BC2.B.l, sizeof (byte), 1, fp); // BC'
  fread (&regs.BC2.B.h, sizeof (byte), 1, fp);
  regs.BC2.B.l &= 0xFF;
  regs.BC2.B.h &= 0xFF;

  fread (&regs.AF2.B.l, sizeof (byte), 1, fp); // AF'
  fread (&regs.AF2.B.h, sizeof (byte), 1, fp);
  regs.AF2.B.l &= 0xFF;
  regs.AF2.B.h &= 0xFF;

  fread (&regs.HL.B.l, sizeof (byte), 1, fp); // HL
  fread (&regs.HL.B.h, sizeof (byte), 1, fp);
  regs.HL.B.l &= 0xFF;
  regs.HL.B.h &= 0xFF;

  fread (&regs.DE.B.l, sizeof (byte), 1, fp); // DE
  fread (&regs.DE.B.h, sizeof (byte), 1, fp);
  regs.DE.B.l &= 0xFF;
  regs.DE.B.h &= 0xFF;

  fread (&regs.BC.B.l, sizeof (byte), 1, fp); // BC
  fread (&regs.BC.B.h, sizeof (byte), 1, fp);
  regs.BC.B.l &= 0xFF;
  regs.BC.B.h &= 0xFF;

  fread (&regs.IY.B.l, sizeof (byte), 1, fp); // IY
  fread (&regs.IY.B.h, sizeof (byte), 1, fp);
  regs.IY.B.l &= 0xFF;
  regs.IY.B.h &= 0xFF;

  fread (&regs.IX.B.l, sizeof (byte), 1, fp); // IX
  fread (&regs.IX.B.h, sizeof (byte), 1, fp);
  regs.IX.B.l &= 0xFF;
  regs.IX.B.h &= 0xFF;

  fread (&dummy, sizeof (byte), 1, fp); // IFF2
  regs.IFF2 = (dummy & 0x04) >> 2;
  regs.IFF1 = (dummy & 0x04) >> 2;

  //printf ("IFF1 = %d, IFF2 = %d\n",regs.IFF1, regs.IFF2);
  fread (&dummy, sizeof (byte), 1, fp); // R
  regs.R = dummy & 127;
  regs.R2 = dummy & 128;

  fread (&regs.AF.B.l, sizeof (byte), 1, fp); // AF
  fread (&regs.AF.B.h, sizeof (byte), 1, fp);
  regs.AF.B.l &= 0xFF;
  regs.AF.B.h &= 0xFF;

  fread (&regs.SP.B.l, sizeof (byte), 1, fp); // SP
  fread (&regs.SP.B.h, sizeof (byte), 1, fp);
  regs.SP.B.l &= 0xFF;
  regs.SP.B.h &= 0xFF;

  fread (&regs.IM, sizeof (byte), 1, fp); // IM
  regs.IM &= 0xFF;
  //printf ("IM = %d\n",regs.IM);

  fread (&dummy, sizeof (byte), 1, fp); // Border color
  theZXIO->SetBorderColor (dummy & 0x07);

  // Fill the Memory
  int address = 0x4000;
  byte data;
  while (fread (&data, sizeof (byte), 1, fp) != 0) {
    if (theZXMemory->WriteToZXMemory ((word)address, data) == 1) {
      fclose (fp);
      return 1;
    }
    address++;
  }

  /* get pc from stack */
  address = regs.SP.W.l & 0xFFFF; // Get SP
  theZXMemory->ReadFromZXMemory ((word) address, regs.PC.B.l);
  theZXMemory->ReadFromZXMemory ((word) (address+1), regs.PC.B.h);
    
  address += 2;
  regs.SP.W.l = address & 0xFFFF; // New stack pointer

  regs.pending_irq = 0;
  regs.pending_nmi = 0;

  // Set the real Registers
  theProcessor->Z80_SetRegs (regs);
  fclose (fp);

  return 0; 
}


/*
 *      Save a 48K  .SNA file.
 *
 *      48K Format as follows:
 *      Offset  Size    Description (all registers stored with LSB first)
 *      0       1       I
 *      1       18      HL',DE',BC',AF',HL,DE,BC,IY,IX
 *      19      1       Interrupt (bit 2 contains IFF2 1=EI/0=DI
 *      20      1       R
 *      21      4       AF,SP
 *      25      1       Interrupt Mode (0=IM0/1=IM1/2=IM2)
 *      26      1       Border Colour (0..7)
 *      27      48K     RAM dump 0x4000-0xffff
 *      PC is stored on stack. */

int Spectrum::SaveSNAFile (const char *aSNAFile) const {
  FILE *fp = fopen (aSNAFile, "wb");
  if (!fp) {
    printf ("Spectrum::SaveSNAFile : unable to open SNA file %s\n", aSNAFile);
    return 1;
  }

  Z80_Regs regs= theProcessor->Z80_GetRegs ();
  byte dummy;

  /* set pc to stack */
  word adr;
  adr = regs.SP.W.l & 0xFFFF; // Get SP
  theZXMemory->WriteToZXMemory ((word) (adr-2), regs.PC.B.l);
  theZXMemory->WriteToZXMemory ((word) (adr-1), regs.PC.B.h);
  adr -= 2;
  regs.SP.W.l = adr & 0xFFFF; // New stack pointer

  regs.I &= 0xFF;
  fwrite (&regs.I, sizeof (byte), 1, fp); // I

  regs.HL2.B.l &= 0xFF;
  fwrite (&regs.HL2.B.l, sizeof (byte), 1, fp); // HL'
  regs.HL2.B.h &= 0xFF;
  fwrite (&regs.HL2.B.h, sizeof (byte), 1, fp);

  regs.DE2.B.l &= 0xFF;
  regs.DE2.B.h &= 0xFF;
  fwrite (&regs.DE2.B.l, sizeof (byte), 1, fp); // DE'
  fwrite (&regs.DE2.B.h, sizeof (byte), 1, fp);

  regs.BC2.B.l &= 0xFF;
  regs.BC2.B.h &= 0xFF;
  fwrite (&regs.BC2.B.l, sizeof (byte), 1, fp); // BC'
  fwrite (&regs.BC2.B.h, sizeof (byte), 1, fp);

  regs.AF2.B.l &= 0xFF;
  regs.AF2.B.h &= 0xFF;
  fwrite (&regs.AF2.B.l, sizeof (byte), 1, fp); // AF'
  fwrite (&regs.AF2.B.h, sizeof (byte), 1, fp);

  regs.HL.B.l &= 0xFF;
  regs.HL.B.h &= 0xFF;
  fwrite (&regs.HL.B.l, sizeof (byte), 1, fp); // HL
  fwrite (&regs.HL.B.h, sizeof (byte), 1, fp);

  regs.DE.B.l &= 0xFF;
  regs.DE.B.h &= 0xFF;
  fwrite (&regs.DE.B.l, sizeof (byte), 1, fp); // DE
  fwrite (&regs.DE.B.h, sizeof (byte), 1, fp);

  regs.BC.B.l &= 0xFF;
  regs.BC.B.h &= 0xFF;
  fwrite (&regs.BC.B.l, sizeof (byte), 1, fp); // BC
  fwrite (&regs.BC.B.h, sizeof (byte), 1, fp);
  
  regs.IY.B.l &= 0xFF;
  regs.IY.B.h &= 0xFF;
  fwrite (&regs.IY.B.l, sizeof (byte), 1, fp); // IY
  fwrite (&regs.IY.B.h, sizeof (byte), 1, fp);

  regs.IX.B.l &= 0xFF;
  regs.IX.B.h &= 0xFF;
  fwrite (&regs.IX.B.l, sizeof (byte), 1, fp); // IX
  fwrite (&regs.IX.B.h, sizeof (byte), 1, fp);

  dummy = (regs.IFF1 & 0x01) << 2;
  fwrite (&dummy, sizeof (byte), 1, fp); // IFF2

  dummy = (regs.R & 127) | (regs.R2 & 128);
  fwrite (&dummy, sizeof (byte), 1, fp); // R

  regs.AF.B.l &= 0xFF;
  regs.AF.B.h &= 0xFF;
  fwrite (&regs.AF.B.l, sizeof (byte), 1, fp); // AF
  fwrite (&regs.AF.B.h, sizeof (byte), 1, fp);
  
  regs.SP.B.l &= 0xFF;
  regs.SP.B.h &= 0xFF;
  fwrite (&regs.SP.B.l, sizeof (byte), 1, fp); // SP
  fwrite (&regs.SP.B.h, sizeof (byte), 1, fp);
 
  regs.IM &= 0xFF;
  fwrite (&regs.IM, sizeof (byte), 1, fp); // IM
 
  dummy = theZXIO->GetBorderColor ();
  fwrite (&dummy, sizeof (byte), 1, fp); // Border color

  // Dump the Memory
  byte data;
  for (int address = 0x4000; address <= 0xFFFF; address++) {
    theZXMemory->ReadFromZXMemory ((word) address, data);
    fwrite (&data, sizeof (byte), 1, fp);
  }

  fclose (fp);

  return 0; 
}
