// File : Spectrum.h

#ifndef SPECTRUM_H
#define SPECTRUM_H

#include "Z80.h"
#include "juce.h"

#define SCREEN_W 256
#define SCREEN_H 192
#define FULL_SCREEN_W SCREEN_W+32
#define FULL_SCREEN_H SCREEN_H+32

// Modifiers scan codes (dummy codes)
#define KEY_ALT 6
#define KEY_SHFT 7
#define KEY_CTRL 8

// KEYS scan codes
#define KEY_SPACE 32
#define KEY_ENTER 13
#define KEY_BACKSPACE 1
#define KEY_LEFT 2
#define KEY_RIGHT 3
#define KEY_UP 4
#define KEY_DOWN 5
#define KEY_Z 122
#define KEY_X 120
#define KEY_C 99
#define KEY_V 118
#define KEY_A 97
#define KEY_S 115
#define KEY_D 100
#define KEY_F 102
#define KEY_G 103
#define KEY_Q 113
#define KEY_W 119
#define KEY_E 101
#define KEY_R 114
#define KEY_T 116
#define KEY_P 112
#define KEY_O 111
#define KEY_I 105
#define KEY_U 117
#define KEY_Y 121
#define KEY_L 108
#define KEY_K 107
#define KEY_J 106
#define KEY_H 104
#define KEY_M 109
#define KEY_N 110
#define KEY_B 98

#define KEY_1 38
#define KEY_1_PAD 49
#define KEY_2 233
#define KEY_2_PAD 50
#define KEY_3 34
#define KEY_3_PAD 51
#define KEY_4 39
#define KEY_4_PAD 52
#define KEY_5 40
#define KEY_5_PAD 53
#define KEY_0 224
#define KEY_0_PAD 48
#define KEY_9 231
#define KEY_9_PAD 57
#define KEY_8 95
#define KEY_8_PAD 56
#define KEY_7 232
#define KEY_7_PAD 55
#define KEY_6 45
#define KEY_6_PAD 54

class Samples {
 public:
  Samples () { nb_samples = 0; }
  ~Samples () { }
  void AddSample (int aSampleIndex, byte aSampleValue);
  void GetSample (int index, int &aSampleIndex, byte& aSampleValue) const;
  int GetNbSamples (void) const { return nb_samples; }
  void EmptySamples (void) { nb_samples = 0; }

 private:
  int SampleIndex [300];
  byte SampleValue [300];
  int nb_samples;
};

class ZXMemory {
 public:
  ZXMemory (void);
  ~ZXMemory (void);

  int LoadFromFile (const char* aRomFile);
  int SaveToFile (const char* aRomFile);

  int WriteToZXMemory (word adr, byte data);
  int ReadFromZXMemory (word adr, byte& data) const;

 private:
  byte *memory;
};


  // 0 : None
  // 1 : For Basic
  // 2 : Sinclair 2 interface Joystick 1
  // 3 : Sinclair 2 interface Joystick 2
  // 4 : Cursor interface
  // 5 : Kempston interface
#define NO_MAPPING			0
#define BASIC_MAPPING		1
#define SINCLAIR_J1_MAPPING 2
#define SINCLAIR_J2_MAPPING 3
#define CURSOR_MAPPING		4		
#define KEMPSTON_MAPPING	5

class ZXULA {
 public:
  ZXULA (ZXMemory *aZXMemory, ZXIO *aZXIO);
  ~ZXULA (void);
  void RefreshScreen ();
  void KeyChanged (word aKeyCode, byte aState);
  void RefreshSound (void);
  void ChangeJoystick (int aMode);
  Image *GetImage(void) const { return image; }
  void ClearAudioBuffer ();
  void prepareAudioSampleBuffer (AudioSampleBuffer &outputsb, int startSample, int numSamples);
  void setGain (float g) { gain = g; }
  float getGain () const { return gain; }

 private:
  void Draw (int x, int y, byte data);
  Colour ColorTable [16];
  Image *image;
  int off_x;
  int off_y;

  int flash;
  int joystick_mode;

  ZXMemory *theZXMemory;
  ZXIO *theZXIO;

  // Sound
  float AudioBuffer[300];
  float gain;
};

class ZXIO {
 public:
  ZXIO (void);
  ~ZXIO (void);

  // Access to IO without time-stamp
  int WriteToZXIO (word Port, byte data);
  int ReadFromZXIO (word Port, byte& data) const;

  // Access to IO with time-stamp
  int CPUWriteToZXIO (word Port, byte data, int current_cycle);
  int CPUReadFromZXIO (word Port, byte& data) const;

  int GetBorderColor (void) const { return border_color; }
  void SetBorderColor (byte aColor) { border_color = aColor; }

  const Samples &GetSamples (void) const { return theSamples;}
  Samples& ModifySamples (void) { return theSamples;}

 private:
  Samples theSamples; // For sound

  byte *io;
  byte border_color; // Color bit D2-D0
  byte mic; // MIC bit D3
  byte speaker; // Speaker bit D4
};

class Spectrum {
 public:
  Spectrum ();
  Spectrum (const char* aRomFile);
  ~Spectrum (void);
  int LoadSNAFile (const char *aSNAFile);
  int LoadRom (const char *aRomFile);
  int SaveRom (const char *aRomFile) const;
  int SaveSNAFile (const char *aSNAFile) const;
  void Reset (void);
  int PokeMemory (word adr, byte data);
  void ChangeJoystick (int aMode); 
  int Run (void); 
  ZXULA *GetULA () { return theZXULA; }
  int IsValid () const { return validity; }

 private:
  int validity;
  Z80 *theProcessor;
  ZXMemory *theZXMemory;
  ZXULA *theZXULA;
  ZXIO *theZXIO;
};




#endif
