#include "juce.h"

#include "SpectrumApplication.h"
#include "SpectrumWindow.h"
#include "AudioObjects.h"
#include "Spectrum.h"

SpectrumApplication::SpectrumApplication()
        : spectrumWindow (0), spectrum (0), tooltipwindow(0), audioobject(0) {
}

SpectrumApplication::~SpectrumApplication() {
}

void SpectrumApplication::setPause (int p) {
	if (p) { // Need to start
		startTimer (refresh_period);
	}
	else { // Need to stop
		if (isTimerRunning ()) stopTimer();
	}
}

void SpectrumApplication::timerCallback() { 
		if (spectrum) {
			spectrum->Run();
			spectrum->GetULA()->RefreshScreen();
		}
		if (spectrumWindow) spectrumWindow->ForceRepaint();
	}

void SpectrumApplication::initialise (const String& commandLine) {
        LookAndFeel::setDefaultLookAndFeel (&shinyLookAndFeel);
		refresh_period = 1000/50; // 20 ms

		// Creating the virtual machine
		File aFile = File::getCurrentWorkingDirectory();
		String s;
		s = aFile.getFullPathName() + String ("/spectrum.rom");
		spectrum = new Spectrum (s);

		audioobject = new AudioObject(spectrum); // The SpectrumComponent needs to know the application
        spectrumWindow = new SpectrumWindow(this); // The SpectrumComponent needs to know the application
		tooltipwindow = new TooltipWindow ();

		setPause (1); // Starting
		audioobject->StartSound();

		if 	(!spectrum->IsValid ()) {
			AlertWindow::showMessageBox (AlertWindow::WarningIcon,
			                               getApplicationName(),
				                           T("Unable to launch the emulator. Check the rom file"),
					                       T("ok"));
		}
    }

void SpectrumApplication::shutdown() {
	if (spectrumWindow != 0) delete spectrumWindow;
	if (spectrum) delete spectrum;
    if (tooltipwindow) delete tooltipwindow;
	if (audioobject) delete audioobject;

	LookAndFeel::setDefaultLookAndFeel (0);
}

