#include "juce.h"

#include "Spectrum.h"
#include "SpectrumComponent.h"
#include "SpectrumApplication.h"
#include "ScreenComponent.h"
#include "ParametersComponent.h"
#include "ActionsComponent.h"
#include "AudioObjects.h"

SpectrumComponent::SpectrumComponent (SpectrumApplication *anapplication)
    : Component (T("SpectrumComponent")),
      tabbedComponent (0),
      label (0),
	  textButtonKeyboard (0),
	  keyboardWindow (0),
	  application(anapplication)
{
	keyboardWindow = new KeyboardWindow ();

    addAndMakeVisible (tabbedComponent = new TabbedComponent (TabbedButtonBar::TabsAtBottom));
    tabbedComponent->setTabBarDepth (20);	

	Spectrum *spectrum = anapplication->getSpectrum();
    tabbedComponent->addTab (T("Screen"), Colours::lightgrey, screencomponent = new ScreenComponent(spectrum), true);
    tabbedComponent->addTab (T("Parameters"), Colours::lightgrey, parameterscomponent = new ParametersComponent(spectrum, application), true);
    tabbedComponent->addTab (T("Actions"), Colours::lightgrey, actionscomponent = new ActionsComponent(spectrum, application), true);
	tabbedComponent->setCurrentTabIndex (0);

    addAndMakeVisible (label = new Label (T("new label"),
                                          T("ZX Spectrum")));
    label->setFont (Font (15.0000f, Font::bold));
    label->setJustificationType (Justification::centredLeft);
    label->setEditable (false, false, false);
    label->setColour (Label::backgroundColourId, Colour (0x0));
    label->setColour (Label::textColourId, Colours::black);
    label->setColour (Label::outlineColourId, Colour (0x0));
    label->setColour (TextEditor::textColourId, Colours::black);
    label->setColour (TextEditor::backgroundColourId, Colour (0x0));

	addAndMakeVisible (textButtonKeyboard = new TextButton (T("textButtonKeyboard")));
	textButtonKeyboard->setTooltip (T("Show keyboard layout"));
    textButtonKeyboard->setButtonText (T("Keyboard"));
    textButtonKeyboard->addButtonListener (this);

	addAndMakeVisible (toggleButtonPause = new ToggleButton (T("toggleButtonPause")));
	toggleButtonPause->setTooltip (T("Pause the emulator"));
    toggleButtonPause->setButtonText (T("Pause"));
    toggleButtonPause->addButtonListener (this);

    centreWithSize (320,240);
}

void SpectrumComponent::buttonClicked (Button* buttonThatWasClicked) {
    if (buttonThatWasClicked == textButtonKeyboard) {
		if (keyboardWindow) keyboardWindow->setVisible(true);
	}
	else if (buttonThatWasClicked == toggleButtonPause) {
		int p;
		if (toggleButtonPause->getToggleState()) p=0; // Pause
		else p=1; // Run
		application->setPause (p);
	}
}
SpectrumComponent::~SpectrumComponent() {
    deleteAndZero (tabbedComponent);
    deleteAndZero (label);
	deleteAndZero (textButtonKeyboard);
	deleteAndZero (keyboardWindow);
	deleteAndZero (toggleButtonPause);
}

void SpectrumComponent::paint (Graphics& g) {
    g.fillAll (Colours::white);
    g.setColour (Colours::red);
    g.fillPath (internalPath1);
    g.setColour (Colours::yellow);
    g.fillPath (internalPath2);
    g.setColour (Colour (0xff00ff03));
    g.fillPath (internalPath3);
    g.setColour (Colours::aqua);
    g.fillPath (internalPath4);
}

void SpectrumComponent::resized()
{
    tabbedComponent->setBounds (0, 40, proportionOfWidth (1.0000f), proportionOfHeight (0.8457f));
    label->setBounds (64, 8, 150, 24);
    internalPath1.clear();
    internalPath1.startNewSubPath (32.0f, 8.0f);
    internalPath1.lineTo (40.0f, 8.0f);
    internalPath1.lineTo (17.0f, 32.0f);
    internalPath1.lineTo (8.0f, 32.0f);
    internalPath1.closeSubPath();

    internalPath2.clear();
    internalPath2.startNewSubPath (40.0f, 8.0f);
    internalPath2.lineTo (48.0f, 8.0f);
    internalPath2.lineTo (25.0f, 32.0f);
    internalPath2.lineTo (16.0f, 32.0f);
    internalPath2.closeSubPath();

    internalPath3.clear();
    internalPath3.startNewSubPath (48.0f, 8.0f);
    internalPath3.lineTo (56.0f, 8.0f);
    internalPath3.lineTo (33.0f, 32.0f);
    internalPath3.lineTo (24.0f, 32.0f);
    internalPath3.closeSubPath();

    internalPath4.clear();
    internalPath4.startNewSubPath (56.0f, 8.0f);
    internalPath4.lineTo (64.0f, 8.0f);
    internalPath4.lineTo (41.0f, 32.0f);
    internalPath4.lineTo (32.0f, 32.0f);
    internalPath4.closeSubPath();

	textButtonKeyboard->setBounds (224, 8, 60, 24);
	toggleButtonPause->setBounds (160, 8, 60, 24);
}

