// File Z80.cc

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "Spectrum.h"
#include "Z80.h"

// Pointer on member function
// Declaration : void (Z80::*opcode[256]) (void);
// Affectation : opcode [0] = &Z80::fct1;
// Usage       : (this->*opcode[i])();


Z80_Regs::Z80_Regs () {
  AF.D = 0;
  BC.D = 0;
  DE.D = 0;
  HL.D = 0;
  IX.D = 0;
  IY.D = 0;

  AF2.D = 0; 
  BC2.D = 0;
  DE2.D = 0;
  HL2.D = 0;

  PC.D = 0;
  SP.D = 0;

  IFF1 = 0; 
  IFF2 = 0;

  HALT = 0;
  IM = 0; 

  I = 0;
  R = 0; 
  R2 = 0;

  pending_irq = 0;
  pending_nmi = 0;
}

Z80_Regs::~Z80_Regs () {
}

Z80_Regs::Z80_Regs (const Z80_Regs &aZ80_Regs) {
  AF.W.l = aZ80_Regs.AF.W.l;
  BC.W.l = aZ80_Regs.BC.W.l;
  DE.W.l = aZ80_Regs.DE.W.l;
  HL.W.l = aZ80_Regs.HL.W.l;
  IX.W.l = aZ80_Regs.IX.W.l;
  IY.W.l = aZ80_Regs.IY.W.l;

  AF2.W.l = aZ80_Regs.AF2.W.l; 
  BC2.W.l = aZ80_Regs.BC2.W.l;
  DE2.W.l = aZ80_Regs.DE2.W.l;
  HL2.W.l = aZ80_Regs.HL2.W.l;

  PC.W.l = aZ80_Regs.PC.W.l;
  SP.W.l = aZ80_Regs.SP.W.l;

  IFF1 = aZ80_Regs.IFF1; 
  IFF2 = aZ80_Regs.IFF2;

  HALT = aZ80_Regs.HALT;
  IM = aZ80_Regs.IM; 

  I = aZ80_Regs.I;
  R = aZ80_Regs.R; 
  R2 = aZ80_Regs.R2;

  pending_irq = aZ80_Regs.pending_irq;
  pending_nmi = aZ80_Regs.pending_nmi;
}
                                                                                
Z80_Regs& Z80_Regs::operator=(const Z80_Regs &aZ80_Regs) {
  if (this == &aZ80_Regs) return *this;
  AF.W.l = aZ80_Regs.AF.W.l;
  BC.W.l = aZ80_Regs.BC.W.l;
  DE.W.l = aZ80_Regs.DE.W.l;
  HL.W.l = aZ80_Regs.HL.W.l;
  IX.W.l = aZ80_Regs.IX.W.l;
  IY.W.l = aZ80_Regs.IY.W.l;

  AF2.W.l = aZ80_Regs.AF2.W.l; 
  BC2.W.l = aZ80_Regs.BC2.W.l;
  DE2.W.l = aZ80_Regs.DE2.W.l;
  HL2.W.l = aZ80_Regs.HL2.W.l;

  PC.W.l = aZ80_Regs.PC.W.l;
  SP.W.l = aZ80_Regs.SP.W.l;

  IFF1 = aZ80_Regs.IFF1; 
  IFF2 = aZ80_Regs.IFF2;

  HALT = aZ80_Regs.HALT;
  IM = aZ80_Regs.IM; 

  I = aZ80_Regs.I;
  R = aZ80_Regs.R; 
  R2 = aZ80_Regs.R2;

  pending_irq = aZ80_Regs.pending_irq;
  pending_nmi = aZ80_Regs.pending_nmi;

  return *this;
}

void Z80_Regs::Dump (void) {
  printf ("AF:%04X HL:%04X DE:%04X BC:%04X PC:%04X SP:%04X IX:%04X IY:%04X  \n",
	  AF.W.l,HL.W.l,DE.W.l,BC.W.l,PC.W.l,SP.W.l,IX.W.l,IY.W.l);
}

Z80::Z80 (ZXMemory* aZXMemory, ZXULA* aZXULA, ZXIO* aZXIO) {
  theZXMemory = aZXMemory;
  theZXULA = aZXULA;
  theZXIO = aZXIO;
  
  Z80_ICount = 0;

  InitTables ();
}

Z80::~Z80 () {
}

byte Z80::Z80_In(word Port) const{
  byte data;
  theZXIO->CPUReadFromZXIO (Port, data);
  return data;
}

void Z80::Z80_Out(word Port, byte Value) {
  theZXIO->CPUWriteToZXIO (Port, Value, Z80_ICount);
}

byte Z80::Z80_RDMEM (word adr) const {
  byte data;
  theZXMemory->ReadFromZXMemory (adr, data);
  return data;
}

void Z80::Z80_WRMEM(word adr, byte data) {
  theZXMemory->WriteToZXMemory (adr, data);
}

void Z80::call_opcode_dd_cb (byte aCode){
  switch(aCode){
  case 0:rlc_xix_b();break;
  case 1:rlc_xix_c();break;
  case 2:rlc_xix_d();break;
  case 3:rlc_xix_e();break;
  case 4:rlc_xix_h();break;
  case 5:rlc_xix_l();break;
  case 6:rlc_xix();break;
  case 7:rlc_xix_a();break;
  case 8:rrc_xix_b();break;
  case 9:rrc_xix_c();break;
  case 10:rrc_xix_d();break;
  case 11:rrc_xix_e();break;
  case 12:rrc_xix_h();break;
  case 13:rrc_xix_l();break;
  case 14:rrc_xix();break;
  case 15:rrc_xix_a();break;
  case 16:rl_xix_b();break;
  case 17:rl_xix_c();break;
  case 18:rl_xix_d();break;
  case 19:rl_xix_e();break;
  case 20:rl_xix_h();break;
  case 21:rl_xix_l();break;
  case 22:rl_xix();break;
  case 23:rl_xix_a();break;
  case 24:rr_xix_b();break;
  case 25:rr_xix_c();break;
  case 26:rr_xix_d();break;
  case 27:rr_xix_e();break;
  case 28:rr_xix_h();break;
  case 29:rr_xix_l();break;
  case 30:rr_xix();break;
  case 31:rr_xix_a();break;
  case 32:sla_xix_b();break;
  case 33:sla_xix_c();break;
  case 34:sla_xix_d();break;
  case 35:sla_xix_e();break;
  case 36:sla_xix_h();break;
  case 37:sla_xix_l();break;
  case 38:sla_xix();break;
  case 39:sla_xix_a();break;
  case 40:sra_xix_b();break;
  case 41:sra_xix_c();break;
  case 42:sra_xix_d();break;
  case 43:sra_xix_e();break;
  case 44:sra_xix_h();break;
  case 45:sra_xix_l();break;
  case 46:sra_xix();break;
  case 47:sra_xix_a();break;
  case 48:sll_xix_b();break;
  case 49:sll_xix_c();break;
  case 50:sll_xix_d();break;
  case 51:sll_xix_e();break;
  case 52:sll_xix_h();break;
  case 53:sll_xix_l();break;
  case 54:sll_xix();break;
  case 55:sll_xix_a();break;
  case 56:srl_xix_b();break;
  case 57:srl_xix_c();break;
  case 58:srl_xix_d();break;
  case 59:srl_xix_e();break;
  case 60:srl_xix_h();break;
  case 61:srl_xix_l();break;
  case 62:srl_xix();break;
  case 63:srl_xix_a();break;
  case 64:bit_0_xix_b();break;
  case 65:bit_0_xix_c();break;
  case 66:bit_0_xix_d();break;
  case 67:bit_0_xix_e();break;
  case 68:bit_0_xix_h();break;
  case 69:bit_0_xix_l();break;
  case 70:bit_0_xix();break;
  case 71:bit_0_xix_a();break;
  case 72:bit_1_xix_b();break;
  case 73:bit_1_xix_c();break;
  case 74:bit_1_xix_d();break;
  case 75:bit_1_xix_e();break;
  case 76:bit_1_xix_h();break;
  case 77:bit_1_xix_l();break;
  case 78:bit_1_xix();break;
  case 79:bit_1_xix_a();break;
  case 80:bit_2_xix_b();break;
  case 81:bit_2_xix_c();break;
  case 82:bit_2_xix_d();break;
  case 83:bit_2_xix_e();break;
  case 84:bit_2_xix_h();break;
  case 85:bit_2_xix_l();break;
  case 86:bit_2_xix();break;
  case 87:bit_2_xix_a();break;
  case 88:bit_3_xix_b();break;
  case 89:bit_3_xix_c();break;
  case 90:bit_3_xix_d();break;
  case 91:bit_3_xix_e();break;
  case 92:bit_3_xix_h();break;
  case 93:bit_3_xix_l();break;
  case 94:bit_3_xix();break;
  case 95:bit_3_xix_a();break;
  case 96:bit_4_xix_b();break;
  case 97:bit_4_xix_c();break;
  case 98:bit_4_xix_d();break;
  case 99:bit_4_xix_e();break;
  case 100:bit_4_xix_h();break;
  case 101:bit_4_xix_l();break;
  case 102:bit_4_xix();break;
  case 103:bit_4_xix_a();break;
  case 104:bit_5_xix_b();break;
  case 105:bit_5_xix_c();break;
  case 106:bit_5_xix_d();break;
  case 107:bit_5_xix_e();break;
  case 108:bit_5_xix_h();break;
  case 109:bit_5_xix_l();break;
  case 110:bit_5_xix();break;
  case 111:bit_5_xix_a();break;
  case 112:bit_6_xix_b();break;
  case 113:bit_6_xix_c();break;
  case 114:bit_6_xix_d();break;
  case 115:bit_6_xix_e();break;
  case 116:bit_6_xix_h();break;
  case 117:bit_6_xix_l();break;
  case 118:bit_6_xix();break;
  case 119:bit_6_xix_a();break;
  case 120:bit_7_xix_b();break;
  case 121:bit_7_xix_c();break;
  case 122:bit_7_xix_d();break;
  case 123:bit_7_xix_e();break;
  case 124:bit_7_xix_h();break;
  case 125:bit_7_xix_l();break;
  case 126:bit_7_xix();break;
  case 127:bit_7_xix_a();break;
  case 128:res_0_xix_b();break;
  case 129:res_0_xix_c();break;
  case 130:res_0_xix_d();break;
  case 131:res_0_xix_e();break;
  case 132:res_0_xix_h();break;
  case 133:res_0_xix_l();break;
  case 134:res_0_xix();break;
  case 135:res_0_xix_a();break;
  case 136:res_1_xix_b();break;
  case 137:res_1_xix_c();break;
  case 138:res_1_xix_d();break;
  case 139:res_1_xix_e();break;
  case 140:res_1_xix_h();break;
  case 141:res_1_xix_l();break;
  case 142:res_1_xix();break;
  case 143:res_1_xix_a();break;
  case 144:res_2_xix_b();break;
  case 145:res_2_xix_c();break;
  case 146:res_2_xix_d();break;
  case 147:res_2_xix_e();break;
  case 148:res_2_xix_h();break;
  case 149:res_2_xix_l();break;
  case 150:res_2_xix();break;
  case 151:res_2_xix_a();break;
  case 152:res_3_xix_b();break;
  case 153:res_3_xix_c();break;
  case 154:res_3_xix_d();break;
  case 155:res_3_xix_e();break;
  case 156:res_3_xix_h();break;
  case 157:res_3_xix_l();break;
  case 158:res_3_xix();break;
  case 159:res_3_xix_a();break;
  case 160:res_4_xix_b();break;
  case 161:res_4_xix_c();break;
  case 162:res_4_xix_d();break;
  case 163:res_4_xix_e();break;
  case 164:res_4_xix_h();break;
  case 165:res_4_xix_l();break;
  case 166:res_4_xix();break;
  case 167:res_4_xix_a();break;
  case 168:res_5_xix_b();break;
  case 169:res_5_xix_c();break;
  case 170:res_5_xix_d();break;
  case 171:res_5_xix_e();break;
  case 172:res_5_xix_h();break;
  case 173:res_5_xix_l();break;
  case 174:res_5_xix();break;
  case 175:res_5_xix_a();break;
  case 176:res_6_xix_b();break;
  case 177:res_6_xix_c();break;
  case 178:res_6_xix_d();break;
  case 179:res_6_xix_e();break;
  case 180:res_6_xix_h();break;
  case 181:res_6_xix_l();break;
  case 182:res_6_xix();break;
  case 183:res_6_xix_a();break;
  case 184:res_7_xix_b();break;
  case 185:res_7_xix_c();break;
  case 186:res_7_xix_d();break;
  case 187:res_7_xix_e();break;
  case 188:res_7_xix_h();break;
  case 189:res_7_xix_l();break;
  case 190:res_7_xix();break;
  case 191:res_7_xix_a();break;
  case 192:set_0_xix_b();break;
  case 193:set_0_xix_c();break;
  case 194:set_0_xix_d();break;
  case 195:set_0_xix_e();break;
  case 196:set_0_xix_h();break;
  case 197:set_0_xix_l();break;
  case 198:set_0_xix();break;
  case 199:set_0_xix_a();break;
  case 200:set_1_xix_b();break;
  case 201:set_1_xix_c();break;
  case 202:set_1_xix_d();break;
  case 203:set_1_xix_e();break;
  case 204:set_1_xix_h();break;
  case 205:set_1_xix_l();break;
  case 206:set_1_xix();break;
  case 207:set_1_xix_a();break;
  case 208:set_2_xix_b();break;
  case 209:set_2_xix_c();break;
  case 210:set_2_xix_d();break;
  case 211:set_2_xix_e();break;
  case 212:set_2_xix_h();break;
  case 213:set_2_xix_l();break;
  case 214:set_2_xix();break;
  case 215:set_2_xix_a();break;
  case 216:set_3_xix_b();break;
  case 217:set_3_xix_c();break;
  case 218:set_3_xix_d();break;
  case 219:set_3_xix_e();break;
  case 220:set_3_xix_h();break;
  case 221:set_3_xix_l();break;
  case 222:set_3_xix();break;
  case 223:set_3_xix_a();break;
  case 224:set_4_xix_b();break;
  case 225:set_4_xix_c();break;
  case 226:set_4_xix_d();break;
  case 227:set_4_xix_e();break;
  case 228:set_4_xix_h();break;
  case 229:set_4_xix_l();break;
  case 230:set_4_xix();break;
  case 231:set_4_xix_a();break;
  case 232:set_5_xix_b();break;
  case 233:set_5_xix_c();break;
  case 234:set_5_xix_d();break;
  case 235:set_5_xix_e();break;
  case 236:set_5_xix_h();break;
  case 237:set_5_xix_l();break;
  case 238:set_5_xix();break;
  case 239:set_5_xix_a();break;
  case 240:set_6_xix_b();break;
  case 241:set_6_xix_c();break;
  case 242:set_6_xix_d();break;
  case 243:set_6_xix_e();break;
  case 244:set_6_xix_h();break;
  case 245:set_6_xix_l();break;
  case 246:set_6_xix();break;
  case 247:set_6_xix_a();break;
  case 248:set_7_xix_b();break;
  case 249:set_7_xix_c();break;
  case 250:set_7_xix_d();break;
  case 251:set_7_xix_e();break;
  case 252:set_7_xix_h();break;
  case 253:set_7_xix_l();break;
  case 254:set_7_xix();break;
  case 255:set_7_xix_a();break;

  default:    printf("Z80::call_opcode_dd_cb(%d):Invalidcode!\n",aCode);
    break;
  }
}
void Z80::call_opcode_fd_cb(byte aCode){
  switch(aCode){
  case 0:rlc_xiy_b();break;
  case 1:rlc_xiy_c();break;
  case 2:rlc_xiy_d();break;
  case 3:rlc_xiy_e();break;
  case 4:rlc_xiy_h();break;
  case 5:rlc_xiy_l();break;
  case 6:rlc_xiy();break;
  case 7:rlc_xiy_a();break;
  case 8:rrc_xiy_b();break;
  case 9:rrc_xiy_c();break;
  case 10:rrc_xiy_d();break;
  case 11:rrc_xiy_e();break;
  case 12:rrc_xiy_h();break;
  case 13:rrc_xiy_l();break;
  case 14:rrc_xiy();break;
  case 15:rrc_xiy_a();break;
  case 16:rl_xiy_b();break;
  case 17:rl_xiy_c();break;
  case 18:rl_xiy_d();break;
  case 19:rl_xiy_e();break;
  case 20:rl_xiy_h();break;
  case 21:rl_xiy_l();break;
  case 22:rl_xiy();break;
  case 23:rl_xiy_a();break;
  case 24:rr_xiy_b();break;
  case 25:rr_xiy_c();break;
  case 26:rr_xiy_d();break;
  case 27:rr_xiy_e();break;
  case 28:rr_xiy_h();break;
  case 29:rr_xiy_l();break;
  case 30:rr_xiy();break;
  case 31:rr_xiy_a();break;
  case 32:sla_xiy_b();break;
  case 33:sla_xiy_c();break;
  case 34:sla_xiy_d();break;
  case 35:sla_xiy_e();break;
  case 36:sla_xiy_h();break;
  case 37:sla_xiy_l();break;
  case 38:sla_xiy();break;
  case 39:sla_xiy_a();break;
  case 40:sra_xiy_b();break;
  case 41:sra_xiy_c();break;
  case 42:sra_xiy_d();break;
  case 43:sra_xiy_e();break;
  case 44:sra_xiy_h();break;
  case 45:sra_xiy_l();break;
  case 46:sra_xiy();break;
  case 47:sra_xiy_a();break;
  case 48:sll_xiy_b();break;
  case 49:sll_xiy_c();break;
  case 50:sll_xiy_d();break;
  case 51:sll_xiy_e();break;
  case 52:sll_xiy_h();break;
  case 53:sll_xiy_l();break;
  case 54:sll_xiy();break;
  case 55:sll_xiy_a();break;
  case 56:srl_xiy_b();break;
  case 57:srl_xiy_c();break;
  case 58:srl_xiy_d();break;
  case 59:srl_xiy_e();break;
  case 60:srl_xiy_h();break;
  case 61:srl_xiy_l();break;
  case 62:srl_xiy();break;
  case 63:srl_xiy_a();break;
  case 64:bit_0_xiy_b();break;
  case 65:bit_0_xiy_c();break;
  case 66:bit_0_xiy_d();break;
  case 67:bit_0_xiy_e();break;
  case 68:bit_0_xiy_h();break;
  case 69:bit_0_xiy_l();break;
  case 70:bit_0_xiy();break;
  case 71:bit_0_xiy_a();break;
  case 72:bit_1_xiy_b();break;
  case 73:bit_1_xiy_c();break;
  case 74:bit_1_xiy_d();break;
  case 75:bit_1_xiy_e();break;
  case 76:bit_1_xiy_h();break;
  case 77:bit_1_xiy_l();break;
  case 78:bit_1_xiy();break;
  case 79:bit_1_xiy_a();break;
  case 80:bit_2_xiy_b();break;
  case 81:bit_2_xiy_c();break;
  case 82:bit_2_xiy_d();break;
  case 83:bit_2_xiy_e();break;
  case 84:bit_2_xiy_h();break;
  case 85:bit_2_xiy_l();break;
  case 86:bit_2_xiy();break;
  case 87:bit_2_xiy_a();break;
  case 88:bit_3_xiy_b();break;
  case 89:bit_3_xiy_c();break;
  case 90:bit_3_xiy_d();break;
  case 91:bit_3_xiy_e();break;
  case 92:bit_3_xiy_h();break;
  case 93:bit_3_xiy_l();break;
  case 94:bit_3_xiy();break;
  case 95:bit_3_xiy_a();break;
  case 96:bit_4_xiy_b();break;
  case 97:bit_4_xiy_c();break;
  case 98:bit_4_xiy_d();break;
  case 99:bit_4_xiy_e();break;
  case 100:bit_4_xiy_h();break;
  case 101:bit_4_xiy_l();break;
  case 102:bit_4_xiy();break;
  case 103:bit_4_xiy_a();break;
  case 104:bit_5_xiy_b();break;
  case 105:bit_5_xiy_c();break;
  case 106:bit_5_xiy_d();break;
  case 107:bit_5_xiy_e();break;
  case 108:bit_5_xiy_h();break;
  case 109:bit_5_xiy_l();break;
  case 110:bit_5_xiy();break;
  case 111:bit_5_xiy_a();break;
  case 112:bit_6_xiy_b();break;
  case 113:bit_6_xiy_c();break;
  case 114:bit_6_xiy_d();break;
  case 115:bit_6_xiy_e();break;
  case 116:bit_6_xiy_h();break;
  case 117:bit_6_xiy_l();break;
  case 118:bit_6_xiy();break;
  case 119:bit_6_xiy_a();break;
  case 120:bit_7_xiy_b();break;
  case 121:bit_7_xiy_c();break;
  case 122:bit_7_xiy_d();break;
  case 123:bit_7_xiy_e();break;
  case 124:bit_7_xiy_h();break;
  case 125:bit_7_xiy_l();break;
  case 126:bit_7_xiy();break;
  case 127:bit_7_xiy_a();break;
  case 128:res_0_xiy_b();break;
  case 129:res_0_xiy_c();break;
  case 130:res_0_xiy_d();break;
  case 131:res_0_xiy_e();break;
  case 132:res_0_xiy_h();break;
  case 133:res_0_xiy_l();break;
  case 134:res_0_xiy();break;
  case 135:res_0_xiy_a();break;
  case 136:res_1_xiy_b();break;
  case 137:res_1_xiy_c();break;
  case 138:res_1_xiy_d();break;
  case 139:res_1_xiy_e();break;
  case 140:res_1_xiy_h();break;
  case 141:res_1_xiy_l();break;
  case 142:res_1_xiy();break;
  case 143:res_1_xiy_a();break;
  case 144:res_2_xiy_b();break;
  case 145:res_2_xiy_c();break;
  case 146:res_2_xiy_d();break;
  case 147:res_2_xiy_e();break;
  case 148:res_2_xiy_h();break;
  case 149:res_2_xiy_l();break;
  case 150:res_2_xiy();break;
  case 151:res_2_xiy_a();break;
  case 152:res_3_xiy_b();break;
  case 153:res_3_xiy_c();break;
  case 154:res_3_xiy_d();break;
  case 155:res_3_xiy_e();break;
  case 156:res_3_xiy_h();break;
  case 157:res_3_xiy_l();break;
  case 158:res_3_xiy();break;
  case 159:res_3_xiy_a();break;
  case 160:res_4_xiy_b();break;
  case 161:res_4_xiy_c();break;
  case 162:res_4_xiy_d();break;
  case 163:res_4_xiy_e();break;
  case 164:res_4_xiy_h();break;
  case 165:res_4_xiy_l();break;
  case 166:res_4_xiy();break;
  case 167:res_4_xiy_a();break;
  case 168:res_5_xiy_b();break;
  case 169:res_5_xiy_c();break;
  case 170:res_5_xiy_d();break;
  case 171:res_5_xiy_e();break;
  case 172:res_5_xiy_h();break;
  case 173:res_5_xiy_l();break;
  case 174:res_5_xiy();break;
  case 175:res_5_xiy_a();break;
  case 176:res_6_xiy_b();break;
  case 177:res_6_xiy_c();break;
  case 178:res_6_xiy_d();break;
  case 179:res_6_xiy_e();break;
  case 180:res_6_xiy_h();break;
  case 181:res_6_xiy_l();break;
  case 182:res_6_xiy();break;
  case 183:res_6_xiy_a();break;
  case 184:res_7_xiy_b();break;
  case 185:res_7_xiy_c();break;
  case 186:res_7_xiy_d();break;
  case 187:res_7_xiy_e();break;
  case 188:res_7_xiy_h();break;
  case 189:res_7_xiy_l();break;
  case 190:res_7_xiy();break;
  case 191:res_7_xiy_a();break;
  case 192:set_0_xiy_b();break;
  case 193:set_0_xiy_c();break;
  case 194:set_0_xiy_d();break;
  case 195:set_0_xiy_e();break;
  case 196:set_0_xiy_h();break;
  case 197:set_0_xiy_l();break;
  case 198:set_0_xiy();break;
  case 199:set_0_xiy_a();break;
  case 200:set_1_xiy_b();break;
  case 201:set_1_xiy_c();break;
  case 202:set_1_xiy_d();break;
  case 203:set_1_xiy_e();break;
  case 204:set_1_xiy_h();break;
  case 205:set_1_xiy_l();break;
  case 206:set_1_xiy();break;
  case 207:set_1_xiy_a();break;
  case 208:set_2_xiy_b();break;
  case 209:set_2_xiy_c();break;
  case 210:set_2_xiy_d();break;
  case 211:set_2_xiy_e();break;
  case 212:set_2_xiy_h();break;
  case 213:set_2_xiy_l();break;
  case 214:set_2_xiy();break;
  case 215:set_2_xiy_a();break;
  case 216:set_3_xiy_b();break;
  case 217:set_3_xiy_c();break;
  case 218:set_3_xiy_d();break;
  case 219:set_3_xiy_e();break;
  case 220:set_3_xiy_h();break;
  case 221:set_3_xiy_l();break;
  case 222:set_3_xiy();break;
  case 223:set_3_xiy_a();break;
  case 224:set_4_xiy_b();break;
  case 225:set_4_xiy_c();break;
  case 226:set_4_xiy_d();break;
  case 227:set_4_xiy_e();break;
  case 228:set_4_xiy_h();break;
  case 229:set_4_xiy_l();break;
  case 230:set_4_xiy();break;
  case 231:set_4_xiy_a();break;
  case 232:set_5_xiy_b();break;
  case 233:set_5_xiy_c();break;
  case 234:set_5_xiy_d();break;
  case 235:set_5_xiy_e();break;
  case 236:set_5_xiy_h();break;
  case 237:set_5_xiy_l();break;
  case 238:set_5_xiy();break;
  case 239:set_5_xiy_a();break;
  case 240:set_6_xiy_b();break;
  case 241:set_6_xiy_c();break;
  case 242:set_6_xiy_d();break;
  case 243:set_6_xiy_e();break;
  case 244:set_6_xiy_h();break;
  case 245:set_6_xiy_l();break;
  case 246:set_6_xiy();break;
  case 247:set_6_xiy_a();break;
  case 248:set_7_xiy_b();break;
  case 249:set_7_xiy_c();break;
  case 250:set_7_xiy_d();break;
  case 251:set_7_xiy_e();break;
  case 252:set_7_xiy_h();break;
  case 253:set_7_xiy_l();break;
  case 254:set_7_xiy();break;
  case 255:set_7_xiy_a();break;

  default:    printf("Z80::call_opcode_fd_cb(%d):Invalidcode!\n",aCode);
    break;
  }
}
void Z80::call_opcode_cb(byte aCode){
  switch(aCode){
  case 0:rlc_b();break;
  case 1:rlc_c();break;
  case 2:rlc_d();break;
  case 3:rlc_e();break;
  case 4:rlc_h();break;
  case 5:rlc_l();break;
  case 6:rlc_xhl();break;
  case 7:rlc_a();break;
  case 8:rrc_b();break;
  case 9:rrc_c();break;
  case 10:rrc_d();break;
  case 11:rrc_e();break;
  case 12:rrc_h();break;
  case 13:rrc_l();break;
  case 14:rrc_xhl();break;
  case 15:rrc_a();break;
  case 16:rl_b();break;
  case 17:rl_c();break;
  case 18:rl_d();break;
  case 19:rl_e();break;
  case 20:rl_h();break;
  case 21:rl_l();break;
  case 22:rl_xhl();break;
  case 23:rl_a();break;
  case 24:rr_b();break;
  case 25:rr_c();break;
  case 26:rr_d();break;
  case 27:rr_e();break;
  case 28:rr_h();break;
  case 29:rr_l();break;
  case 30:rr_xhl();break;
  case 31:rr_a();break;
  case 32:sla_b();break;
  case 33:sla_c();break;
  case 34:sla_d();break;
  case 35:sla_e();break;
  case 36:sla_h();break;
  case 37:sla_l();break;
  case 38:sla_xhl();break;
  case 39:sla_a();break;
  case 40:sra_b();break;
  case 41:sra_c();break;
  case 42:sra_d();break;
  case 43:sra_e();break;
  case 44:sra_h();break;
  case 45:sra_l();break;
  case 46:sra_xhl();break;
  case 47:sra_a();break;
  case 48:sll_b();break;
  case 49:sll_c();break;
  case 50:sll_d();break;
  case 51:sll_e();break;
  case 52:sll_h();break;
  case 53:sll_l();break;
  case 54:sll_xhl();break;
  case 55:sll_a();break;
  case 56:srl_b();break;
  case 57:srl_c();break;
  case 58:srl_d();break;
  case 59:srl_e();break;
  case 60:srl_h();break;
  case 61:srl_l();break;
  case 62:srl_xhl();break;
  case 63:srl_a();break;
  case 64:bit_0_b();break;
  case 65:bit_0_c();break;
  case 66:bit_0_d();break;
  case 67:bit_0_e();break;
  case 68:bit_0_h();break;
  case 69:bit_0_l();break;
  case 70:bit_0_xhl();break;
  case 71:bit_0_a();break;
  case 72:bit_1_b();break;
  case 73:bit_1_c();break;
  case 74:bit_1_d();break;
  case 75:bit_1_e();break;
  case 76:bit_1_h();break;
  case 77:bit_1_l();break;
  case 78:bit_1_xhl();break;
  case 79:bit_1_a();break;
  case 80:bit_2_b();break;
  case 81:bit_2_c();break;
  case 82:bit_2_d();break;
  case 83:bit_2_e();break;
  case 84:bit_2_h();break;
  case 85:bit_2_l();break;
  case 86:bit_2_xhl();break;
  case 87:bit_2_a();break;
  case 88:bit_3_b();break;
  case 89:bit_3_c();break;
  case 90:bit_3_d();break;
  case 91:bit_3_e();break;
  case 92:bit_3_h();break;
  case 93:bit_3_l();break;
  case 94:bit_3_xhl();break;
  case 95:bit_3_a();break;
  case 96:bit_4_b();break;
  case 97:bit_4_c();break;
  case 98:bit_4_d();break;
  case 99:bit_4_e();break;
  case 100:bit_4_h();break;
  case 101:bit_4_l();break;
  case 102:bit_4_xhl();break;
  case 103:bit_4_a();break;
  case 104:bit_5_b();break;
  case 105:bit_5_c();break;
  case 106:bit_5_d();break;
  case 107:bit_5_e();break;
  case 108:bit_5_h();break;
  case 109:bit_5_l();break;
  case 110:bit_5_xhl();break;
  case 111:bit_5_a();break;
  case 112:bit_6_b();break;
  case 113:bit_6_c();break;
  case 114:bit_6_d();break;
  case 115:bit_6_e();break;
  case 116:bit_6_h();break;
  case 117:bit_6_l();break;
  case 118:bit_6_xhl();break;
  case 119:bit_6_a();break;
  case 120:bit_7_b();break;
  case 121:bit_7_c();break;
  case 122:bit_7_d();break;
  case 123:bit_7_e();break;
  case 124:bit_7_h();break;
  case 125:bit_7_l();break;
  case 126:bit_7_xhl();break;
  case 127:bit_7_a();break;
  case 128:res_0_b();break;
  case 129:res_0_c();break;
  case 130:res_0_d();break;
  case 131:res_0_e();break;
  case 132:res_0_h();break;
  case 133:res_0_l();break;
  case 134:res_0_xhl();break;
  case 135:res_0_a();break;
  case 136:res_1_b();break;
  case 137:res_1_c();break;
  case 138:res_1_d();break;
  case 139:res_1_e();break;
  case 140:res_1_h();break;
  case 141:res_1_l();break;
  case 142:res_1_xhl();break;
  case 143:res_1_a();break;
  case 144:res_2_b();break;
  case 145:res_2_c();break;
  case 146:res_2_d();break;
  case 147:res_2_e();break;
  case 148:res_2_h();break;
  case 149:res_2_l();break;
  case 150:res_2_xhl();break;
  case 151:res_2_a();break;
  case 152:res_3_b();break;
  case 153:res_3_c();break;
  case 154:res_3_d();break;
  case 155:res_3_e();break;
  case 156:res_3_h();break;
  case 157:res_3_l();break;
  case 158:res_3_xhl();break;
  case 159:res_3_a();break;
  case 160:res_4_b();break;
  case 161:res_4_c();break;
  case 162:res_4_d();break;
  case 163:res_4_e();break;
  case 164:res_4_h();break;
  case 165:res_4_l();break;
  case 166:res_4_xhl();break;
  case 167:res_4_a();break;
  case 168:res_5_b();break;
  case 169:res_5_c();break;
  case 170:res_5_d();break;
  case 171:res_5_e();break;
  case 172:res_5_h();break;
  case 173:res_5_l();break;
  case 174:res_5_xhl();break;
  case 175:res_5_a();break;
  case 176:res_6_b();break;
  case 177:res_6_c();break;
  case 178:res_6_d();break;
  case 179:res_6_e();break;
  case 180:res_6_h();break;
  case 181:res_6_l();break;
  case 182:res_6_xhl();break;
  case 183:res_6_a();break;
  case 184:res_7_b();break;
  case 185:res_7_c();break;
  case 186:res_7_d();break;
  case 187:res_7_e();break;
  case 188:res_7_h();break;
  case 189:res_7_l();break;
  case 190:res_7_xhl();break;
  case 191:res_7_a();break;
  case 192:set_0_b();break;
  case 193:set_0_c();break;
  case 194:set_0_d();break;
  case 195:set_0_e();break;
  case 196:set_0_h();break;
  case 197:set_0_l();break;
  case 198:set_0_xhl();break;
  case 199:set_0_a();break;
  case 200:set_1_b();break;
  case 201:set_1_c();break;
  case 202:set_1_d();break;
  case 203:set_1_e();break;
  case 204:set_1_h();break;
  case 205:set_1_l();break;
  case 206:set_1_xhl();break;
  case 207:set_1_a();break;
  case 208:set_2_b();break;
  case 209:set_2_c();break;
  case 210:set_2_d();break;
  case 211:set_2_e();break;
  case 212:set_2_h();break;
  case 213:set_2_l();break;
  case 214:set_2_xhl();break;
  case 215:set_2_a();break;
  case 216:set_3_b();break;
  case 217:set_3_c();break;
  case 218:set_3_d();break;
  case 219:set_3_e();break;
  case 220:set_3_h();break;
  case 221:set_3_l();break;
  case 222:set_3_xhl();break;
  case 223:set_3_a();break;
  case 224:set_4_b();break;
  case 225:set_4_c();break;
  case 226:set_4_d();break;
  case 227:set_4_e();break;
  case 228:set_4_h();break;
  case 229:set_4_l();break;
  case 230:set_4_xhl();break;
  case 231:set_4_a();break;
  case 232:set_5_b();break;
  case 233:set_5_c();break;
  case 234:set_5_d();break;
  case 235:set_5_e();break;
  case 236:set_5_h();break;
  case 237:set_5_l();break;
  case 238:set_5_xhl();break;
  case 239:set_5_a();break;
  case 240:set_6_b();break;
  case 241:set_6_c();break;
  case 242:set_6_d();break;
  case 243:set_6_e();break;
  case 244:set_6_h();break;
  case 245:set_6_l();break;
  case 246:set_6_xhl();break;
  case 247:set_6_a();break;
  case 248:set_7_b();break;
  case 249:set_7_c();break;
  case 250:set_7_d();break;
  case 251:set_7_e();break;
  case 252:set_7_h();break;
  case 253:set_7_l();break;
  case 254:set_7_xhl();break;
  case 255:set_7_a();break;

  default:    printf("Z80::call_opcode_cb(%d):Invalidcode!\n",aCode);
    break;
  }
}

void Z80::call_opcode_dd(byte aCode){
  switch(aCode){
  case 0:no_op();break;
  case 1:no_op();break;
  case 2:no_op();break;
  case 3:no_op();break;
  case 4:no_op();break;
  case 5:no_op();break;
  case 6:no_op();break;
  case 7:no_op();break;
  case 8:no_op();break;
  case 9:add_ix_bc();break;
  case 10:no_op();break;
  case 11:no_op();break;
  case 12:no_op();break;
  case 13:no_op();break;
  case 14:no_op();break;
  case 15:no_op();break;
  case 16:no_op();break;
  case 17:no_op();break;
  case 18:no_op();break;
  case 19:no_op();break;
  case 20:no_op();break;
  case 21:no_op();break;
  case 22:no_op();break;
  case 23:no_op();break;
  case 24:no_op();break;
  case 25:add_ix_de();break;
  case 26:no_op();break;
  case 27:no_op();break;
  case 28:no_op();break;
  case 29:no_op();break;
  case 30:no_op();break;
  case 31:no_op();break;
  case 32:no_op();break;
  case 33:ld_ix_word();break;
  case 34:ld_xword_ix();break;
  case 35:inc_ix();break;
  case 36:inc_ixh();break;
  case 37:dec_ixh();break;
  case 38:ld_ixh_byte();break;
  case 39:no_op();break;
  case 40:no_op();break;
  case 41:add_ix_ix();break;
  case 42:ld_ix_xword();break;
  case 43:dec_ix();break;
  case 44:inc_ixl();break;
  case 45:dec_ixl();break;
  case 46:ld_ixl_byte();break;
  case 47:no_op();break;
  case 48:no_op();break;
  case 49:no_op();break;
  case 50:no_op();break;
  case 51:no_op();break;
  case 52:inc_xix();break;
  case 53:dec_xix();break;
  case 54:ld_xix_byte();break;
  case 55:no_op();break;
  case 56:no_op();break;
  case 57:add_ix_sp();break;
  case 58:no_op();break;
  case 59:no_op();break;
  case 60:no_op();break;
  case 61:no_op();break;
  case 62:no_op();break;
  case 63:no_op();break;
  case 64:no_op();break;
  case 65:no_op();break;
  case 66:no_op();break;
  case 67:no_op();break;
  case 68:ld_b_ixh();break;
  case 69:ld_b_ixl();break;
  case 70:ld_b_xix();break;
  case 71:no_op();break;
  case 72:no_op();break;
  case 73:no_op();break;
  case 74:no_op();break;
  case 75:no_op();break;
  case 76:ld_c_ixh();break;
  case 77:ld_c_ixl();break;
  case 78:ld_c_xix();break;
  case 79:no_op();break;
  case 80:no_op();break;
  case 81:no_op();break;
  case 82:no_op();break;
  case 83:no_op();break;
  case 84:ld_d_ixh();break;
  case 85:ld_d_ixl();break;
  case 86:ld_d_xix();break;
  case 87:no_op();break;
  case 88:no_op();break;
  case 89:no_op();break;
  case 90:no_op();break;
  case 91:no_op();break;
  case 92:ld_e_ixh();break;
  case 93:ld_e_ixl();break;
  case 94:ld_e_xix();break;
  case 95:no_op();break;
  case 96:ld_ixh_b();break;
  case 97:ld_ixh_c();break;
  case 98:ld_ixh_d();break;
  case 99:ld_ixh_e();break;
  case 100:ld_ixh_ixh();break;
  case 101:ld_ixh_ixl();break;
  case 102:ld_h_xix();break;
  case 103:ld_ixh_a();break;
  case 104:ld_ixl_b();break;
  case 105:ld_ixl_c();break;
  case 106:ld_ixl_d();break;
  case 107:ld_ixl_e();break;
  case 108:ld_ixl_ixh();break;
  case 109:ld_ixl_ixl();break;
  case 110:ld_l_xix();break;
  case 111:ld_ixl_a();break;
  case 112:ld_xix_b();break;
  case 113:ld_xix_c();break;
  case 114:ld_xix_d();break;
  case 115:ld_xix_e();break;
  case 116:ld_xix_h();break;
  case 117:ld_xix_l();break;
  case 118:no_op();break;
  case 119:ld_xix_a();break;
  case 120:no_op();break;
  case 121:no_op();break;
  case 122:no_op();break;
  case 123:no_op();break;
  case 124:ld_a_ixh();break;
  case 125:ld_a_ixl();break;
  case 126:ld_a_xix();break;
  case 127:no_op();break;
  case 128:no_op();break;
  case 129:no_op();break;
  case 130:no_op();break;
  case 131:no_op();break;
  case 132:add_a_ixh();break;
  case 133:add_a_ixl();break;
  case 134:add_a_xix();break;
  case 135:no_op();break;
  case 136:no_op();break;
  case 137:no_op();break;
  case 138:no_op();break;
  case 139:no_op();break;
  case 140:adc_a_ixh();break;
  case 141:adc_a_ixl();break;
  case 142:adc_a_xix();break;
  case 143:no_op();break;
  case 144:no_op();break;
  case 145:no_op();break;
  case 146:no_op();break;
  case 147:no_op();break;
  case 148:sub_ixh();break;
  case 149:sub_ixl();break;
  case 150:sub_xix();break;
  case 151:no_op();break;
  case 152:no_op();break;
  case 153:no_op();break;
  case 154:no_op();break;
  case 155:no_op();break;
  case 156:sbc_a_ixh();break;
  case 157:sbc_a_ixl();break;
  case 158:sbc_a_xix();break;
  case 159:no_op();break;
  case 160:no_op();break;
  case 161:no_op();break;
  case 162:no_op();break;
  case 163:no_op();break;
  case 164:and_ixh();break;
  case 165:and_ixl();break;
  case 166:and_xix();break;
  case 167:no_op();break;
  case 168:no_op();break;
  case 169:no_op();break;
  case 170:no_op();break;
  case 171:no_op();break;
  case 172:xor_ixh();break;
  case 173:xor_ixl();break;
  case 174:xor_xix();break;
  case 175:no_op();break;
  case 176:no_op();break;
  case 177:no_op();break;
  case 178:no_op();break;
  case 179:no_op();break;
  case 180:or_ixh();break;
  case 181:or_ixl();break;
  case 182:or_xix();break;
  case 183:no_op();break;
  case 184:no_op();break;
  case 185:no_op();break;
  case 186:no_op();break;
  case 187:no_op();break;
  case 188:cp_ixh();break;
  case 189:cp_ixl();break;
  case 190:cp_xix();break;
  case 191:no_op();break;
  case 192:no_op();break;
  case 193:no_op();break;
  case 194:no_op();break;
  case 195:no_op();break;
  case 196:no_op();break;
  case 197:no_op();break;
  case 198:no_op();break;
  case 199:no_op();break;
  case 200:no_op();break;
  case 201:no_op();break;
  case 202:no_op();break;
  case 203:dd_cb();break;
  case 204:no_op();break;
  case 205:no_op();break;
  case 206:no_op();break;
  case 207:no_op();break;
  case 208:no_op();break;
  case 209:no_op();break;
  case 210:no_op();break;
  case 211:no_op();break;
  case 212:no_op();break;
  case 213:no_op();break;
  case 214:no_op();break;
  case 215:no_op();break;
  case 216:no_op();break;
  case 217:no_op();break;
  case 218:no_op();break;
  case 219:no_op();break;
  case 220:no_op();break;
  case 221:no_op();break;
  case 222:no_op();break;
  case 223:no_op();break;
  case 224:no_op();break;
  case 225:pop_ix();break;
  case 226:no_op();break;
  case 227:ex_xsp_ix();break;
  case 228:no_op();break;
  case 229:push_ix();break;
  case 230:no_op();break;
  case 231:no_op();break;
  case 232:no_op();break;
  case 233:jp_ix();break;
  case 234:no_op();break;
  case 235:no_op();break;
  case 236:no_op();break;
  case 237:no_op();break;
  case 238:no_op();break;
  case 239:no_op();break;
  case 240:no_op();break;
  case 241:no_op();break;
  case 242:no_op();break;
  case 243:no_op();break;
  case 244:no_op();break;
  case 245:no_op();break;
  case 246:no_op();break;
  case 247:no_op();break;
  case 248:no_op();break;
  case 249:ld_sp_ix();break;
  case 250:no_op();break;
  case 251:no_op();break;
  case 252:no_op();break;
  case 253:no_op();break;
  case 254:no_op();break;
  case 255:no_op();break;

  default:    printf("Z80::call_opcode_dd(%d):Invalidcode!\n",aCode);
    break;
  }
}
void Z80::call_opcode_ed(byte aCode){
  switch(aCode){
  case 0:nop();break;
  case 1:nop();break;
  case 2:nop();break;
  case 3:nop();break;
  case 4:nop();break;
  case 5:nop();break;
  case 6:nop();break;
  case 7:nop();break;
  case 8:nop();break;
  case 9:nop();break;
  case 10:nop();break;
  case 11:nop();break;
  case 12:nop();break;
  case 13:nop();break;
  case 14:nop();break;
  case 15:nop();break;
  case 16:nop();break;
  case 17:nop();break;
  case 18:nop();break;
  case 19:nop();break;
  case 20:nop();break;
  case 21:nop();break;
  case 22:nop();break;
  case 23:nop();break;
  case 24:nop();break;
  case 25:nop();break;
  case 26:nop();break;
  case 27:nop();break;
  case 28:nop();break;
  case 29:nop();break;
  case 30:nop();break;
  case 31:nop();break;
  case 32:nop();break;
  case 33:nop();break;
  case 34:nop();break;
  case 35:nop();break;
  case 36:nop();break;
  case 37:nop();break;
  case 38:nop();break;
  case 39:nop();break;
  case 40:nop();break;
  case 41:nop();break;
  case 42:nop();break;
  case 43:nop();break;
  case 44:nop();break;
  case 45:nop();break;
  case 46:nop();break;
  case 47:nop();break;
  case 48:nop();break;
  case 49:nop();break;
  case 50:nop();break;
  case 51:nop();break;
  case 52:nop();break;
  case 53:nop();break;
  case 54:nop();break;
  case 55:nop();break;
  case 56:nop();break;
  case 57:nop();break;
  case 58:nop();break;
  case 59:nop();break;
  case 60:nop();break;
  case 61:nop();break;
  case 62:nop();break;
  case 63:nop();break;
  case 64:in_b_c();break;
  case 65:out_c_b();break;
  case 66:sbc_hl_bc();break;
  case 67:ld_xword_bc();break;
  case 68:neg();break;
  case 69:retn();break;
  case 70:im_0();break;
  case 71:ld_i_a();break;
  case 72:in_c_c();break;
  case 73:out_c_c();break;
  case 74:adc_hl_bc();break;
  case 75:ld_bc_xword();break;
  case 76:neg();break;
  case 77:reti();break;
  case 78:im_0();break;
  case 79:ld_r_a();break;
  case 80:in_d_c();break;
  case 81:out_c_d();break;
  case 82:sbc_hl_de();break;
  case 83:ld_xword_de();break;
  case 84:neg();break;
  case 85:retn();break;
  case 86:im_1();break;
  case 87:ld_a_i();break;
  case 88:in_e_c();break;
  case 89:out_c_e();break;
  case 90:adc_hl_de();break;
  case 91:ld_de_xword();break;
  case 92:neg();break;
  case 93:reti();break;
  case 94:im_2();break;
  case 95:ld_a_r();break;
  case 96:in_h_c();break;
  case 97:out_c_h();break;
  case 98:sbc_hl_hl();break;
  case 99:ld_xword_hl();break;
  case 100:neg();break;
  case 101:retn();break;
  case 102:im_0();break;
  case 103:rrd();break;
  case 104:in_l_c();break;
  case 105:out_c_l();break;
  case 106:adc_hl_hl();break;
  case 107:ld_hl_xword();break;
  case 108:neg();break;
  case 109:reti();break;
  case 110:im_0();break;
  case 111:rld();break;
  case 112:in_0_c();break;
  case 113:out_c_0();break;
  case 114:sbc_hl_sp();break;
  case 115:ld_xword_sp();break;
  case 116:neg();break;
  case 117:retn();break;
  case 118:im_1();break;
  case 119:nop();break;
  case 120:in_a_c();break;
  case 121:out_c_a();break;
  case 122:adc_hl_sp();break;
  case 123:ld_sp_xword();break;
  case 124:neg();break;
  case 125:reti();break;
  case 126:im_2();break;
  case 127:nop();break;
  case 128:nop();break;
  case 129:nop();break;
  case 130:nop();break;
  case 131:nop();break;
  case 132:nop();break;
  case 133:nop();break;
  case 134:nop();break;
  case 135:nop();break;
  case 136:nop();break;
  case 137:nop();break;
  case 138:nop();break;
  case 139:nop();break;
  case 140:nop();break;
  case 141:nop();break;
  case 142:nop();break;
  case 143:nop();break;
  case 144:nop();break;
  case 145:nop();break;
  case 146:nop();break;
  case 147:nop();break;
  case 148:nop();break;
  case 149:nop();break;
  case 150:nop();break;
  case 151:nop();break;
  case 152:nop();break;
  case 153:nop();break;
  case 154:nop();break;
  case 155:nop();break;
  case 156:nop();break;
  case 157:nop();break;
  case 158:nop();break;
  case 159:nop();break;
  case 160:ldi();break;
  case 161:cpi();break;
  case 162:ini();break;
  case 163:outi();break;
  case 164:nop();break;
  case 165:nop();break;
  case 166:nop();break;
  case 167:nop();break;
  case 168:ldd();break;
  case 169:cpd();break;
  case 170:ind();break;
  case 171:outd();break;
  case 172:nop();break;
  case 173:nop();break;
  case 174:nop();break;
  case 175:nop();break;
  case 176:ldir();break;
  case 177:cpir();break;
  case 178:inir();break;
  case 179:otir();break;
  case 180:nop();break;
  case 181:nop();break;
  case 182:nop();break;
  case 183:nop();break;
  case 184:lddr();break;
  case 185:cpdr();break;
  case 186:indr();break;
  case 187:otdr();break;
  case 188:nop();break;
  case 189:nop();break;
  case 190:nop();break;
  case 191:nop();break;
  case 192:nop();break;
  case 193:nop();break;
  case 194:nop();break;
  case 195:nop();break;
  case 196:nop();break;
  case 197:nop();break;
  case 198:nop();break;
  case 199:nop();break;
  case 200:nop();break;
  case 201:nop();break;
  case 202:nop();break;
  case 203:nop();break;
  case 204:nop();break;
  case 205:nop();break;
  case 206:nop();break;
  case 207:nop();break;
  case 208:nop();break;
  case 209:nop();break;
  case 210:nop();break;
  case 211:nop();break;
  case 212:nop();break;
  case 213:nop();break;
  case 214:nop();break;
  case 215:nop();break;
  case 216:nop();break;
  case 217:nop();break;
  case 218:nop();break;
  case 219:nop();break;
  case 220:nop();break;
  case 221:nop();break;
  case 222:nop();break;
  case 223:nop();break;
  case 224:nop();break;
  case 225:nop();break;
  case 226:nop();break;
  case 227:nop();break;
  case 228:nop();break;
  case 229:nop();break;
  case 230:nop();break;
  case 231:nop();break;
  case 232:nop();break;
  case 233:nop();break;
  case 234:nop();break;
  case 235:nop();break;
  case 236:nop();break;
  case 237:nop();break;
  case 238:nop();break;
  case 239:nop();break;
  case 240:nop();break;
  case 241:nop();break;
  case 242:nop();break;
  case 243:nop();break;
  case 244:nop();break;
  case 245:nop();break;
  case 246:nop();break;
  case 247:nop();break;
  case 248:nop();break;
  case 249:nop();break;
  case 250:nop();break;
  case 251:nop();break;
  case 252:nop();break;
  case 253:nop();break;
  case 254:patch();break;
  case 255:nop();break;

  default:    printf("Z80::call_opcode_ed(%d):Invalidcode!\n",aCode);
    break;
  }
}
void Z80::call_opcode_fd(byte aCode){
  switch(aCode){
  case 0:no_op();break;
  case 1:no_op();break;
  case 2:no_op();break;
  case 3:no_op();break;
  case 4:no_op();break;
  case 5:no_op();break;
  case 6:no_op();break;
  case 7:no_op();break;
  case 8:no_op();break;
  case 9:add_iy_bc();break;
  case 10:no_op();break;
  case 11:no_op();break;
  case 12:no_op();break;
  case 13:no_op();break;
  case 14:no_op();break;
  case 15:no_op();break;
  case 16:no_op();break;
  case 17:no_op();break;
  case 18:no_op();break;
  case 19:no_op();break;
  case 20:no_op();break;
  case 21:no_op();break;
  case 22:no_op();break;
  case 23:no_op();break;
  case 24:no_op();break;
  case 25:add_iy_de();break;
  case 26:no_op();break;
  case 27:no_op();break;
  case 28:no_op();break;
  case 29:no_op();break;
  case 30:no_op();break;
  case 31:no_op();break;
  case 32:no_op();break;
  case 33:ld_iy_word();break;
  case 34:ld_xword_iy();break;
  case 35:inc_iy();break;
  case 36:inc_iyh();break;
  case 37:dec_iyh();break;
  case 38:ld_iyh_byte();break;
  case 39:no_op();break;
  case 40:no_op();break;
  case 41:add_iy_iy();break;
  case 42:ld_iy_xword();break;
  case 43:dec_iy();break;
  case 44:inc_iyl();break;
  case 45:dec_iyl();break;
  case 46:ld_iyl_byte();break;
  case 47:no_op();break;
  case 48:no_op();break;
  case 49:no_op();break;
  case 50:no_op();break;
  case 51:no_op();break;
  case 52:inc_xiy();break;
  case 53:dec_xiy();break;
  case 54:ld_xiy_byte();break;
  case 55:no_op();break;
  case 56:no_op();break;
  case 57:add_iy_sp();break;
  case 58:no_op();break;
  case 59:no_op();break;
  case 60:no_op();break;
  case 61:no_op();break;
  case 62:no_op();break;
  case 63:no_op();break;
  case 64:no_op();break;
  case 65:no_op();break;
  case 66:no_op();break;
  case 67:no_op();break;
  case 68:ld_b_iyh();break;
  case 69:ld_b_iyl();break;
  case 70:ld_b_xiy();break;
  case 71:no_op();break;
  case 72:no_op();break;
  case 73:no_op();break;
  case 74:no_op();break;
  case 75:no_op();break;
  case 76:ld_c_iyh();break;
  case 77:ld_c_iyl();break;
  case 78:ld_c_xiy();break;
  case 79:no_op();break;
  case 80:no_op();break;
  case 81:no_op();break;
  case 82:no_op();break;
  case 83:no_op();break;
  case 84:ld_d_iyh();break;
  case 85:ld_d_iyl();break;
  case 86:ld_d_xiy();break;
  case 87:no_op();break;
  case 88:no_op();break;
  case 89:no_op();break;
  case 90:no_op();break;
  case 91:no_op();break;
  case 92:ld_e_iyh();break;
  case 93:ld_e_iyl();break;
  case 94:ld_e_xiy();break;
  case 95:no_op();break;
  case 96:ld_iyh_b();break;
  case 97:ld_iyh_c();break;
  case 98:ld_iyh_d();break;
  case 99:ld_iyh_e();break;
  case 100:ld_iyh_iyh();break;
  case 101:ld_iyh_iyl();break;
  case 102:ld_h_xiy();break;
  case 103:ld_iyh_a();break;
  case 104:ld_iyl_b();break;
  case 105:ld_iyl_c();break;
  case 106:ld_iyl_d();break;
  case 107:ld_iyl_e();break;
  case 108:ld_iyl_iyh();break;
  case 109:ld_iyl_iyl();break;
  case 110:ld_l_xiy();break;
  case 111:ld_iyl_a();break;
  case 112:ld_xiy_b();break;
  case 113:ld_xiy_c();break;
  case 114:ld_xiy_d();break;
  case 115:ld_xiy_e();break;
  case 116:ld_xiy_h();break;
  case 117:ld_xiy_l();break;
  case 118:no_op();break;
  case 119:ld_xiy_a();break;
  case 120:no_op();break;
  case 121:no_op();break;
  case 122:no_op();break;
  case 123:no_op();break;
  case 124:ld_a_iyh();break;
  case 125:ld_a_iyl();break;
  case 126:ld_a_xiy();break;
  case 127:no_op();break;
  case 128:no_op();break;
  case 129:no_op();break;
  case 130:no_op();break;
  case 131:no_op();break;
  case 132:add_a_iyh();break;
  case 133:add_a_iyl();break;
  case 134:add_a_xiy();break;
  case 135:no_op();break;
  case 136:no_op();break;
  case 137:no_op();break;
  case 138:no_op();break;
  case 139:no_op();break;
  case 140:adc_a_iyh();break;
  case 141:adc_a_iyl();break;
  case 142:adc_a_xiy();break;
  case 143:no_op();break;
  case 144:no_op();break;
  case 145:no_op();break;
  case 146:no_op();break;
  case 147:no_op();break;
  case 148:sub_iyh();break;
  case 149:sub_iyl();break;
  case 150:sub_xiy();break;
  case 151:no_op();break;
  case 152:no_op();break;
  case 153:no_op();break;
  case 154:no_op();break;
  case 155:no_op();break;
  case 156:sbc_a_iyh();break;
  case 157:sbc_a_iyl();break;
  case 158:sbc_a_xiy();break;
  case 159:no_op();break;
  case 160:no_op();break;
  case 161:no_op();break;
  case 162:no_op();break;
  case 163:no_op();break;
  case 164:and_iyh();break;
  case 165:and_iyl();break;
  case 166:and_xiy();break;
  case 167:no_op();break;
  case 168:no_op();break;
  case 169:no_op();break;
  case 170:no_op();break;
  case 171:no_op();break;
  case 172:xor_iyh();break;
  case 173:xor_iyl();break;
  case 174:xor_xiy();break;
  case 175:no_op();break;
  case 176:no_op();break;
  case 177:no_op();break;
  case 178:no_op();break;
  case 179:no_op();break;
  case 180:or_iyh();break;
  case 181:or_iyl();break;
  case 182:or_xiy();break;
  case 183:no_op();break;
  case 184:no_op();break;
  case 185:no_op();break;
  case 186:no_op();break;
  case 187:no_op();break;
  case 188:cp_iyh();break;
  case 189:cp_iyl();break;
  case 190:cp_xiy();break;
  case 191:no_op();break;
  case 192:no_op();break;
  case 193:no_op();break;
  case 194:no_op();break;
  case 195:no_op();break;
  case 196:no_op();break;
  case 197:no_op();break;
  case 198:no_op();break;
  case 199:no_op();break;
  case 200:no_op();break;
  case 201:no_op();break;
  case 202:no_op();break;
  case 203:fd_cb();break;
  case 204:no_op();break;
  case 205:no_op();break;
  case 206:no_op();break;
  case 207:no_op();break;
  case 208:no_op();break;
  case 209:no_op();break;
  case 210:no_op();break;
  case 211:no_op();break;
  case 212:no_op();break;
  case 213:no_op();break;
  case 214:no_op();break;
  case 215:no_op();break;
  case 216:no_op();break;
  case 217:no_op();break;
  case 218:no_op();break;
  case 219:no_op();break;
  case 220:no_op();break;
  case 221:no_op();break;
  case 222:no_op();break;
  case 223:no_op();break;
  case 224:no_op();break;
  case 225:pop_iy();break;
  case 226:no_op();break;
  case 227:ex_xsp_iy();break;
  case 228:no_op();break;
  case 229:push_iy();break;
  case 230:no_op();break;
  case 231:no_op();break;
  case 232:no_op();break;
  case 233:jp_iy();break;
  case 234:no_op();break;
  case 235:no_op();break;
  case 236:no_op();break;
  case 237:no_op();break;
  case 238:no_op();break;
  case 239:no_op();break;
  case 240:no_op();break;
  case 241:no_op();break;
  case 242:no_op();break;
  case 243:no_op();break;
  case 244:no_op();break;
  case 245:no_op();break;
  case 246:no_op();break;
  case 247:no_op();break;
  case 248:no_op();break;
  case 249:ld_sp_iy();break;
  case 250:no_op();break;
  case 251:no_op();break;
  case 252:no_op();break;
  case 253:no_op();break;
  case 254:no_op();break;
  case 255:no_op();break;

  default:    printf("Z80::call_opcode_fd(%d):Invalidcode!\n",aCode);
    break;
  }
}
void Z80::call_opcode_main(byte aCode){
  switch(aCode){
  case 0:nop();break;
  case 1:ld_bc_word();break;
  case 2:ld_xbc_a();break;
  case 3:inc_bc();break;
  case 4:inc_b();break;
  case 5:dec_b();break;
  case 6:ld_b_byte();break;
  case 7:rlca();break;
  case 8:ex_af_af();break;
  case 9:add_hl_bc();break;
  case 10:ld_a_xbc();break;
  case 11:dec_bc();break;
  case 12:inc_c();break;
  case 13:dec_c();break;
  case 14:ld_c_byte();break;
  case 15:rrca();break;
  case 16:djnz();break;
  case 17:ld_de_word();break;
  case 18:ld_xde_a();break;
  case 19:inc_de();break;
  case 20:inc_d();break;
  case 21:dec_d();break;
  case 22:ld_d_byte();break;
  case 23:rla();break;
  case 24:jr();break;
  case 25:add_hl_de();break;
  case 26:ld_a_xde();break;
  case 27:dec_de();break;
  case 28:inc_e();break;
  case 29:dec_e();break;
  case 30:ld_e_byte();break;
  case 31:rra();break;
  case 32:jr_nz();break;
  case 33:ld_hl_word();break;
  case 34:ld_xword_hl();break;
  case 35:inc_hl();break;
  case 36:inc_h();break;
  case 37:dec_h();break;
  case 38:ld_h_byte();break;
  case 39:daa();break;
  case 40:jr_z();break;
  case 41:add_hl_hl();break;
  case 42:ld_hl_xword();break;
  case 43:dec_hl();break;
  case 44:inc_l();break;
  case 45:dec_l();break;
  case 46:ld_l_byte();break;
  case 47:cpl();break;
  case 48:jr_nc();break;
  case 49:ld_sp_word();break;
  case 50:ld_xbyte_a();break;
  case 51:inc_sp();break;
  case 52:inc_xhl();break;
  case 53:dec_xhl();break;
  case 54:ld_xhl_byte();break;
  case 55:scf();break;
  case 56:jr_c();break;
  case 57:add_hl_sp();break;
  case 58:ld_a_xbyte();break;
  case 59:dec_sp();break;
  case 60:inc_a();break;
  case 61:dec_a();break;
  case 62:ld_a_byte();break;
  case 63:ccf();break;
  case 64:ld_b_b();break;
  case 65:ld_b_c();break;
  case 66:ld_b_d();break;
  case 67:ld_b_e();break;
  case 68:ld_b_h();break;
  case 69:ld_b_l();break;
  case 70:ld_b_xhl();break;
  case 71:ld_b_a();break;
  case 72:ld_c_b();break;
  case 73:ld_c_c();break;
  case 74:ld_c_d();break;
  case 75:ld_c_e();break;
  case 76:ld_c_h();break;
  case 77:ld_c_l();break;
  case 78:ld_c_xhl();break;
  case 79:ld_c_a();break;
  case 80:ld_d_b();break;
  case 81:ld_d_c();break;
  case 82:ld_d_d();break;
  case 83:ld_d_e();break;
  case 84:ld_d_h();break;
  case 85:ld_d_l();break;
  case 86:ld_d_xhl();break;
  case 87:ld_d_a();break;
  case 88:ld_e_b();break;
  case 89:ld_e_c();break;
  case 90:ld_e_d();break;
  case 91:ld_e_e();break;
  case 92:ld_e_h();break;
  case 93:ld_e_l();break;
  case 94:ld_e_xhl();break;
  case 95:ld_e_a();break;
  case 96:ld_h_b();break;
  case 97:ld_h_c();break;
  case 98:ld_h_d();break;
  case 99:ld_h_e();break;
  case 100:ld_h_h();break;
  case 101:ld_h_l();break;
  case 102:ld_h_xhl();break;
  case 103:ld_h_a();break;
  case 104:ld_l_b();break;
  case 105:ld_l_c();break;
  case 106:ld_l_d();break;
  case 107:ld_l_e();break;
  case 108:ld_l_h();break;
  case 109:ld_l_l();break;
  case 110:ld_l_xhl();break;
  case 111:ld_l_a();break;
  case 112:ld_xhl_b();break;
  case 113:ld_xhl_c();break;
  case 114:ld_xhl_d();break;
  case 115:ld_xhl_e();break;
  case 116:ld_xhl_h();break;
  case 117:ld_xhl_l();break;
  case 118:halt();break;
  case 119:ld_xhl_a();break;
  case 120:ld_a_b();break;
  case 121:ld_a_c();break;
  case 122:ld_a_d();break;
  case 123:ld_a_e();break;
  case 124:ld_a_h();break;
  case 125:ld_a_l();break;
  case 126:ld_a_xhl();break;
  case 127:ld_a_a();break;
  case 128:add_a_b();break;
  case 129:add_a_c();break;
  case 130:add_a_d();break;
  case 131:add_a_e();break;
  case 132:add_a_h();break;
  case 133:add_a_l();break;
  case 134:add_a_xhl();break;
  case 135:add_a_a();break;
  case 136:adc_a_b();break;
  case 137:adc_a_c();break;
  case 138:adc_a_d();break;
  case 139:adc_a_e();break;
  case 140:adc_a_h();break;
  case 141:adc_a_l();break;
  case 142:adc_a_xhl();break;
  case 143:adc_a_a();break;
  case 144:sub_b();break;
  case 145:sub_c();break;
  case 146:sub_d();break;
  case 147:sub_e();break;
  case 148:sub_h();break;
  case 149:sub_l();break;
  case 150:sub_xhl();break;
  case 151:sub_a();break;
  case 152:sbc_a_b();break;
  case 153:sbc_a_c();break;
  case 154:sbc_a_d();break;
  case 155:sbc_a_e();break;
  case 156:sbc_a_h();break;
  case 157:sbc_a_l();break;
  case 158:sbc_a_xhl();break;
  case 159:sbc_a_a();break;
  case 160:and_b();break;
  case 161:and_c();break;
  case 162:and_d();break;
  case 163:and_e();break;
  case 164:and_h();break;
  case 165:and_l();break;
  case 166:and_xhl();break;
  case 167:and_a();break;
  case 168:xor_b();break;
  case 169:xor_c();break;
  case 170:xor_d();break;
  case 171:xor_e();break;
  case 172:xor_h();break;
  case 173:xor_l();break;
  case 174:xor_xhl();break;
  case 175:xor_a();break;
  case 176:or_b();break;
  case 177:or_c();break;
  case 178:or_d();break;
  case 179:or_e();break;
  case 180:or_h();break;
  case 181:or_l();break;
  case 182:or_xhl();break;
  case 183:or_a();break;
  case 184:cp_b();break;
  case 185:cp_c();break;
  case 186:cp_d();break;
  case 187:cp_e();break;
  case 188:cp_h();break;
  case 189:cp_l();break;
  case 190:cp_xhl();break;
  case 191:cp_a();break;
  case 192:ret_nz();break;
  case 193:pop_bc();break;
  case 194:jp_nz();break;
  case 195:jp();break;
  case 196:call_nz();break;
  case 197:push_bc();break;
  case 198:add_a_byte();break;
  case 199:rst_00();break;
  case 200:ret_z();break;
  case 201:ret();break;
  case 202:jp_z();break;
  case 203:cb();break;
  case 204:call_z();break;
  case 205:call();break;
  case 206:adc_a_byte();break;
  case 207:rst_08();break;
  case 208:ret_nc();break;
  case 209:pop_de();break;
  case 210:jp_nc();break;
  case 211:out_byte_a();break;
  case 212:call_nc();break;
  case 213:push_de();break;
  case 214:sub_byte();break;
  case 215:rst_10();break;
  case 216:ret_c();break;
  case 217:exx();break;
  case 218:jp_c();break;
  case 219:in_a_byte();break;
  case 220:call_c();break;
  case 221:dd();break;
  case 222:sbc_a_byte();break;
  case 223:rst_18();break;
  case 224:ret_po();break;
  case 225:pop_hl();break;
  case 226:jp_po();break;
  case 227:ex_xsp_hl();break;
  case 228:call_po();break;
  case 229:push_hl();break;
  case 230:and_byte();break;
  case 231:rst_20();break;
  case 232:ret_pe();break;
  case 233:jp_hl();break;
  case 234:jp_pe();break;
  case 235:ex_de_hl();break;
  case 236:call_pe();break;
  case 237:ed();break;
  case 238:xor_byte();break;
  case 239:rst_28();break;
  case 240:ret_p();break;
  case 241:pop_af();break;
  case 242:jp_p();break;
  case 243:di();break;
  case 244:call_p();break;
  case 245:push_af();break;
  case 246:or_byte();break;
  case 247:rst_30();break;
  case 248:ret_m();break;
  case 249:ld_sp_hl();break;
  case 250:jp_m();break;
  case 251:ei();break;
  case 252:call_m();break;
  case 253:fd();break;
  case 254:cp_byte();break;
  case 255:rst_38();break;

  default:    printf("Z80::call_opcode_main(%d):Invalidcode!\n",aCode);
    break;
  }
}

/* Get next opcode argument and increment program counter */
byte Z80::Z80_RDMEM_OPCODE (void) {
  byte retval;
  retval=Z80_RDMEM(R.PC.W.l);
  R.PC.W.l++;
  return retval;
}

word Z80::Z80_RDMEM_WORD (word A) {
  word lsb, msb;
  word retval;
  lsb=(word) Z80_RDMEM(A);
  msb=(word) Z80_RDMEM(A+1);
  retval = lsb + (msb <<8);

  return retval;
}

void Z80::Z80_WRMEM_WORD (word A,word V) {
 Z80_WRMEM (A,V&255);
 Z80_WRMEM ((word)(A+1),V>>8);
}

word Z80::Z80_RDMEM_OPCODE_WORD (void) {
 word i;
 i=(word) Z80_RDMEM_OPCODE();
 i+=(word) Z80_RDMEM_OPCODE()<<8;
 return i;
}

byte Z80::M_RD_XIX(void) {
  word i;
  i=M_XIX;
  return Z80_RDMEM(i);
}

byte Z80::M_RD_XIY(void) {
  word i;
  i=M_XIY;
  return Z80_RDMEM(i);
}

void Z80::M_WR_XIX(byte a) {
  word i;
  i=M_XIX;
  Z80_WRMEM(i,a);
}

void Z80::M_WR_XIY(byte a) {
  word i;
  i=M_XIY;
  Z80_WRMEM(i,a);
}

void Z80::adc_a_xhl(void) { byte i=M_RD_XHL; M_ADC(i); }
void Z80::adc_a_xix(void) { byte i=M_RD_XIX(); M_ADC(i); }
void Z80::adc_a_xiy(void) { byte i=M_RD_XIY(); M_ADC(i); }
void Z80::adc_a_a(void) { M_ADC(R.AF.B.h); }
void Z80::adc_a_b(void) { M_ADC(R.BC.B.h); }
void Z80::adc_a_c(void) { M_ADC(R.BC.B.l); }
void Z80::adc_a_d(void) { M_ADC(R.DE.B.h); }
void Z80::adc_a_e(void) { M_ADC(R.DE.B.l); }
void Z80::adc_a_h(void) { M_ADC(R.HL.B.h); }
void Z80::adc_a_l(void) { M_ADC(R.HL.B.l); }
void Z80::adc_a_ixl(void) { M_ADC(R.IX.B.l); }
void Z80::adc_a_ixh(void) { M_ADC(R.IX.B.h); }
void Z80::adc_a_iyl(void) { M_ADC(R.IY.B.l); }
void Z80::adc_a_iyh(void) { M_ADC(R.IY.B.h); }
void Z80::adc_a_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_ADC(i); }

void Z80::adc_hl_bc(void) { M_ADCW(BC); }
void Z80::adc_hl_de(void) { M_ADCW(DE); }
void Z80::adc_hl_hl(void) { M_ADCW(HL); }
void Z80::adc_hl_sp(void) { M_ADCW(SP); }

void Z80::add_a_xhl(void) { byte i=M_RD_XHL; M_ADD(i); }
void Z80::add_a_xix(void) { byte i=M_RD_XIX(); M_ADD(i); }
void Z80::add_a_xiy(void) { byte i=M_RD_XIY(); M_ADD(i); }
void Z80::add_a_a(void) { M_ADD(R.AF.B.h); }
void Z80::add_a_b(void) { M_ADD(R.BC.B.h); }
void Z80::add_a_c(void) { M_ADD(R.BC.B.l); }
void Z80::add_a_d(void) { M_ADD(R.DE.B.h); }
void Z80::add_a_e(void) { M_ADD(R.DE.B.l); }
void Z80::add_a_h(void) { M_ADD(R.HL.B.h); }
void Z80::add_a_l(void) { M_ADD(R.HL.B.l); }
void Z80::add_a_ixl(void) { M_ADD(R.IX.B.l); }
void Z80::add_a_ixh(void) { M_ADD(R.IX.B.h); }
void Z80::add_a_iyl(void) { M_ADD(R.IY.B.l); }
void Z80::add_a_iyh(void) { M_ADD(R.IY.B.h); }
void Z80::add_a_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_ADD(i); }

void Z80::add_hl_bc(void) { M_ADDW(HL,BC); }
void Z80::add_hl_de(void) { M_ADDW(HL,DE); }
void Z80::add_hl_hl(void) { M_ADDW(HL,HL); }
void Z80::add_hl_sp(void) { M_ADDW(HL,SP); }
void Z80::add_ix_bc(void) { M_ADDW(IX,BC); }
void Z80::add_ix_de(void) { M_ADDW(IX,DE); }
void Z80::add_ix_ix(void) { M_ADDW(IX,IX); }
void Z80::add_ix_sp(void) { M_ADDW(IX,SP); }
void Z80::add_iy_bc(void) { M_ADDW(IY,BC); }
void Z80::add_iy_de(void) { M_ADDW(IY,DE); }
void Z80::add_iy_iy(void) { M_ADDW(IY,IY); }
void Z80::add_iy_sp(void) { M_ADDW(IY,SP); }

void Z80::and_xhl(void) { byte i=M_RD_XHL; M_AND(i); }
void Z80::and_xix(void) { byte i=M_RD_XIX(); M_AND(i); }
void Z80::and_xiy(void) { byte i=M_RD_XIY(); M_AND(i); }
void Z80::and_a(void) { R.AF.B.l=ZSPTable[R.AF.B.h]|H_FLAG; }
void Z80::and_b(void) { M_AND(R.BC.B.h); }
void Z80::and_c(void) { M_AND(R.BC.B.l); }
void Z80::and_d(void) { M_AND(R.DE.B.h); }
void Z80::and_e(void) { M_AND(R.DE.B.l); }
void Z80::and_h(void) { M_AND(R.HL.B.h); }
void Z80::and_l(void) { M_AND(R.HL.B.l); }
void Z80::and_ixh(void) { M_AND(R.IX.B.h); }
void Z80::and_ixl(void) { M_AND(R.IX.B.l); }
void Z80::and_iyh(void) { M_AND(R.IY.B.h); }
void Z80::and_iyl(void) { M_AND(R.IY.B.l); }
void Z80::and_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_AND(i); }

void Z80::bit_0_xhl(void) { byte i=M_RD_XHL; M_BIT(0,i); }
void Z80::bit_0_xix(void) { byte i=M_RD_XIX(); M_BIT(0,i); }
void Z80::bit_0_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(0,R.BC.B.h); }
void Z80::bit_0_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(0,R.BC.B.l); }
void Z80::bit_0_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(0,R.DE.B.h); }
void Z80::bit_0_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(0,R.DE.B.l); }
void Z80::bit_0_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(0,R.HL.B.h); }
void Z80::bit_0_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(0,R.HL.B.l); }
void Z80::bit_0_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(0,R.AF.B.h); }
void Z80::bit_0_xiy(void) { byte i=M_RD_XIY(); M_BIT(0,i); }
void Z80::bit_0_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(0,R.BC.B.h); }
void Z80::bit_0_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(0,R.BC.B.l); }
void Z80::bit_0_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(0,R.DE.B.h); }
void Z80::bit_0_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(0,R.DE.B.l); }
void Z80::bit_0_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(0,R.HL.B.h); }
void Z80::bit_0_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(0,R.HL.B.l); }
void Z80::bit_0_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(0,R.AF.B.h); }
void Z80::bit_0_a(void) { M_BIT(0,R.AF.B.h); }
void Z80::bit_0_b(void) { M_BIT(0,R.BC.B.h); }
void Z80::bit_0_c(void) { M_BIT(0,R.BC.B.l); }
void Z80::bit_0_d(void) { M_BIT(0,R.DE.B.h); }
void Z80::bit_0_e(void) { M_BIT(0,R.DE.B.l); }
void Z80::bit_0_h(void) { M_BIT(0,R.HL.B.h); }
void Z80::bit_0_l(void) { M_BIT(0,R.HL.B.l); }

void Z80::bit_1_xhl(void) { byte i=M_RD_XHL; M_BIT(1,i); }
void Z80::bit_1_xix(void) { byte i=M_RD_XIX(); M_BIT(1,i); }
void Z80::bit_1_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(1,R.BC.B.h); }
void Z80::bit_1_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(1,R.BC.B.l); }
void Z80::bit_1_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(1,R.DE.B.h); }
void Z80::bit_1_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(1,R.DE.B.l); }
void Z80::bit_1_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(1,R.HL.B.h); }
void Z80::bit_1_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(1,R.HL.B.l); }
void Z80::bit_1_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(1,R.AF.B.h); }
void Z80::bit_1_xiy(void) { byte i=M_RD_XIY(); M_BIT(1,i); }
void Z80::bit_1_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(1,R.BC.B.h); }
void Z80::bit_1_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(1,R.BC.B.l); }
void Z80::bit_1_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(1,R.DE.B.h); }
void Z80::bit_1_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(1,R.DE.B.l); }
void Z80::bit_1_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(1,R.HL.B.h); }
void Z80::bit_1_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(1,R.HL.B.l); }
void Z80::bit_1_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(1,R.AF.B.h); }
void Z80::bit_1_a(void) { M_BIT(1,R.AF.B.h); }
void Z80::bit_1_b(void) { M_BIT(1,R.BC.B.h); }
void Z80::bit_1_c(void) { M_BIT(1,R.BC.B.l); }
void Z80::bit_1_d(void) { M_BIT(1,R.DE.B.h); }
void Z80::bit_1_e(void) { M_BIT(1,R.DE.B.l); }
void Z80::bit_1_h(void) { M_BIT(1,R.HL.B.h); }
void Z80::bit_1_l(void) { M_BIT(1,R.HL.B.l); }

void Z80::bit_2_xhl(void) { byte i=M_RD_XHL; M_BIT(2,i); }
void Z80::bit_2_xix(void) { byte i=M_RD_XIX(); M_BIT(2,i); }
void Z80::bit_2_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(2,R.BC.B.h); }
void Z80::bit_2_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(2,R.BC.B.l); }
void Z80::bit_2_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(2,R.DE.B.h); }
void Z80::bit_2_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(2,R.DE.B.l); }
void Z80::bit_2_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(2,R.HL.B.h); }
void Z80::bit_2_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(2,R.HL.B.l); }
void Z80::bit_2_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(2,R.AF.B.h); }
void Z80::bit_2_xiy(void) { byte i=M_RD_XIY(); M_BIT(2,i); }
void Z80::bit_2_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(2,R.BC.B.h); }
void Z80::bit_2_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(2,R.BC.B.l); }
void Z80::bit_2_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(2,R.DE.B.h); }
void Z80::bit_2_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(2,R.DE.B.l); }
void Z80::bit_2_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(2,R.HL.B.h); }
void Z80::bit_2_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(2,R.HL.B.l); }
void Z80::bit_2_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(2,R.AF.B.h); }
void Z80::bit_2_a(void) { M_BIT(2,R.AF.B.h); }
void Z80::bit_2_b(void) { M_BIT(2,R.BC.B.h); }
void Z80::bit_2_c(void) { M_BIT(2,R.BC.B.l); }
void Z80::bit_2_d(void) { M_BIT(2,R.DE.B.h); }
void Z80::bit_2_e(void) { M_BIT(2,R.DE.B.l); }
void Z80::bit_2_h(void) { M_BIT(2,R.HL.B.h); }
void Z80::bit_2_l(void) { M_BIT(2,R.HL.B.l); }

void Z80::bit_3_xhl(void) { byte i=M_RD_XHL; M_BIT(3,i); }
void Z80::bit_3_xix(void) { byte i=M_RD_XIX(); M_BIT(3,i); }
void Z80::bit_3_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(3,R.BC.B.h); }
void Z80::bit_3_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(3,R.BC.B.l); }
void Z80::bit_3_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(3,R.DE.B.h); }
void Z80::bit_3_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(3,R.DE.B.l); }
void Z80::bit_3_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(3,R.HL.B.h); }
void Z80::bit_3_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(3,R.HL.B.l); }
void Z80::bit_3_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(3,R.AF.B.h); }
void Z80::bit_3_xiy(void) { byte i=M_RD_XIY(); M_BIT(3,i); }
void Z80::bit_3_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(3,R.BC.B.h); }
void Z80::bit_3_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(3,R.BC.B.l); }
void Z80::bit_3_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(3,R.DE.B.h); }
void Z80::bit_3_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(3,R.DE.B.l); }
void Z80::bit_3_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(3,R.HL.B.h); }
void Z80::bit_3_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(3,R.HL.B.l); }
void Z80::bit_3_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(3,R.AF.B.h); }
void Z80::bit_3_a(void) { M_BIT(3,R.AF.B.h); }
void Z80::bit_3_b(void) { M_BIT(3,R.BC.B.h); }
void Z80::bit_3_c(void) { M_BIT(3,R.BC.B.l); }
void Z80::bit_3_d(void) { M_BIT(3,R.DE.B.h); }
void Z80::bit_3_e(void) { M_BIT(3,R.DE.B.l); }
void Z80::bit_3_h(void) { M_BIT(3,R.HL.B.h); }
void Z80::bit_3_l(void) { M_BIT(3,R.HL.B.l); }

void Z80::bit_4_xhl(void) { byte i=M_RD_XHL; M_BIT(4,i); }
void Z80::bit_4_xix(void) { byte i=M_RD_XIX(); M_BIT(4,i); }
void Z80::bit_4_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(4,R.BC.B.h); }
void Z80::bit_4_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(4,R.BC.B.l); }
void Z80::bit_4_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(4,R.DE.B.h); }
void Z80::bit_4_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(4,R.DE.B.l); }
void Z80::bit_4_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(4,R.HL.B.h); }
void Z80::bit_4_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(4,R.HL.B.l); }
void Z80::bit_4_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(4,R.AF.B.h); }
void Z80::bit_4_xiy(void) { byte i=M_RD_XIY(); M_BIT(4,i); }
void Z80::bit_4_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(4,R.BC.B.h); }
void Z80::bit_4_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(4,R.BC.B.l); }
void Z80::bit_4_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(4,R.DE.B.h); }
void Z80::bit_4_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(4,R.DE.B.l); }
void Z80::bit_4_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(4,R.HL.B.h); }
void Z80::bit_4_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(4,R.HL.B.l); }
void Z80::bit_4_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(4,R.AF.B.h); }
void Z80::bit_4_a(void) { M_BIT(4,R.AF.B.h); }
void Z80::bit_4_b(void) { M_BIT(4,R.BC.B.h); }
void Z80::bit_4_c(void) { M_BIT(4,R.BC.B.l); }
void Z80::bit_4_d(void) { M_BIT(4,R.DE.B.h); }
void Z80::bit_4_e(void) { M_BIT(4,R.DE.B.l); }
void Z80::bit_4_h(void) { M_BIT(4,R.HL.B.h); }
void Z80::bit_4_l(void) { M_BIT(4,R.HL.B.l); }

void Z80::bit_5_xhl(void) { byte i=M_RD_XHL; M_BIT(5,i); }
void Z80::bit_5_xix(void) { byte i=M_RD_XIX(); M_BIT(5,i); }
void Z80::bit_5_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(5,R.BC.B.h); }
void Z80::bit_5_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(5,R.BC.B.l); }
void Z80::bit_5_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(5,R.DE.B.h); }
void Z80::bit_5_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(5,R.DE.B.l); }
void Z80::bit_5_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(5,R.HL.B.h); }
void Z80::bit_5_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(5,R.HL.B.l); }
void Z80::bit_5_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(5,R.AF.B.h); }
void Z80::bit_5_xiy(void) { byte i=M_RD_XIY(); M_BIT(5,i); }
void Z80::bit_5_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(5,R.BC.B.h); }
void Z80::bit_5_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(5,R.BC.B.l); }
void Z80::bit_5_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(5,R.DE.B.h); }
void Z80::bit_5_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(5,R.DE.B.l); }
void Z80::bit_5_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(5,R.HL.B.h); }
void Z80::bit_5_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(5,R.HL.B.l); }
void Z80::bit_5_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(5,R.AF.B.h); }
void Z80::bit_5_a(void) { M_BIT(5,R.AF.B.h); }
void Z80::bit_5_b(void) { M_BIT(5,R.BC.B.h); }
void Z80::bit_5_c(void) { M_BIT(5,R.BC.B.l); }
void Z80::bit_5_d(void) { M_BIT(5,R.DE.B.h); }
void Z80::bit_5_e(void) { M_BIT(5,R.DE.B.l); }
void Z80::bit_5_h(void) { M_BIT(5,R.HL.B.h); }
void Z80::bit_5_l(void) { M_BIT(5,R.HL.B.l); }

void Z80::bit_6_xhl(void) { byte i=M_RD_XHL; M_BIT(6,i); }
void Z80::bit_6_xix(void) { byte i=M_RD_XIX(); M_BIT(6,i); }
void Z80::bit_6_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(6,R.BC.B.h); }
void Z80::bit_6_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(6,R.BC.B.l); }
void Z80::bit_6_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(6,R.DE.B.h); }
void Z80::bit_6_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(6,R.DE.B.l); }
void Z80::bit_6_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(6,R.HL.B.h); }
void Z80::bit_6_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(6,R.HL.B.l); }
void Z80::bit_6_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(6,R.AF.B.h); }
void Z80::bit_6_xiy(void) { byte i=M_RD_XIY(); M_BIT(6,i); }
void Z80::bit_6_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(6,R.BC.B.h); }
void Z80::bit_6_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(6,R.BC.B.l); }
void Z80::bit_6_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(6,R.DE.B.h); }
void Z80::bit_6_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(6,R.DE.B.l); }
void Z80::bit_6_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(6,R.HL.B.h); }
void Z80::bit_6_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(6,R.HL.B.l); }
void Z80::bit_6_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(6,R.AF.B.h); }
void Z80::bit_6_a(void) { M_BIT(6,R.AF.B.h); }
void Z80::bit_6_b(void) { M_BIT(6,R.BC.B.h); }
void Z80::bit_6_c(void) { M_BIT(6,R.BC.B.l); }
void Z80::bit_6_d(void) { M_BIT(6,R.DE.B.h); }
void Z80::bit_6_e(void) { M_BIT(6,R.DE.B.l); }
void Z80::bit_6_h(void) { M_BIT(6,R.HL.B.h); }
void Z80::bit_6_l(void) { M_BIT(6,R.HL.B.l); }

void Z80::bit_7_xhl(void) { byte i=M_RD_XHL; M_BIT(7,i); }
void Z80::bit_7_xix(void) { byte i=M_RD_XIX(); M_BIT(7,i); }
void Z80::bit_7_xix_b(void) { R.BC.B.h=M_RD_XIX(); M_BIT(7,R.BC.B.h); }
void Z80::bit_7_xix_c(void) { R.BC.B.l=M_RD_XIX(); M_BIT(7,R.BC.B.l); }
void Z80::bit_7_xix_d(void) { R.DE.B.h=M_RD_XIX(); M_BIT(7,R.DE.B.h); }
void Z80::bit_7_xix_e(void) { R.DE.B.l=M_RD_XIX(); M_BIT(7,R.DE.B.l); }
void Z80::bit_7_xix_h(void) { R.HL.B.h=M_RD_XIX(); M_BIT(7,R.HL.B.h); }
void Z80::bit_7_xix_l(void) { R.HL.B.l=M_RD_XIX(); M_BIT(7,R.HL.B.l); }
void Z80::bit_7_xix_a(void) { R.AF.B.h=M_RD_XIX(); M_BIT(7,R.AF.B.h); }
void Z80::bit_7_xiy(void) { byte i=M_RD_XIY(); M_BIT(7,i); }
void Z80::bit_7_xiy_b(void) { R.BC.B.h=M_RD_XIY(); M_BIT(7,R.BC.B.h); }
void Z80::bit_7_xiy_c(void) { R.BC.B.l=M_RD_XIY(); M_BIT(7,R.BC.B.l); }
void Z80::bit_7_xiy_d(void) { R.DE.B.h=M_RD_XIY(); M_BIT(7,R.DE.B.h); }
void Z80::bit_7_xiy_e(void) { R.DE.B.l=M_RD_XIY(); M_BIT(7,R.DE.B.l); }
void Z80::bit_7_xiy_h(void) { R.HL.B.h=M_RD_XIY(); M_BIT(7,R.HL.B.h); }
void Z80::bit_7_xiy_l(void) { R.HL.B.l=M_RD_XIY(); M_BIT(7,R.HL.B.l); }
void Z80::bit_7_xiy_a(void) { R.AF.B.h=M_RD_XIY(); M_BIT(7,R.AF.B.h); }
void Z80::bit_7_a(void) { M_BIT(7,R.AF.B.h); }
void Z80::bit_7_b(void) { M_BIT(7,R.BC.B.h); }
void Z80::bit_7_c(void) { M_BIT(7,R.BC.B.l); }
void Z80::bit_7_d(void) { M_BIT(7,R.DE.B.h); }
void Z80::bit_7_e(void) { M_BIT(7,R.DE.B.l); }
void Z80::bit_7_h(void) { M_BIT(7,R.HL.B.h); }
void Z80::bit_7_l(void) { M_BIT(7,R.HL.B.l); }

void Z80::call_c(void) { if (M_C) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_m(void) { if (M_M) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_nc(void) { if (M_NC) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_nz(void) { if (M_NZ) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_p(void) { if (M_P) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_pe(void) { if (M_PE) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_po(void) { if (M_PO) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call_z(void) { if (M_Z) { M_CALL; } else { M_SKIP_CALL; } }
void Z80::call(void) { M_CALL; }

void Z80::ccf(void) { R.AF.B.l=((R.AF.B.l&0xED)|((R.AF.B.l&1)<<4))^1; }

void Z80::cp_xhl(void) { byte i=M_RD_XHL; M_CP(i); }
void Z80::cp_xix(void) { byte i=M_RD_XIX(); M_CP(i); }
void Z80::cp_xiy(void) { byte i=M_RD_XIY(); M_CP(i); }
void Z80::cp_a(void) { M_CP(R.AF.B.h); }
void Z80::cp_b(void) { M_CP(R.BC.B.h); }
void Z80::cp_c(void) { M_CP(R.BC.B.l); }
void Z80::cp_d(void) { M_CP(R.DE.B.h); }
void Z80::cp_e(void) { M_CP(R.DE.B.l); }
void Z80::cp_h(void) { M_CP(R.HL.B.h); }
void Z80::cp_l(void) { M_CP(R.HL.B.l); }
void Z80::cp_ixh(void) { M_CP(R.IX.B.h); }
void Z80::cp_ixl(void) { M_CP(R.IX.B.l); }
void Z80::cp_iyh(void) { M_CP(R.IY.B.h); }
void Z80::cp_iyl(void) { M_CP(R.IY.B.l); }
void Z80::cp_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_CP(i); }

void Z80::cpd(void)
{
 byte i,j;
 i=Z80_RDMEM(R.HL.D); // TO DO
 j=R.AF.B.h-i;
 --R.HL.W.l;
 --R.BC.W.l;
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSTable[j]|
   ((R.AF.B.h^i^j)&H_FLAG)|(R.BC.D? V_FLAG:0)|N_FLAG; // TO DO
}

void Z80::cpdr(void)
{
 cpd ();
 if (R.BC.D && !(R.AF.B.l&Z_FLAG)) { Z80_ICount-=5; R.PC.W.l-=2; }// TO DO
}

void Z80::cpi(void)
{
 byte i,j;
 i=Z80_RDMEM(R.HL.D);// TO DO
 j=R.AF.B.h-i;
 ++R.HL.W.l;
 --R.BC.W.l;
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSTable[j]|
          ((R.AF.B.h^i^j)&H_FLAG)|(R.BC.D? V_FLAG:0)|N_FLAG;// TO DO
}

void Z80::cpir(void)
{
 cpi ();
 if (R.BC.D && !(R.AF.B.l&Z_FLAG)) { Z80_ICount-=5; R.PC.W.l-=2; }// TO DO
}

void Z80::cpl(void) { R.AF.B.h^=0xFF; R.AF.B.l|=(H_FLAG|N_FLAG); }

void Z80::daa (void) {	
  byte cf, nf, hf, lo, hi, diff;	
  cf = R.AF.B.l & C_FLAG;
  nf = R.AF.B.l & N_FLAG;
  hf = R.AF.B.l & H_FLAG;
  lo = R.AF.B.h & 15;
  hi = R.AF.B.h / 16;
  if (cf) {
    diff = (lo <= 9 && !hf) ? 0x60 : 0x66; 
  }
  else {
    if (lo >= 10) {
      diff = hi <= 8 ? 0x06 : 0x66;
    }
    else {
      if (hi >= 10) {
	diff = hf ? 0x66 : 0x60;
      }
      else {
	diff = hf ? 0x06 : 0x00;
      }
    }
  }
  if (nf) R.AF.B.h -= diff;
  else R.AF.B.h += diff;
  
  R.AF.B.l = ZSPTable[R.AF.B.h] | (R.AF.B.l & N_FLAG);
  if (cf || (lo <= 9 ? hi >= 10 : hi >= 9)) R.AF.B.l |= C_FLAG;
  if (nf ? hf && lo <= 5 : lo >= 10)	R.AF.B.l |= H_FLAG;
}

void Z80::dec_xhl(void)
{
 byte i;
 i=Z80_RDMEM(R.HL.D);// TO DO
 M_DEC(i);
 Z80_WRMEM(R.HL.D,i);// TO DO
}
void Z80::dec_xix(void)
{
 byte i;
 int j;
 j=M_XIX;
 i=Z80_RDMEM(j);
 M_DEC(i);
 Z80_WRMEM(j,i);
}
void Z80::dec_xiy(void)
{
 byte i;
 int j;
 j=M_XIY;
 i=Z80_RDMEM(j);
 M_DEC(i);
 Z80_WRMEM(j,i);
}
void Z80::dec_a(void) { M_DEC(R.AF.B.h); }
void Z80::dec_b(void) { M_DEC(R.BC.B.h); }
void Z80::dec_c(void) { M_DEC(R.BC.B.l); }
void Z80::dec_d(void) { M_DEC(R.DE.B.h); }
void Z80::dec_e(void) { M_DEC(R.DE.B.l); }
void Z80::dec_h(void) { M_DEC(R.HL.B.h); }
void Z80::dec_l(void) { M_DEC(R.HL.B.l); }
void Z80::dec_ixh(void) { M_DEC(R.IX.B.h); }
void Z80::dec_ixl(void) { M_DEC(R.IX.B.l); }
void Z80::dec_iyh(void) { M_DEC(R.IY.B.h); }
void Z80::dec_iyl(void) { M_DEC(R.IY.B.l); }

void Z80::dec_bc(void) { --R.BC.W.l; }
void Z80::dec_de(void) { --R.DE.W.l; }
void Z80::dec_hl(void) { --R.HL.W.l; }
void Z80::dec_ix(void) { --R.IX.W.l; }
void Z80::dec_iy(void) { --R.IY.W.l; }
void Z80::dec_sp(void) { --R.SP.W.l; }

void Z80::di(void) { R.IFF1=R.IFF2=0; }

void Z80::djnz(void) { if (--R.BC.B.h) { M_JR; } else { M_SKIP_JR; } }

void Z80::ex_xsp_hl(void)
{
 int i;
 i=Z80_RDMEM_WORD(R.SP.D);// TO DO
 Z80_WRMEM_WORD(R.SP.D,R.HL.D);// TO DO
 R.HL.D=i;// TO DO
}

void Z80::ex_xsp_ix(void)
{
 int i;
 i=Z80_RDMEM_WORD(R.SP.D);// TO DO
 Z80_WRMEM_WORD(R.SP.D,R.IX.D);// TO DO
 R.IX.D=i;// TO DO
}

void Z80::ex_xsp_iy(void)
{
 int i;
 i=Z80_RDMEM_WORD(R.SP.D);// TO DO
 Z80_WRMEM_WORD(R.SP.D,R.IY.D);
 R.IY.D=i;
}

void Z80::ex_af_af(void)
{
 int i;
 i=R.AF.D;// TO DO
 R.AF.D=R.AF2.D;// TO DO
 R.AF2.D=i;// TO DO
}

void Z80::ex_de_hl(void)
{
 int i;
 i=R.DE.D;// TO DO
 R.DE.D=R.HL.D;// TO DO
 R.HL.D=i;// TO DO
}

void Z80::exx(void)
{
 int i;
 i=R.BC.D;// TO DO
 R.BC.D=R.BC2.D;// TO DO
 R.BC2.D=i;// TO DO
 i=R.DE.D;// TO DO
 R.DE.D=R.DE2.D;// TO DO
 R.DE2.D=i;// TO DO
 i=R.HL.D;// TO DO
 R.HL.D=R.HL2.D;// TO DO
 R.HL2.D=i;// TO DO
}

void Z80::halt(void)
{
 --R.PC.W.l;
 R.HALT=1;
 if (Z80_ICount>0) Z80_ICount=0;
}

void Z80::im_0(void) { R.IM=0; }
void Z80::im_1(void) { R.IM=1; }
void Z80::im_2(void) { R.IM=2; }

void Z80::in_a_c(void) { M_IN(R.AF.B.h); }
void Z80::in_b_c(void) { M_IN(R.BC.B.h); }
void Z80::in_c_c(void) { M_IN(R.BC.B.l); }
void Z80::in_d_c(void) { M_IN(R.DE.B.h); }
void Z80::in_e_c(void) { M_IN(R.DE.B.l); }
void Z80::in_h_c(void) { M_IN(R.HL.B.h); }
void Z80::in_l_c(void) { M_IN(R.HL.B.l); }
void Z80::in_0_c(void) { byte i; M_IN(i); }

void Z80::in_a_byte(void)
{
 byte i=Z80_RDMEM_OPCODE();
 R.AF.B.h=DoIn(i,R.AF.B.h);
}

void Z80::inc_xhl(void)
{
 byte i;
 i=Z80_RDMEM(R.HL.D);
 M_INC(i);
 Z80_WRMEM(R.HL.D,i);
}
void Z80::inc_xix(void)
{
 byte i;
 int j;
 j=M_XIX;
 i=Z80_RDMEM(j);
 M_INC(i);
 Z80_WRMEM(j,i);
}
void Z80::inc_xiy(void)
{
 byte i;
 int j;
 j=M_XIY;
 i=Z80_RDMEM(j);
 M_INC(i);
 Z80_WRMEM(j,i);
}
void Z80::inc_a(void) { M_INC(R.AF.B.h); }
void Z80::inc_b(void) { M_INC(R.BC.B.h); }
void Z80::inc_c(void) { M_INC(R.BC.B.l); }
void Z80::inc_d(void) { M_INC(R.DE.B.h); }
void Z80::inc_e(void) { M_INC(R.DE.B.l); }
void Z80::inc_h(void) { M_INC(R.HL.B.h); }
void Z80::inc_l(void) { M_INC(R.HL.B.l); }
void Z80::inc_ixh(void) { M_INC(R.IX.B.h); }
void Z80::inc_ixl(void) { M_INC(R.IX.B.l); }
void Z80::inc_iyh(void) { M_INC(R.IY.B.h); }
void Z80::inc_iyl(void) { M_INC(R.IY.B.l); }

void Z80::inc_bc(void) { ++R.BC.W.l; }
void Z80::inc_de(void) { ++R.DE.W.l; }
void Z80::inc_hl(void) { ++R.HL.W.l; }
void Z80::inc_ix(void) { ++R.IX.W.l; }
void Z80::inc_iy(void) { ++R.IY.W.l; }
void Z80::inc_sp(void) { ++R.SP.W.l; }

void Z80::ind(void)
{
 Z80_WRMEM(R.HL.D,DoIn(R.BC.B.l,R.BC.B.h));
 --R.BC.B.h;
 --R.HL.W.l;
 R.AF.B.l=(R.BC.B.h)? N_FLAG:(N_FLAG|Z_FLAG);
}

void Z80::indr(void)
{
 ind ();
 if (R.BC.B.h) { Z80_ICount-=5; R.PC.W.l-=2; }
}

void Z80::ini(void)
{
 Z80_WRMEM(R.HL.D,DoIn(R.BC.B.l,R.BC.B.h));// TO DO
 --R.BC.B.h;
 ++R.HL.W.l;
 R.AF.B.l=(R.BC.B.h)? N_FLAG:(N_FLAG|Z_FLAG);
}

void Z80::inir(void)
{
 ini ();
 if (R.BC.B.h) { Z80_ICount-=5; R.PC.W.l-=2; }
}

/*void Z80::jp(void) { M_JP; } -NS- changed to speed up busy loops */
void Z80::jp(void)
{
  dword oldpc,newpc;

  oldpc = R.PC.D-1;// TO DO
  M_JP;
  newpc = R.PC.D;
  if (newpc == oldpc) { if (Z80_ICount > 0) Z80_ICount = 0; } /* speed up busy loop */
  else if (newpc == oldpc-1 && Z80_RDMEM(newpc) == 0x00) /* NOP - JP */
    { if (Z80_ICount > 0) Z80_ICount = 0; }
  else if (newpc == oldpc-3 && Z80_RDMEM(newpc) == 0x31) /* LD SP,#xxxx - Galaga */
  { if (Z80_ICount > 10) Z80_ICount = 10; }
  else if (newpc == oldpc-1 && Z80_RDMEM(newpc) == 0xfb && R.pending_irq == Z80_IGNORE_INT)	/* EI - JP */
	  { if (Z80_ICount > 4) Z80_ICount = 4; }
}
void Z80::jp_hl(void) { R.PC.D=R.HL.D; }
void Z80::jp_ix(void) { R.PC.D=R.IX.D; }
void Z80::jp_iy(void) { R.PC.D=R.IY.D; }
void Z80::jp_c(void) { if (M_C) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_m(void) { if (M_M) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_nc(void) { if (M_NC) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_nz(void) { if (M_NZ) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_p(void) { if (M_P) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_pe(void) { if (M_PE) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_po(void) { if (M_PO) { M_JP; } else { M_SKIP_JP; } }
void Z80::jp_z(void) { if (M_Z) { M_JP; } else { M_SKIP_JP; } }

/*void Z80::jr(void) { M_JR; }	-NS- changed to speed up busy loops */
void Z80::jr(void)
{
  dword oldpc,newpc;

  oldpc = R.PC.D-1;
  M_JR;
  newpc = R.PC.D;
  if (newpc == oldpc) { if (Z80_ICount > 0) Z80_ICount = 0; } /* speed up busy loop */
  else if (newpc == oldpc-1 && Z80_RDMEM(newpc) == 0xfb && R.pending_irq == Z80_IGNORE_INT)	/* EI - 1942 */
	  { if (Z80_ICount > 4) Z80_ICount = 4; }
}
void Z80::jr_c(void) { if (M_C) { M_JR; } else { M_SKIP_JR; } }
void Z80::jr_nc(void) { if (M_NC) { M_JR; } else { M_SKIP_JR; } }
void Z80::jr_nz(void) { if (M_NZ) { M_JR; } else { M_SKIP_JR; } }
void Z80::jr_z(void) { if (M_Z) { M_JR; } else { M_SKIP_JR; } }

void Z80::ld_xbc_a(void) { Z80_WRMEM(R.BC.D,R.AF.B.h); }
void Z80::ld_xde_a(void) { Z80_WRMEM(R.DE.D,R.AF.B.h); }
void Z80::ld_xhl_a(void) { Z80_WRMEM(R.HL.D,R.AF.B.h); }
void Z80::ld_xhl_b(void) { Z80_WRMEM(R.HL.D,R.BC.B.h); }
void Z80::ld_xhl_c(void) { Z80_WRMEM(R.HL.D,R.BC.B.l); }
void Z80::ld_xhl_d(void) { Z80_WRMEM(R.HL.D,R.DE.B.h); }
void Z80::ld_xhl_e(void) { Z80_WRMEM(R.HL.D,R.DE.B.l); }
void Z80::ld_xhl_h(void) { Z80_WRMEM(R.HL.D,R.HL.B.h); }
void Z80::ld_xhl_l(void) { Z80_WRMEM(R.HL.D,R.HL.B.l); }
void Z80::ld_xhl_byte(void) { byte i=Z80_RDMEM_OPCODE(); Z80_WRMEM(R.HL.D,i); }
void Z80::ld_xix_a(void) { M_WR_XIX(R.AF.B.h); }
void Z80::ld_xix_b(void) { M_WR_XIX(R.BC.B.h); }
void Z80::ld_xix_c(void) { M_WR_XIX(R.BC.B.l); }
void Z80::ld_xix_d(void) { M_WR_XIX(R.DE.B.h); }
void Z80::ld_xix_e(void) { M_WR_XIX(R.DE.B.l); }
void Z80::ld_xix_h(void) { M_WR_XIX(R.HL.B.h); }
void Z80::ld_xix_l(void) { M_WR_XIX(R.HL.B.l); }
void Z80::ld_xix_byte(void)
{
 int i,j;
 i=M_XIX;
 j=Z80_RDMEM_OPCODE();
 Z80_WRMEM(i,j);
}
void Z80::ld_xiy_a(void) { M_WR_XIY(R.AF.B.h); }
void Z80::ld_xiy_b(void) { M_WR_XIY(R.BC.B.h); }
void Z80::ld_xiy_c(void) { M_WR_XIY(R.BC.B.l); }
void Z80::ld_xiy_d(void) { M_WR_XIY(R.DE.B.h); }
void Z80::ld_xiy_e(void) { M_WR_XIY(R.DE.B.l); }
void Z80::ld_xiy_h(void) { M_WR_XIY(R.HL.B.h); }
void Z80::ld_xiy_l(void) { M_WR_XIY(R.HL.B.l); }
void Z80::ld_xiy_byte(void)
{
 int i,j;
 i=M_XIY;
 j=Z80_RDMEM_OPCODE();
 Z80_WRMEM(i,j);
}
void Z80::ld_xbyte_a(void)
{ int i=Z80_RDMEM_OPCODE_WORD(); Z80_WRMEM(i,R.AF.B.h); }
void Z80::ld_xword_bc(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.BC.D); }
void Z80::ld_xword_de(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.DE.D); }
void Z80::ld_xword_hl(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.HL.D); }
void Z80::ld_xword_ix(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.IX.D); }
void Z80::ld_xword_iy(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.IY.D); }
void Z80::ld_xword_sp(void) { Z80_WRMEM_WORD(Z80_RDMEM_OPCODE_WORD(),R.SP.D); }
void Z80::ld_a_xbc(void) { R.AF.B.h=Z80_RDMEM(R.BC.D); }
void Z80::ld_a_xde(void) { R.AF.B.h=Z80_RDMEM(R.DE.D); }
void Z80::ld_a_xhl(void) { R.AF.B.h=M_RD_XHL; }
void Z80::ld_a_xix(void) { R.AF.B.h=M_RD_XIX(); }
void Z80::ld_a_xiy(void) { R.AF.B.h=M_RD_XIY(); }
void Z80::ld_a_xbyte(void)
{ int i=Z80_RDMEM_OPCODE_WORD(); R.AF.B.h=Z80_RDMEM(i); }

void Z80::ld_a_byte(void) { R.AF.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_b_byte(void) { R.BC.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_c_byte(void) { R.BC.B.l=Z80_RDMEM_OPCODE(); }
void Z80::ld_d_byte(void) { R.DE.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_e_byte(void) { R.DE.B.l=Z80_RDMEM_OPCODE(); }
void Z80::ld_h_byte(void) { R.HL.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_l_byte(void) { R.HL.B.l=Z80_RDMEM_OPCODE(); }
void Z80::ld_ixh_byte(void) { R.IX.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_ixl_byte(void) { R.IX.B.l=Z80_RDMEM_OPCODE(); }
void Z80::ld_iyh_byte(void) { R.IY.B.h=Z80_RDMEM_OPCODE(); }
void Z80::ld_iyl_byte(void) { R.IY.B.l=Z80_RDMEM_OPCODE(); }

void Z80::ld_b_xhl(void) { R.BC.B.h=M_RD_XHL; }
void Z80::ld_c_xhl(void) { R.BC.B.l=M_RD_XHL; }
void Z80::ld_d_xhl(void) { R.DE.B.h=M_RD_XHL; }
void Z80::ld_e_xhl(void) { R.DE.B.l=M_RD_XHL; }
void Z80::ld_h_xhl(void) { R.HL.B.h=M_RD_XHL; }
void Z80::ld_l_xhl(void) { R.HL.B.l=M_RD_XHL; }
void Z80::ld_b_xix(void) { R.BC.B.h=M_RD_XIX(); }
void Z80::ld_c_xix(void) { R.BC.B.l=M_RD_XIX(); }
void Z80::ld_d_xix(void) { R.DE.B.h=M_RD_XIX(); }
void Z80::ld_e_xix(void) { R.DE.B.l=M_RD_XIX(); }
void Z80::ld_h_xix(void) { R.HL.B.h=M_RD_XIX(); }
void Z80::ld_l_xix(void) { R.HL.B.l=M_RD_XIX(); }
void Z80::ld_b_xiy(void) { R.BC.B.h=M_RD_XIY(); }
void Z80::ld_c_xiy(void) { R.BC.B.l=M_RD_XIY(); }
void Z80::ld_d_xiy(void) { R.DE.B.h=M_RD_XIY(); }
void Z80::ld_e_xiy(void) { R.DE.B.l=M_RD_XIY(); }
void Z80::ld_h_xiy(void) { R.HL.B.h=M_RD_XIY(); }
void Z80::ld_l_xiy(void) { R.HL.B.l=M_RD_XIY(); }
void Z80::ld_a_a(void) { }
void Z80::ld_a_b(void) { R.AF.B.h=R.BC.B.h; }
void Z80::ld_a_c(void) { R.AF.B.h=R.BC.B.l; }
void Z80::ld_a_d(void) { R.AF.B.h=R.DE.B.h; }
void Z80::ld_a_e(void) { R.AF.B.h=R.DE.B.l; }
void Z80::ld_a_h(void) { R.AF.B.h=R.HL.B.h; }
void Z80::ld_a_l(void) { R.AF.B.h=R.HL.B.l; }
void Z80::ld_a_ixh(void) { R.AF.B.h=R.IX.B.h; }
void Z80::ld_a_ixl(void) { R.AF.B.h=R.IX.B.l; }
void Z80::ld_a_iyh(void) { R.AF.B.h=R.IY.B.h; }
void Z80::ld_a_iyl(void) { R.AF.B.h=R.IY.B.l; }
void Z80::ld_b_b(void) { }
void Z80::ld_b_a(void) { R.BC.B.h=R.AF.B.h; }
void Z80::ld_b_c(void) { R.BC.B.h=R.BC.B.l; }
void Z80::ld_b_d(void) { R.BC.B.h=R.DE.B.h; }
void Z80::ld_b_e(void) { R.BC.B.h=R.DE.B.l; }
void Z80::ld_b_h(void) { R.BC.B.h=R.HL.B.h; }
void Z80::ld_b_l(void) { R.BC.B.h=R.HL.B.l; }
void Z80::ld_b_ixh(void) { R.BC.B.h=R.IX.B.h; }
void Z80::ld_b_ixl(void) { R.BC.B.h=R.IX.B.l; }
void Z80::ld_b_iyh(void) { R.BC.B.h=R.IY.B.h; }
void Z80::ld_b_iyl(void) { R.BC.B.h=R.IY.B.l; }
void Z80::ld_c_c(void) { }
void Z80::ld_c_a(void) { R.BC.B.l=R.AF.B.h; }
void Z80::ld_c_b(void) { R.BC.B.l=R.BC.B.h; }
void Z80::ld_c_d(void) { R.BC.B.l=R.DE.B.h; }
void Z80::ld_c_e(void) { R.BC.B.l=R.DE.B.l; }
void Z80::ld_c_h(void) { R.BC.B.l=R.HL.B.h; }
void Z80::ld_c_l(void) { R.BC.B.l=R.HL.B.l; }
void Z80::ld_c_ixh(void) { R.BC.B.l=R.IX.B.h; }
void Z80::ld_c_ixl(void) { R.BC.B.l=R.IX.B.l; }
void Z80::ld_c_iyh(void) { R.BC.B.l=R.IY.B.h; }
void Z80::ld_c_iyl(void) { R.BC.B.l=R.IY.B.l; }
void Z80::ld_d_d(void) { }
void Z80::ld_d_a(void) { R.DE.B.h=R.AF.B.h; }
void Z80::ld_d_c(void) { R.DE.B.h=R.BC.B.l; }
void Z80::ld_d_b(void) { R.DE.B.h=R.BC.B.h; }
void Z80::ld_d_e(void) { R.DE.B.h=R.DE.B.l; }
void Z80::ld_d_h(void) { R.DE.B.h=R.HL.B.h; }
void Z80::ld_d_l(void) { R.DE.B.h=R.HL.B.l; }
void Z80::ld_d_ixh(void) { R.DE.B.h=R.IX.B.h; }
void Z80::ld_d_ixl(void) { R.DE.B.h=R.IX.B.l; }
void Z80::ld_d_iyh(void) { R.DE.B.h=R.IY.B.h; }
void Z80::ld_d_iyl(void) { R.DE.B.h=R.IY.B.l; }
void Z80::ld_e_e(void) { }
void Z80::ld_e_a(void) { R.DE.B.l=R.AF.B.h; }
void Z80::ld_e_c(void) { R.DE.B.l=R.BC.B.l; }
void Z80::ld_e_b(void) { R.DE.B.l=R.BC.B.h; }
void Z80::ld_e_d(void) { R.DE.B.l=R.DE.B.h; }
void Z80::ld_e_h(void) { R.DE.B.l=R.HL.B.h; }
void Z80::ld_e_l(void) { R.DE.B.l=R.HL.B.l; }
void Z80::ld_e_ixh(void) { R.DE.B.l=R.IX.B.h; }
void Z80::ld_e_ixl(void) { R.DE.B.l=R.IX.B.l; }
void Z80::ld_e_iyh(void) { R.DE.B.l=R.IY.B.h; }
void Z80::ld_e_iyl(void) { R.DE.B.l=R.IY.B.l; }
void Z80::ld_h_h(void) { }
void Z80::ld_h_a(void) { R.HL.B.h=R.AF.B.h; }
void Z80::ld_h_c(void) { R.HL.B.h=R.BC.B.l; }
void Z80::ld_h_b(void) { R.HL.B.h=R.BC.B.h; }
void Z80::ld_h_e(void) { R.HL.B.h=R.DE.B.l; }
void Z80::ld_h_d(void) { R.HL.B.h=R.DE.B.h; }
void Z80::ld_h_l(void) { R.HL.B.h=R.HL.B.l; }
void Z80::ld_l_l(void) { }
void Z80::ld_l_a(void) { R.HL.B.l=R.AF.B.h; }
void Z80::ld_l_c(void) { R.HL.B.l=R.BC.B.l; }
void Z80::ld_l_b(void) { R.HL.B.l=R.BC.B.h; }
void Z80::ld_l_e(void) { R.HL.B.l=R.DE.B.l; }
void Z80::ld_l_d(void) { R.HL.B.l=R.DE.B.h; }
void Z80::ld_l_h(void) { R.HL.B.l=R.HL.B.h; }
void Z80::ld_ixh_a(void) { R.IX.B.h=R.AF.B.h; }
void Z80::ld_ixh_b(void) { R.IX.B.h=R.BC.B.h; }
void Z80::ld_ixh_c(void) { R.IX.B.h=R.BC.B.l; }
void Z80::ld_ixh_d(void) { R.IX.B.h=R.DE.B.h; }
void Z80::ld_ixh_e(void) { R.IX.B.h=R.DE.B.l; }
void Z80::ld_ixh_ixh(void) { }
void Z80::ld_ixh_ixl(void) { R.IX.B.h=R.IX.B.l; }
void Z80::ld_ixl_a(void) { R.IX.B.l=R.AF.B.h; }
void Z80::ld_ixl_b(void) { R.IX.B.l=R.BC.B.h; }
void Z80::ld_ixl_c(void) { R.IX.B.l=R.BC.B.l; }
void Z80::ld_ixl_d(void) { R.IX.B.l=R.DE.B.h; }
void Z80::ld_ixl_e(void) { R.IX.B.l=R.DE.B.l; }
void Z80::ld_ixl_ixh(void) { R.IX.B.l=R.IX.B.h; }
void Z80::ld_ixl_ixl(void) { }
void Z80::ld_iyh_a(void) { R.IY.B.h=R.AF.B.h; }
void Z80::ld_iyh_b(void) { R.IY.B.h=R.BC.B.h; }
void Z80::ld_iyh_c(void) { R.IY.B.h=R.BC.B.l; }
void Z80::ld_iyh_d(void) { R.IY.B.h=R.DE.B.h; }
void Z80::ld_iyh_e(void) { R.IY.B.h=R.DE.B.l; }
void Z80::ld_iyh_iyh(void) { }
void Z80::ld_iyh_iyl(void) { R.IY.B.h=R.IY.B.l; }
void Z80::ld_iyl_a(void) { R.IY.B.l=R.AF.B.h; }
void Z80::ld_iyl_b(void) { R.IY.B.l=R.BC.B.h; }
void Z80::ld_iyl_c(void) { R.IY.B.l=R.BC.B.l; }
void Z80::ld_iyl_d(void) { R.IY.B.l=R.DE.B.h; }
void Z80::ld_iyl_e(void) { R.IY.B.l=R.DE.B.l; }
void Z80::ld_iyl_iyh(void) { R.IY.B.l=R.IY.B.h; }
void Z80::ld_iyl_iyl(void) { }
void Z80::ld_bc_xword(void) { R.BC.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_bc_word(void) { R.BC.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_de_xword(void) { R.DE.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_de_word(void) { R.DE.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_hl_xword(void) { R.HL.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_hl_word(void) { R.HL.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_ix_xword(void) { R.IX.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_ix_word(void) { R.IX.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_iy_xword(void) { R.IY.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_iy_word(void) { R.IY.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_sp_xword(void) { R.SP.D=Z80_RDMEM_WORD(Z80_RDMEM_OPCODE_WORD()); }
void Z80::ld_sp_word(void) { R.SP.D=Z80_RDMEM_OPCODE_WORD(); }
void Z80::ld_sp_hl(void) { R.SP.D=R.HL.D; }
void Z80::ld_sp_ix(void) { R.SP.D=R.IX.D; }
void Z80::ld_sp_iy(void) { R.SP.D=R.IY.D; }
void Z80::ld_a_i(void)
{
 R.AF.B.h=R.I;
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSTable[R.I]|(R.IFF2<<2);
}
void Z80::ld_i_a(void) { R.I=R.AF.B.h; }
void Z80::ld_a_r(void)
{
 R.AF.B.h=(R.R&127)|(R.R2&128);
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSTable[R.AF.B.h]|(R.IFF2<<2);
}
void Z80::ld_r_a(void) { R.R=R.R2=R.AF.B.h; }

void Z80::ldd(void)
{
 Z80_WRMEM(R.DE.D,Z80_RDMEM(R.HL.D));
 --R.DE.W.l;
 --R.HL.W.l;
 --R.BC.W.l;
 R.AF.B.l=(R.AF.B.l&0xE9)|(R.BC.D? V_FLAG:0);
}
void Z80::lddr(void)
{
 ldd ();
 if (R.BC.D) { Z80_ICount-=5; R.PC.W.l-=2; }
}
void Z80::ldi(void)
{
 Z80_WRMEM(R.DE.D,Z80_RDMEM(R.HL.D));
 ++R.DE.W.l;
 ++R.HL.W.l;
 --R.BC.W.l;
 R.AF.B.l=(R.AF.B.l&0xE9)|(R.BC.D? V_FLAG:0);
}
void Z80::ldir(void)
{
 ldi ();
 if (R.BC.D) { Z80_ICount-=5; R.PC.W.l-=2; }
}

void Z80::neg(void)
{
 byte i;
 i=R.AF.B.h;
 R.AF.B.h=0;
 M_SUB(i);
}

void Z80::nop(void) { }

void Z80::or_xhl(void) { byte i=M_RD_XHL; M_OR(i); }
void Z80::or_xix(void) { byte i=M_RD_XIX(); M_OR(i); }
void Z80::or_xiy(void) { byte i=M_RD_XIY(); M_OR(i); }
void Z80::or_a(void) { R.AF.B.l=ZSPTable[R.AF.B.h]; }
void Z80::or_b(void) { M_OR(R.BC.B.h); }
void Z80::or_c(void) { M_OR(R.BC.B.l); }
void Z80::or_d(void) { M_OR(R.DE.B.h); }
void Z80::or_e(void) { M_OR(R.DE.B.l); }
void Z80::or_h(void) { M_OR(R.HL.B.h); }
void Z80::or_l(void) { M_OR(R.HL.B.l); }
void Z80::or_ixh(void) { M_OR(R.IX.B.h); }
void Z80::or_ixl(void) { M_OR(R.IX.B.l); }
void Z80::or_iyh(void) { M_OR(R.IY.B.h); }
void Z80::or_iyl(void) { M_OR(R.IY.B.l); }
void Z80::or_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_OR(i); }

void Z80::outd(void)
{
 DoOut (R.BC.B.l,R.BC.B.h,Z80_RDMEM(R.HL.D));
 --R.BC.B.h;
 --R.HL.W.l;
 R.AF.B.l=(R.BC.B.h)? N_FLAG:(Z_FLAG|N_FLAG);
}
void Z80::otdr(void)
{
 outd ();
 if (R.BC.B.h) { Z80_ICount-=5; R.PC.W.l-=2; }
}
void Z80::outi(void)
{
 DoOut (R.BC.B.l,R.BC.B.h,Z80_RDMEM(R.HL.D));
 --R.BC.B.h;
 ++R.HL.W.l;
 R.AF.B.l=(R.BC.B.h)? N_FLAG:(Z_FLAG|N_FLAG);
}
void Z80::otir(void)
{
 outi ();
 if (R.BC.B.h) { Z80_ICount-=5; R.PC.W.l-=2; }
}

void Z80::out_c_a(void) { DoOut(R.BC.B.l,R.BC.B.h,R.AF.B.h); }
void Z80::out_c_b(void) { DoOut(R.BC.B.l,R.BC.B.h,R.BC.B.h); }
void Z80::out_c_c(void) { DoOut(R.BC.B.l,R.BC.B.h,R.BC.B.l); }
void Z80::out_c_d(void) { DoOut(R.BC.B.l,R.BC.B.h,R.DE.B.h); }
void Z80::out_c_e(void) { DoOut(R.BC.B.l,R.BC.B.h,R.DE.B.l); }
void Z80::out_c_h(void) { DoOut(R.BC.B.l,R.BC.B.h,R.HL.B.h); }
void Z80::out_c_l(void) { DoOut(R.BC.B.l,R.BC.B.h,R.HL.B.l); }
void Z80::out_c_0(void) { DoOut(R.BC.B.l,R.BC.B.h,0); }
void Z80::out_byte_a(void)
{
 byte i=Z80_RDMEM_OPCODE();
 DoOut(i,R.AF.B.h,R.AF.B.h);
}

void Z80::pop_af(void) { M_POP(AF); }
void Z80::pop_bc(void) { M_POP(BC); }
void Z80::pop_de(void) { M_POP(DE); }
void Z80::pop_hl(void) { M_POP(HL); }
void Z80::pop_ix(void) { M_POP(IX); }
void Z80::pop_iy(void) { M_POP(IY); }

void Z80::push_af(void) { M_PUSH(AF); }
void Z80::push_bc(void) { M_PUSH(BC); }
void Z80::push_de(void) { M_PUSH(DE); }
void Z80::push_hl(void) { M_PUSH(HL); }
void Z80::push_ix(void) { M_PUSH(IX); }
void Z80::push_iy(void) { M_PUSH(IY); }

void Z80::res_0_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(0,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_0_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(0,i); Z80_WRMEM(j,i); }
void Z80::res_0_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(0,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_0_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(0,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_0_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(0,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_0_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(0,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_0_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(0,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_0_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(0,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_0_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(0,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_0_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(0,i); Z80_WRMEM(j,i); }
void Z80::res_0_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(0,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_0_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(0,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_0_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(0,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_0_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(0,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_0_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(0,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_0_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(0,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_0_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(0,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_0_a(void) { M_RES(0,R.AF.B.h); }
void Z80::res_0_b(void) { M_RES(0,R.BC.B.h); }
void Z80::res_0_c(void) { M_RES(0,R.BC.B.l); }
void Z80::res_0_d(void) { M_RES(0,R.DE.B.h); }
void Z80::res_0_e(void) { M_RES(0,R.DE.B.l); }
void Z80::res_0_h(void) { M_RES(0,R.HL.B.h); }
void Z80::res_0_l(void) { M_RES(0,R.HL.B.l); }

void Z80::res_1_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(1,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_1_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(1,i); Z80_WRMEM(j,i); }
void Z80::res_1_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(1,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_1_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(1,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_1_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(1,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_1_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(1,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_1_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(1,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_1_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(1,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_1_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(1,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_1_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(1,i); Z80_WRMEM(j,i); }
void Z80::res_1_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(1,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_1_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(1,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_1_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(1,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_1_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(1,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_1_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(1,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_1_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(1,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_1_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(1,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_1_a(void) { M_RES(1,R.AF.B.h); }
void Z80::res_1_b(void) { M_RES(1,R.BC.B.h); }
void Z80::res_1_c(void) { M_RES(1,R.BC.B.l); }
void Z80::res_1_d(void) { M_RES(1,R.DE.B.h); }
void Z80::res_1_e(void) { M_RES(1,R.DE.B.l); }
void Z80::res_1_h(void) { M_RES(1,R.HL.B.h); }
void Z80::res_1_l(void) { M_RES(1,R.HL.B.l); }

void Z80::res_2_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(2,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_2_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(2,i); Z80_WRMEM(j,i); }
void Z80::res_2_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(2,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_2_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(2,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_2_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(2,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_2_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(2,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_2_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(2,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_2_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(2,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_2_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(2,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_2_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(2,i); Z80_WRMEM(j,i); }
void Z80::res_2_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(2,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_2_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(2,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_2_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(2,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_2_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(2,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_2_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(2,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_2_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(2,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_2_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(2,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_2_a(void) { M_RES(2,R.AF.B.h); }
void Z80::res_2_b(void) { M_RES(2,R.BC.B.h); }
void Z80::res_2_c(void) { M_RES(2,R.BC.B.l); }
void Z80::res_2_d(void) { M_RES(2,R.DE.B.h); }
void Z80::res_2_e(void) { M_RES(2,R.DE.B.l); }
void Z80::res_2_h(void) { M_RES(2,R.HL.B.h); }
void Z80::res_2_l(void) { M_RES(2,R.HL.B.l); }

void Z80::res_3_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(3,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_3_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(3,i); Z80_WRMEM(j,i); }
void Z80::res_3_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(3,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_3_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(3,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_3_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(3,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_3_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(3,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_3_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(3,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_3_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(3,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_3_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(3,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_3_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(3,i); Z80_WRMEM(j,i); }
void Z80::res_3_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(3,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_3_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(3,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_3_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(3,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_3_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(3,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_3_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(3,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_3_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(3,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_3_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(3,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_3_a(void) { M_RES(3,R.AF.B.h); }
void Z80::res_3_b(void) { M_RES(3,R.BC.B.h); }
void Z80::res_3_c(void) { M_RES(3,R.BC.B.l); }
void Z80::res_3_d(void) { M_RES(3,R.DE.B.h); }
void Z80::res_3_e(void) { M_RES(3,R.DE.B.l); }
void Z80::res_3_h(void) { M_RES(3,R.HL.B.h); }
void Z80::res_3_l(void) { M_RES(3,R.HL.B.l); }

void Z80::res_4_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(4,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_4_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(4,i); Z80_WRMEM(j,i); }
void Z80::res_4_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(4,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_4_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(4,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_4_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(4,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_4_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(4,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_4_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(4,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_4_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(4,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_4_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(4,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_4_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(4,i); Z80_WRMEM(j,i); }
void Z80::res_4_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(4,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_4_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(4,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_4_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(4,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_4_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(4,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_4_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(4,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_4_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(4,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_4_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(4,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_4_a(void) { M_RES(4,R.AF.B.h); }
void Z80::res_4_b(void) { M_RES(4,R.BC.B.h); }
void Z80::res_4_c(void) { M_RES(4,R.BC.B.l); }
void Z80::res_4_d(void) { M_RES(4,R.DE.B.h); }
void Z80::res_4_e(void) { M_RES(4,R.DE.B.l); }
void Z80::res_4_h(void) { M_RES(4,R.HL.B.h); }
void Z80::res_4_l(void) { M_RES(4,R.HL.B.l); }

void Z80::res_5_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(5,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_5_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(5,i); Z80_WRMEM(j,i); }
void Z80::res_5_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(5,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_5_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(5,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_5_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(5,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_5_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(5,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_5_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(5,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_5_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(5,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_5_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(5,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_5_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(5,i); Z80_WRMEM(j,i); }
void Z80::res_5_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(5,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_5_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(5,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_5_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(5,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_5_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(5,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_5_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(5,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_5_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(5,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_5_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(5,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_5_a(void) { M_RES(5,R.AF.B.h); }
void Z80::res_5_b(void) { M_RES(5,R.BC.B.h); }
void Z80::res_5_c(void) { M_RES(5,R.BC.B.l); }
void Z80::res_5_d(void) { M_RES(5,R.DE.B.h); }
void Z80::res_5_e(void) { M_RES(5,R.DE.B.l); }
void Z80::res_5_h(void) { M_RES(5,R.HL.B.h); }
void Z80::res_5_l(void) { M_RES(5,R.HL.B.l); }

void Z80::res_6_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(6,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_6_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(6,i); Z80_WRMEM(j,i); }
void Z80::res_6_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(6,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_6_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(6,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_6_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(6,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_6_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(6,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_6_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(6,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_6_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(6,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_6_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(6,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_6_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(6,i); Z80_WRMEM(j,i); }
void Z80::res_6_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(6,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_6_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(6,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_6_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(6,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_6_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(6,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_6_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(6,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_6_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(6,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_6_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(6,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_6_a(void) { M_RES(6,R.AF.B.h); }
void Z80::res_6_b(void) { M_RES(6,R.BC.B.h); }
void Z80::res_6_c(void) { M_RES(6,R.BC.B.l); }
void Z80::res_6_d(void) { M_RES(6,R.DE.B.h); }
void Z80::res_6_e(void) { M_RES(6,R.DE.B.l); }
void Z80::res_6_h(void) { M_RES(6,R.HL.B.h); }
void Z80::res_6_l(void) { M_RES(6,R.HL.B.l); }

void Z80::res_7_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RES(7,i); Z80_WRMEM(R.HL.D,i); }
void Z80::res_7_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RES(7,i); Z80_WRMEM(j,i); }
void Z80::res_7_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RES(7,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_7_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RES(7,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_7_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RES(7,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_7_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RES(7,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_7_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RES(7,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_7_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RES(7,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_7_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RES(7,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_7_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RES(7,i); Z80_WRMEM(j,i); }
void Z80::res_7_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RES(7,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::res_7_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RES(7,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::res_7_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RES(7,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::res_7_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RES(7,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::res_7_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RES(7,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::res_7_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RES(7,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::res_7_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RES(7,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::res_7_a(void) { M_RES(7,R.AF.B.h); }
void Z80::res_7_b(void) { M_RES(7,R.BC.B.h); }
void Z80::res_7_c(void) { M_RES(7,R.BC.B.l); }
void Z80::res_7_d(void) { M_RES(7,R.DE.B.h); }
void Z80::res_7_e(void) { M_RES(7,R.DE.B.l); }
void Z80::res_7_h(void) { M_RES(7,R.HL.B.h); }
void Z80::res_7_l(void) { M_RES(7,R.HL.B.l); }

void Z80::ret(void) { M_RET; }
void Z80::ret_c(void) { if (M_C) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_m(void) { if (M_M) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_nc(void) { if (M_NC) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_nz(void) { if (M_NZ) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_p(void) { if (M_P) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_pe(void) { if (M_PE) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_po(void) { if (M_PO) { M_RET; } else { M_SKIP_RET; } }
void Z80::ret_z(void) { if (M_Z) { M_RET; } else { M_SKIP_RET; } }


void Z80::reti(void) { M_RET; }
void Z80::retn(void) { R.IFF1=R.IFF2; M_RET; }

void Z80::rl_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RL(i); Z80_WRMEM(R.HL.D,i); }
void Z80::rl_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RL(i); Z80_WRMEM(j,i); }
void Z80::rl_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rl_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rl_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rl_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rl_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rl_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rl_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rl_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RL(i); Z80_WRMEM(j,i); }
void Z80::rl_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rl_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rl_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rl_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rl_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rl_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rl_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rl_a(void) { M_RL(R.AF.B.h); }
void Z80::rl_b(void) { M_RL(R.BC.B.h); }
void Z80::rl_c(void) { M_RL(R.BC.B.l); }
void Z80::rl_d(void) { M_RL(R.DE.B.h); }
void Z80::rl_e(void) { M_RL(R.DE.B.l); }
void Z80::rl_h(void) { M_RL(R.HL.B.h); }
void Z80::rl_l(void) { M_RL(R.HL.B.l); }

void Z80::rla(void)  { M_RLA; }

void Z80::rlc_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RLC(i); Z80_WRMEM(R.HL.D,i); }
void Z80::rlc_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RLC(i); Z80_WRMEM(j,i); }
void Z80::rlc_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RLC(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rlc_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RLC(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rlc_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RLC(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rlc_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RLC(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rlc_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RLC(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rlc_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RLC(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rlc_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RLC(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rlc_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RLC(i); Z80_WRMEM(j,i); }
void Z80::rlc_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RLC(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rlc_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RLC(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rlc_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RLC(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rlc_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RLC(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rlc_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RLC(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rlc_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RLC(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rlc_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RLC(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rlc_a(void) { M_RLC(R.AF.B.h); }
void Z80::rlc_b(void) { M_RLC(R.BC.B.h); }
void Z80::rlc_c(void) { M_RLC(R.BC.B.l); }
void Z80::rlc_d(void) { M_RLC(R.DE.B.h); }
void Z80::rlc_e(void) { M_RLC(R.DE.B.l); }
void Z80::rlc_h(void) { M_RLC(R.HL.B.h); }
void Z80::rlc_l(void) { M_RLC(R.HL.B.l); }

void Z80::rlca(void)  { M_RLCA; }

void Z80::rld(void)
{
 byte i;
 i=Z80_RDMEM(R.HL.D);
 Z80_WRMEM(R.HL.D,(i<<4)|(R.AF.B.h&0x0F));
 R.AF.B.h=(R.AF.B.h&0xF0)|(i>>4);
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSPTable[R.AF.B.h];
}

void Z80::rr_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RR(i); Z80_WRMEM(R.HL.D,i); }
void Z80::rr_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RR(i); Z80_WRMEM(j,i); }
void Z80::rr_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RR(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rr_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RR(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rr_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RR(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rr_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RR(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rr_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RR(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rr_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RR(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rr_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RR(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rr_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RR(i); Z80_WRMEM(j,i); }
void Z80::rr_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RR(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rr_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RR(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rr_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RR(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rr_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RR(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rr_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RR(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rr_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RR(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rr_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RR(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rr_a(void) { M_RR(R.AF.B.h); }
void Z80::rr_b(void) { M_RR(R.BC.B.h); }
void Z80::rr_c(void) { M_RR(R.BC.B.l); }
void Z80::rr_d(void) { M_RR(R.DE.B.h); }
void Z80::rr_e(void) { M_RR(R.DE.B.l); }
void Z80::rr_h(void) { M_RR(R.HL.B.h); }
void Z80::rr_l(void) { M_RR(R.HL.B.l); }

void Z80::rra(void)  { M_RRA; }

void Z80::rrc_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_RRC(i); Z80_WRMEM(R.HL.D,i); }
void Z80::rrc_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_RRC(i); Z80_WRMEM(j,i); }
void Z80::rrc_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_RRC(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rrc_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_RRC(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rrc_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_RRC(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rrc_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_RRC(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rrc_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_RRC(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rrc_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_RRC(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rrc_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_RRC(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rrc_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_RRC(i); Z80_WRMEM(j,i); }
void Z80::rrc_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_RRC(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::rrc_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_RRC(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::rrc_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_RRC(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::rrc_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_RRC(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::rrc_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_RRC(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::rrc_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_RRC(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::rrc_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_RRC(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::rrc_a(void) { M_RRC(R.AF.B.h); }
void Z80::rrc_b(void) { M_RRC(R.BC.B.h); }
void Z80::rrc_c(void) { M_RRC(R.BC.B.l); }
void Z80::rrc_d(void) { M_RRC(R.DE.B.h); }
void Z80::rrc_e(void) { M_RRC(R.DE.B.l); }
void Z80::rrc_h(void) { M_RRC(R.HL.B.h); }
void Z80::rrc_l(void) { M_RRC(R.HL.B.l); }

void Z80::rrca(void)  { M_RRCA; }

void Z80::rrd(void)
{
 byte i;
 i=Z80_RDMEM(R.HL.D);
 Z80_WRMEM(R.HL.D,(i>>4)|(R.AF.B.h<<4));
 R.AF.B.h=(R.AF.B.h&0xF0)|(i&0x0F);
 R.AF.B.l=(R.AF.B.l&C_FLAG)|ZSPTable[R.AF.B.h];
}

void Z80::rst_00(void) { M_RST(0x00); }
void Z80::rst_08(void) { M_RST(0x08); }
void Z80::rst_10(void) { M_RST(0x10); }
void Z80::rst_18(void) { M_RST(0x18); }
void Z80::rst_20(void) { M_RST(0x20); }
void Z80::rst_28(void) { M_RST(0x28); }
void Z80::rst_30(void) { M_RST(0x30); }
void Z80::rst_38(void) { M_RST(0x38); }

void Z80::sbc_a_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_SBC(i); }
void Z80::sbc_a_xhl(void) { byte i=M_RD_XHL; M_SBC(i); }
void Z80::sbc_a_xix(void) { byte i=M_RD_XIX(); M_SBC(i); }
void Z80::sbc_a_xiy(void) { byte i=M_RD_XIY(); M_SBC(i); }
void Z80::sbc_a_a(void) { M_SBC(R.AF.B.h); }
void Z80::sbc_a_b(void) { M_SBC(R.BC.B.h); }
void Z80::sbc_a_c(void) { M_SBC(R.BC.B.l); }
void Z80::sbc_a_d(void) { M_SBC(R.DE.B.h); }
void Z80::sbc_a_e(void) { M_SBC(R.DE.B.l); }
void Z80::sbc_a_h(void) { M_SBC(R.HL.B.h); }
void Z80::sbc_a_l(void) { M_SBC(R.HL.B.l); }
void Z80::sbc_a_ixh(void) { M_SBC(R.IX.B.h); }
void Z80::sbc_a_ixl(void) { M_SBC(R.IX.B.l); }
void Z80::sbc_a_iyh(void) { M_SBC(R.IY.B.h); }
void Z80::sbc_a_iyl(void) { M_SBC(R.IY.B.l); }

void Z80::sbc_hl_bc(void) { M_SBCW(BC); }
void Z80::sbc_hl_de(void) { M_SBCW(DE); }
void Z80::sbc_hl_hl(void) { M_SBCW(HL); }
void Z80::sbc_hl_sp(void) { M_SBCW(SP); }

void Z80::scf(void) { R.AF.B.l=(R.AF.B.l&0xEC)|C_FLAG; }

void Z80::set_0_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(0,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_0_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(0,i); Z80_WRMEM(j,i); }
void Z80::set_0_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(0,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_0_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(0,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_0_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(0,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_0_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(0,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_0_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(0,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_0_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(0,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_0_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(0,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_0_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(0,i); Z80_WRMEM(j,i); }
void Z80::set_0_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(0,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_0_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(0,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_0_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(0,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_0_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(0,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_0_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(0,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_0_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(0,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_0_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(0,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_0_a(void) { M_SET(0,R.AF.B.h); }
void Z80::set_0_b(void) { M_SET(0,R.BC.B.h); }
void Z80::set_0_c(void) { M_SET(0,R.BC.B.l); }
void Z80::set_0_d(void) { M_SET(0,R.DE.B.h); }
void Z80::set_0_e(void) { M_SET(0,R.DE.B.l); }
void Z80::set_0_h(void) { M_SET(0,R.HL.B.h); }
void Z80::set_0_l(void) { M_SET(0,R.HL.B.l); }

void Z80::set_1_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(1,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_1_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(1,i); Z80_WRMEM(j,i); }
void Z80::set_1_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(1,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_1_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(1,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_1_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(1,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_1_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(1,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_1_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(1,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_1_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(1,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_1_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(1,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_1_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(1,i); Z80_WRMEM(j,i); }
void Z80::set_1_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(1,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_1_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(1,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_1_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(1,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_1_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(1,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_1_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(1,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_1_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(1,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_1_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(1,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_1_a(void) { M_SET(1,R.AF.B.h); }
void Z80::set_1_b(void) { M_SET(1,R.BC.B.h); }
void Z80::set_1_c(void) { M_SET(1,R.BC.B.l); }
void Z80::set_1_d(void) { M_SET(1,R.DE.B.h); }
void Z80::set_1_e(void) { M_SET(1,R.DE.B.l); }
void Z80::set_1_h(void) { M_SET(1,R.HL.B.h); }
void Z80::set_1_l(void) { M_SET(1,R.HL.B.l); }

void Z80::set_2_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(2,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_2_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(2,i); Z80_WRMEM(j,i); }
void Z80::set_2_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(2,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_2_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(2,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_2_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(2,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_2_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(2,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_2_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(2,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_2_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(2,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_2_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(2,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_2_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(2,i); Z80_WRMEM(j,i); }
void Z80::set_2_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(2,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_2_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(2,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_2_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(2,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_2_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(2,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_2_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(2,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_2_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(2,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_2_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(2,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_2_a(void) { M_SET(2,R.AF.B.h); }
void Z80::set_2_b(void) { M_SET(2,R.BC.B.h); }
void Z80::set_2_c(void) { M_SET(2,R.BC.B.l); }
void Z80::set_2_d(void) { M_SET(2,R.DE.B.h); }
void Z80::set_2_e(void) { M_SET(2,R.DE.B.l); }
void Z80::set_2_h(void) { M_SET(2,R.HL.B.h); }
void Z80::set_2_l(void) { M_SET(2,R.HL.B.l); }

void Z80::set_3_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(3,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_3_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(3,i); Z80_WRMEM(j,i); }
void Z80::set_3_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(3,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_3_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(3,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_3_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(3,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_3_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(3,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_3_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(3,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_3_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(3,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_3_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(3,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_3_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(3,i); Z80_WRMEM(j,i); }
void Z80::set_3_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(3,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_3_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(3,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_3_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(3,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_3_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(3,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_3_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(3,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_3_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(3,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_3_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(3,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_3_a(void) { M_SET(3,R.AF.B.h); }
void Z80::set_3_b(void) { M_SET(3,R.BC.B.h); }
void Z80::set_3_c(void) { M_SET(3,R.BC.B.l); }
void Z80::set_3_d(void) { M_SET(3,R.DE.B.h); }
void Z80::set_3_e(void) { M_SET(3,R.DE.B.l); }
void Z80::set_3_h(void) { M_SET(3,R.HL.B.h); }
void Z80::set_3_l(void) { M_SET(3,R.HL.B.l); }

void Z80::set_4_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(4,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_4_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(4,i); Z80_WRMEM(j,i); }
void Z80::set_4_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(4,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_4_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(4,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_4_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(4,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_4_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(4,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_4_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(4,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_4_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(4,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_4_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(4,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_4_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(4,i); Z80_WRMEM(j,i); }
void Z80::set_4_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(4,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_4_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(4,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_4_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(4,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_4_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(4,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_4_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(4,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_4_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(4,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_4_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(4,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_4_a(void) { M_SET(4,R.AF.B.h); }
void Z80::set_4_b(void) { M_SET(4,R.BC.B.h); }
void Z80::set_4_c(void) { M_SET(4,R.BC.B.l); }
void Z80::set_4_d(void) { M_SET(4,R.DE.B.h); }
void Z80::set_4_e(void) { M_SET(4,R.DE.B.l); }
void Z80::set_4_h(void) { M_SET(4,R.HL.B.h); }
void Z80::set_4_l(void) { M_SET(4,R.HL.B.l); }

void Z80::set_5_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(5,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_5_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(5,i); Z80_WRMEM(j,i); }
void Z80::set_5_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(5,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_5_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(5,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_5_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(5,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_5_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(5,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_5_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(5,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_5_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(5,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_5_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(5,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_5_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(5,i); Z80_WRMEM(j,i); }
void Z80::set_5_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(5,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_5_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(5,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_5_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(5,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_5_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(5,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_5_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(5,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_5_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(5,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_5_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(5,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_5_a(void) { M_SET(5,R.AF.B.h); }
void Z80::set_5_b(void) { M_SET(5,R.BC.B.h); }
void Z80::set_5_c(void) { M_SET(5,R.BC.B.l); }
void Z80::set_5_d(void) { M_SET(5,R.DE.B.h); }
void Z80::set_5_e(void) { M_SET(5,R.DE.B.l); }
void Z80::set_5_h(void) { M_SET(5,R.HL.B.h); }
void Z80::set_5_l(void) { M_SET(5,R.HL.B.l); }

void Z80::set_6_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(6,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_6_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(6,i); Z80_WRMEM(j,i); }
void Z80::set_6_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(6,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_6_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(6,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_6_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(6,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_6_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(6,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_6_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(6,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_6_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(6,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_6_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(6,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_6_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(6,i); Z80_WRMEM(j,i); }
void Z80::set_6_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(6,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_6_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(6,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_6_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(6,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_6_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(6,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_6_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(6,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_6_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(6,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_6_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(6,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_6_a(void) { M_SET(6,R.AF.B.h); }
void Z80::set_6_b(void) { M_SET(6,R.BC.B.h); }
void Z80::set_6_c(void) { M_SET(6,R.BC.B.l); }
void Z80::set_6_d(void) { M_SET(6,R.DE.B.h); }
void Z80::set_6_e(void) { M_SET(6,R.DE.B.l); }
void Z80::set_6_h(void) { M_SET(6,R.HL.B.h); }
void Z80::set_6_l(void) { M_SET(6,R.HL.B.l); }

void Z80::set_7_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SET(7,i); Z80_WRMEM(R.HL.D,i); }
void Z80::set_7_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SET(7,i); Z80_WRMEM(j,i); }
void Z80::set_7_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SET(7,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_7_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SET(7,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_7_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SET(7,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_7_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SET(7,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_7_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SET(7,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_7_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SET(7,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_7_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SET(7,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_7_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SET(7,i); Z80_WRMEM(j,i); }
void Z80::set_7_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SET(7,R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::set_7_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SET(7,R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::set_7_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SET(7,R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::set_7_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SET(7,R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::set_7_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SET(7,R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::set_7_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SET(7,R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::set_7_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SET(7,R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::set_7_a(void) { M_SET(7,R.AF.B.h); }
void Z80::set_7_b(void) { M_SET(7,R.BC.B.h); }
void Z80::set_7_c(void) { M_SET(7,R.BC.B.l); }
void Z80::set_7_d(void) { M_SET(7,R.DE.B.h); }
void Z80::set_7_e(void) { M_SET(7,R.DE.B.l); }
void Z80::set_7_h(void) { M_SET(7,R.HL.B.h); }
void Z80::set_7_l(void) { M_SET(7,R.HL.B.l); }


void Z80::sla_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SLA(i); Z80_WRMEM(R.HL.D,i); }
void Z80::sla_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SLA(i); Z80_WRMEM(j,i); }
void Z80::sla_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SLA(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sla_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SLA(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sla_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SLA(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sla_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SLA(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sla_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SLA(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sla_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SLA(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sla_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SLA(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sla_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SLA(i); Z80_WRMEM(j,i); }
void Z80::sla_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SLA(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sla_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SLA(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sla_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SLA(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sla_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SLA(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sla_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SLA(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sla_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SLA(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sla_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SLA(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sla_a(void) { M_SLA(R.AF.B.h); }
void Z80::sla_b(void) { M_SLA(R.BC.B.h); }
void Z80::sla_c(void) { M_SLA(R.BC.B.l); }
void Z80::sla_d(void) { M_SLA(R.DE.B.h); }
void Z80::sla_e(void) { M_SLA(R.DE.B.l); }
void Z80::sla_h(void) { M_SLA(R.HL.B.h); }
void Z80::sla_l(void) { M_SLA(R.HL.B.l); }


void Z80::sll_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SLL(i); Z80_WRMEM(R.HL.D,i); }
void Z80::sll_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SLL(i); Z80_WRMEM(j,i); }
void Z80::sll_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SLL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sll_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SLL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sll_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SLL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sll_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SLL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sll_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SLL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sll_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SLL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sll_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SLL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sll_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SLL(i); Z80_WRMEM(j,i); }
void Z80::sll_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SLL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sll_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SLL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sll_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SLL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sll_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SLL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sll_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SLL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sll_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SLL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sll_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SLL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sll_a(void) { M_SLL(R.AF.B.h); }
void Z80::sll_b(void) { M_SLL(R.BC.B.h); }
void Z80::sll_c(void) { M_SLL(R.BC.B.l); }
void Z80::sll_d(void) { M_SLL(R.DE.B.h); }
void Z80::sll_e(void) { M_SLL(R.DE.B.l); }
void Z80::sll_h(void) { M_SLL(R.HL.B.h); }
void Z80::sll_l(void) { M_SLL(R.HL.B.l); }

void Z80::sra_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SRA(i); Z80_WRMEM(R.HL.D,i); }
void Z80::sra_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SRA(i); Z80_WRMEM(j,i); }
void Z80::sra_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SRA(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sra_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SRA(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sra_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SRA(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sra_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SRA(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sra_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SRA(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sra_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SRA(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sra_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SRA(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sra_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SRA(i); Z80_WRMEM(j,i); }
void Z80::sra_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SRA(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::sra_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SRA(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::sra_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SRA(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::sra_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SRA(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::sra_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SRA(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::sra_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SRA(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::sra_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SRA(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::sra_a(void) { M_SRA(R.AF.B.h); }
void Z80::sra_b(void) { M_SRA(R.BC.B.h); }
void Z80::sra_c(void) { M_SRA(R.BC.B.l); }
void Z80::sra_d(void) { M_SRA(R.DE.B.h); }
void Z80::sra_e(void) { M_SRA(R.DE.B.l); }
void Z80::sra_h(void) { M_SRA(R.HL.B.h); }
void Z80::sra_l(void) { M_SRA(R.HL.B.l); }

void Z80::srl_xhl(void) { byte i=Z80_RDMEM(R.HL.D); M_SRL(i); Z80_WRMEM(R.HL.D,i); }
void Z80::srl_xix(void) { int j=M_XIX; byte i=Z80_RDMEM(j); M_SRL(i); Z80_WRMEM(j,i); }
void Z80::srl_xix_a(void) { int j=M_XIX; R.AF.B.h=Z80_RDMEM(j); M_SRL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::srl_xix_b(void) { int j=M_XIX; R.BC.B.h=Z80_RDMEM(j); M_SRL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::srl_xix_c(void) { int j=M_XIX; R.BC.B.l=Z80_RDMEM(j); M_SRL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::srl_xix_d(void) { int j=M_XIX; R.DE.B.h=Z80_RDMEM(j); M_SRL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::srl_xix_e(void) { int j=M_XIX; R.DE.B.l=Z80_RDMEM(j); M_SRL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::srl_xix_h(void) { int j=M_XIX; R.HL.B.h=Z80_RDMEM(j); M_SRL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::srl_xix_l(void) { int j=M_XIX; R.HL.B.l=Z80_RDMEM(j); M_SRL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::srl_xiy(void) { int j=M_XIY; byte i=Z80_RDMEM(j); M_SRL(i); Z80_WRMEM(j,i); }
void Z80::srl_xiy_a(void) { int j=M_XIY; R.AF.B.h=Z80_RDMEM(j); M_SRL(R.AF.B.h); Z80_WRMEM(j,R.AF.B.h); }
void Z80::srl_xiy_b(void) { int j=M_XIY; R.BC.B.h=Z80_RDMEM(j); M_SRL(R.BC.B.h); Z80_WRMEM(j,R.BC.B.h); }
void Z80::srl_xiy_c(void) { int j=M_XIY; R.BC.B.l=Z80_RDMEM(j); M_SRL(R.BC.B.l); Z80_WRMEM(j,R.BC.B.l); }
void Z80::srl_xiy_d(void) { int j=M_XIY; R.DE.B.h=Z80_RDMEM(j); M_SRL(R.DE.B.h); Z80_WRMEM(j,R.DE.B.h); }
void Z80::srl_xiy_e(void) { int j=M_XIY; R.DE.B.l=Z80_RDMEM(j); M_SRL(R.DE.B.l); Z80_WRMEM(j,R.DE.B.l); }
void Z80::srl_xiy_h(void) { int j=M_XIY; R.HL.B.h=Z80_RDMEM(j); M_SRL(R.HL.B.h); Z80_WRMEM(j,R.HL.B.h); }
void Z80::srl_xiy_l(void) { int j=M_XIY; R.HL.B.l=Z80_RDMEM(j); M_SRL(R.HL.B.l); Z80_WRMEM(j,R.HL.B.l); }
void Z80::srl_a(void) { M_SRL(R.AF.B.h); }
void Z80::srl_b(void) { M_SRL(R.BC.B.h); }
void Z80::srl_c(void) { M_SRL(R.BC.B.l); }
void Z80::srl_d(void) { M_SRL(R.DE.B.h); }
void Z80::srl_e(void) { M_SRL(R.DE.B.l); }
void Z80::srl_h(void) { M_SRL(R.HL.B.h); }
void Z80::srl_l(void) { M_SRL(R.HL.B.l); }

void Z80::sub_xhl(void) { byte i=M_RD_XHL; M_SUB(i); }
void Z80::sub_xix(void) { byte i=M_RD_XIX(); M_SUB(i); }
void Z80::sub_xiy(void) { byte i=M_RD_XIY(); M_SUB(i); }
void Z80::sub_a(void) { R.AF.D=Z_FLAG|N_FLAG; }
void Z80::sub_b(void) { M_SUB(R.BC.B.h); }
void Z80::sub_c(void) { M_SUB(R.BC.B.l); }
void Z80::sub_d(void) { M_SUB(R.DE.B.h); }
void Z80::sub_e(void) { M_SUB(R.DE.B.l); }
void Z80::sub_h(void) { M_SUB(R.HL.B.h); }
void Z80::sub_l(void) { M_SUB(R.HL.B.l); }
void Z80::sub_ixh(void) { M_SUB(R.IX.B.h); }
void Z80::sub_ixl(void) { M_SUB(R.IX.B.l); }
void Z80::sub_iyh(void) { M_SUB(R.IY.B.h); }
void Z80::sub_iyl(void) { M_SUB(R.IY.B.l); }
void Z80::sub_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_SUB(i); }

void Z80::xor_xhl(void) { byte i=M_RD_XHL; M_XOR(i); }
void Z80::xor_xix(void) { byte i=M_RD_XIX(); M_XOR(i); }
void Z80::xor_xiy(void) { byte i=M_RD_XIY(); M_XOR(i); }
void Z80::xor_a(void) { R.AF.D=Z_FLAG|V_FLAG; }
void Z80::xor_b(void) { M_XOR(R.BC.B.h); }
void Z80::xor_c(void) { M_XOR(R.BC.B.l); }
void Z80::xor_d(void) { M_XOR(R.DE.B.h); }
void Z80::xor_e(void) { M_XOR(R.DE.B.l); }
void Z80::xor_h(void) { M_XOR(R.HL.B.h); }
void Z80::xor_l(void) { M_XOR(R.HL.B.l); }
void Z80::xor_ixh(void) { M_XOR(R.IX.B.h); }
void Z80::xor_ixl(void) { M_XOR(R.IX.B.l); }
void Z80::xor_iyh(void) { M_XOR(R.IY.B.h); }
void Z80::xor_iyl(void) { M_XOR(R.IY.B.l); }
void Z80::xor_byte(void) { byte i=Z80_RDMEM_OPCODE(); M_XOR(i); }

void Z80::no_op(void)
{
 --R.PC.W.l;
}

void Z80::patch(void) { 
  //NYI
}

static unsigned cycles_main[256]=
{
  4,10,7,6,4,4,7,4,
  4,11,7,6,4,4,7,4,
  8,10,7,6,4,4,7,4,
  7,11,7,6,4,4,7,4,
  7,10,16,6,4,4,7,4,
  7,11,16,6,4,4,7,4,
  7,10,13,6,11,11,10,4,
  7,11,13,6,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  7,7,7,7,7,7,4,7,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  4,4,4,4,4,4,7,4,
  5,10,10,10,10,11,7,11,
  5,4,10,0,10,10,7,11,
  5,10,10,11,10,11,7,11,
  5,4,10,11,10,0,7,11,
  5,10,10,19,10,11,7,11,
  5,4,10,4,10,0,7,11,
  5,10,10,4,10,11,7,11,
  5,6,10,4,10,0,7,11
};

static unsigned cycles_cb[256]=
{
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,12,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8,
  8,8,8,8,8,8,15,8
};
static unsigned cycles_xx_cb[]=
{
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  20,20,20,20,20,20,20,20,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23,
  23,23,23,23,23,23,23,23
};
static unsigned cycles_xx[256]=
{
  0,0,0,0,0,0,0,0,
  0,15,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,15,0,0,0,0,0,0,
  0,14,20,10,9,9,9,0,
  0,15,20,10,9,9,9,0,
  0,0,0,0,23,23,19,0,
  0,15,0,0,0,0,0,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  9,9,9,9,9,9,19,9,	/* ASG 220997 */
  9,9,9,9,9,9,19,9,	/* ASG 220997 */
  19,19,19,19,19,19,19,19,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,9,9,19,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,14,0,23,0,15,0,0,
  0,8,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,10,0,0,0,0,0,0
};
static unsigned cycles_ed[256]=
{
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  12,12,15,20,8,8,8,9,
  12,12,15,20,8,8,8,9,
  12,12,15,20,8,8,8,9,
  12,12,15,20,8,8,8,9,
  12,12,15,20,8,8,8,18,
  12,12,15,20,8,8,8,18,
  12,12,15,20,8,8,8,0,
  12,12,15,20,8,8,8,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  16,16,16,16,0,0,0,0,
  16,16,16,16,0,0,0,0,
  16,16,16,16,0,0,0,0,  /* ASG 220997 */
  16,16,16,16,0,0,0,0,	/* ASG 220997 */
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0
};

void Z80::dd_cb(void)
{
 byte opcode;
 opcode=Z80_RDMEM((R.PC.D+1)&0xFFFF);
 Z80_ICount-=cycles_xx_cb[opcode];
 //(*(opcode_dd_cb[opcode]))();
 call_opcode_dd_cb (opcode);
 ++R.PC.W.l;
}
void Z80::fd_cb(void)
{
 byte opcode;
 opcode=Z80_RDMEM((R.PC.D+1)&0xFFFF);
 Z80_ICount-=cycles_xx_cb[opcode];
 //(*(opcode_fd_cb[opcode]))();
 call_opcode_fd_cb (opcode);
 ++R.PC.W.l;
}

void Z80::cb(void)
{
 byte opcode;
 ++R.R;
 opcode=Z80_RDMEM(R.PC.D);
 R.PC.W.l++;
 Z80_ICount-=cycles_cb[opcode];
 //(*(opcode_cb[opcode]))();
 call_opcode_cb (opcode);
}
void Z80::dd(void)
{
 byte opcode;
 ++R.R;
 opcode=Z80_RDMEM(R.PC.D);
 R.PC.W.l++;
 Z80_ICount-=cycles_xx[opcode];
 //(*(opcode_dd[opcode]))();
 call_opcode_dd (opcode);
}
void Z80::ed(void)
{
 byte opcode;
 ++R.R;
 opcode=Z80_RDMEM(R.PC.D);
 R.PC.W.l++;
 Z80_ICount-=cycles_ed[opcode];
 //(*(opcode_ed[opcode]))();
 call_opcode_ed (opcode);
}
void Z80::fd (void)
{
 byte opcode;
 ++R.R;
 opcode=Z80_RDMEM(R.PC.D);
 R.PC.W.l++;
 Z80_ICount-=cycles_xx[opcode];
 //(*(opcode_fd[opcode]))();
 call_opcode_fd (opcode);
}

void Z80::ei(void) {
  byte opcode;
  /* If interrupts were disabled, execute one more instruction and check the */
  /* IRQ line. If not, simply set interrupt flip-flop 2                      */
  if (!R.IFF1)
    {
      R.IFF1=R.IFF2=1;
      ++R.R;
      opcode=Z80_RDMEM(R.PC.D);
  
      R.PC.W.l++;
      Z80_ICount-=cycles_main[opcode];
      //(*(opcode_main[opcode]))();
      call_opcode_main (opcode);
      
      Interrupt(/*Z80_IRQ*/);	/* NS 970904 */
    }
  else
    R.IFF2=1;
}

/****************************************************************************/
/* Reset registers to their initial values                                  */
/****************************************************************************/
void Z80::Reset (void) {
  Z80_Regs aRegs;
  Z80_SetRegs (aRegs);
}

/****************************************************************************/
/* Initialise the various lookup tables used by the emulation code          */
/****************************************************************************/
void Z80::InitTables (void)
{
 static int InitTables_virgin=1;
 byte zs;
 int i,p;
 if (!InitTables_virgin) return;
 InitTables_virgin=0;
 for (i=0;i<256;++i)
 {
  zs=0;
  if (i==0)
   zs|=Z_FLAG;
  if (i&0x80)
   zs|=S_FLAG;
  p=0;
  if (i&1) ++p;
  if (i&2) ++p;
  if (i&4) ++p;
  if (i&8) ++p;
  if (i&16) ++p;
  if (i&32) ++p;
  if (i&64) ++p;
  if (i&128) ++p;
  PTable[i]=(p&1)? 0:V_FLAG;
  ZSTable[i]=zs;
  ZSPTable[i]=zs|PTable[i];
 }
 for (i=0;i<256;++i)
 {
  ZSTable[i+256]=ZSTable[i]|C_FLAG;
  ZSPTable[i+256]=ZSPTable[i]|C_FLAG;
  PTable[i+256]=PTable[i]|C_FLAG;
 }
}

/****************************************************************************/
/* Issue an interrupt if necessary                                          */
/****************************************************************************/
void Z80::Interrupt (void/*int j*/) {
  //printf ("Z80::Interrupt\n");
  if (R.pending_irq == Z80_IGNORE_INT && R.pending_nmi == 0) return;	/* NS 970904 */
  if (R.pending_nmi != 0 || R.IFF1) {
    
    /* Clear interrupt flip-flop 1 */
    R.IFF1=0;
    /* Check if processor was halted */
    if (R.HALT) {
      ++R.PC.W.l;
      R.HALT=0;
    }
    
    if (R.pending_nmi != 0) {
      R.pending_nmi = 0;
      M_PUSH (PC);
      R.PC.D=0x0066;
    }
    else {
      int j;
      j = R.pending_irq;	/* NS 970904 */
      R.pending_irq = Z80_IGNORE_INT;	/* NS 970904 */
      
      /* Interrupt mode 2. Call [R.I:databyte] */
      if (R.IM==2) {
	//printf ("Interrupt : MODE 2\n");
	M_PUSH (PC);
	R.PC.D=Z80_RDMEM_WORD((j&255)|(R.I<<8));
      }
      else /* Interrupt mode 1. RST 38h */
	if (R.IM==1) {
	  //printf ("Interrupt : MODE 1\n");
	  Z80_ICount-=cycles_main[0xFF];
	  //(*(opcode_main[0xFF]))();
	  call_opcode_main (0xFF);
	}
	else {
	  /* Interrupt mode 0. We check for CALL and JP instructions, if neither  */
	  /* of these were found we assume a 1 byte opcode was placed on the      */
	  /* databus                                                              */
	  printf ("Interrupt : MODE 0\n");
	  switch (j&0xFF0000) {
	  case 0xCD0000:	/* bugfix NS 970904 */
	    M_PUSH(PC);
	  case 0xC30000:	/* bugfix NS 970904 */
	    R.PC.D=j&0xFFFF;
	    break;
	  default:
	    j&=255;
	    Z80_ICount-=cycles_main[j];
	    //(*(opcode_main[j]))();
	    call_opcode_main (j);
	    break;
	  }
	}
    }
  }
}


/*******************************/
/* Execute IPeriod T-States.   */
/*******************************/
int Z80::Execute(int cycles) {
 byte opcode;
 Z80_ICount=cycles;	/* NS 970904 */
 
 do {
   if (R.pending_nmi != 0 || R.pending_irq != Z80_IGNORE_INT) Interrupt();	/* NS 970901 */
   ++R.R;
   opcode=Z80_RDMEM(R.PC.D);
   
   R.PC.W.l++;
   Z80_ICount-=cycles_main[opcode];
   //(*(opcode_main[opcode]))();
   call_opcode_main (opcode);
 }
 while (Z80_ICount>0);
 
 return cycles - Z80_ICount;	/* NS 970904 */
}

int Z80::Step(void) {
 byte opcode;
 Z80_ICount=0;	/* NS 970904 */
 
 if (R.pending_nmi != 0 || R.pending_irq != Z80_IGNORE_INT) Interrupt();	/* NS 970901 */
 ++R.R;
 opcode=Z80_RDMEM(R.PC.D);
 
 R.PC.W.l++;
 Z80_ICount = cycles_main[opcode];

 call_opcode_main (opcode);
 
 
 return Z80_ICount;	/* NS 970904 */
}

void Z80::Z80_Cause_Interrupt(int type) {
  if (type == Z80_NMI_INT) {
    printf ("Z80::Z80_Cause_Interrupt : Z80_NMI_INT\n");
    R.pending_nmi = 1;
  }
  else if (type != Z80_IGNORE_INT) {
    //printf ("Z80::Z80_Cause_Interrupt : Z80_INT\n");
    R.pending_irq = type;
  }
}

// End of file
